/* jshint forin:true, noarg:true, noempty:true, eqeqeq:true, boss:true, undef:true, curly:true, browser:true, jquery:true */
/*
 * jQuery MultiSelect UI Widget 1.12
 * Copyright (c) 2011 Eric Hynds
 *
 * http://www.erichynds.com/jquery/jquery-ui-multiselect-widget/
 *
 * Depends:
 *   - jQuery 1.4.2+
 *   - jQuery UI 1.8 widget factory
 *
 * Optional:
 *   - jQuery UI effects
 *   - jQuery UI position utility
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 *
*/

(function($, undefined){

var multiselectID = 0;

$.widget("ech.multiselect", {
	
	// default options
	options: {
		header: true,
		height: 175,
		minWidth: 225,
		classes: '',
		checkAllText: 'Check all',
		uncheckAllText: 'Uncheck all',
		noneSelectedText: 'Select options',
		selectedText: '# selected',
		selectedList: 0,
		show: '',
		hide: '',
		autoOpen: false,
		multiple: true,
		position: {}
	},

	_create: function(){
		var el = this.element.hide(),
			o = this.options;
		
		this.speed = $.fx.speeds._default; // default speed for effects
		this._isOpen = false; // assume no
	
		var 
			button = (this.button = $('<button type="button"><span class="ui-icon ui-icon-triangle-2-n-s"></span></button>'))
				.addClass('ui-multiselect ui-widget ui-state-default ui-corner-all')
				.addClass( o.classes )
				.attr({ 'title':el.attr('title'), 'aria-haspopup':true, 'tabIndex':el.attr('tabIndex') })
				.insertAfter( el ),
			
			buttonlabel = (this.buttonlabel = $('<span />'))
				.html( o.noneSelectedText )
				.appendTo( button ),
				
			menu = (this.menu = $('<div />'))
				.addClass('ui-multiselect-menu ui-widget ui-widget-content ui-corner-all')
				.addClass( o.classes )
				.appendTo( document.body ),
				
			header = (this.header = $('<div />'))
				.addClass('ui-widget-header ui-corner-all ui-multiselect-header ui-helper-clearfix')
				.appendTo( menu ),
				
			headerLinkContainer = (this.headerLinkContainer = $('<ul />'))
				.addClass('ui-helper-reset')
				.html(function(){
					if( o.header === true ){
						return '<li><a class="ui-multiselect-all" href="#"><span class="ui-icon ui-icon-check"></span><span>' + o.checkAllText + '</span></a></li><li><a class="ui-multiselect-none" href="#"><span class="ui-icon ui-icon-closethick"></span><span>' + o.uncheckAllText + '</span></a></li>';
					} else if(typeof o.header === "string"){
						return '<li>' + o.header + '</li>';
					} else {
						return '';
					}
				})
				.append('<li class="ui-multiselect-close"><a href="#" class="ui-multiselect-close"><span class="ui-icon ui-icon-circle-close"></span></a></li>')
				.appendTo( header ),
			
			checkboxContainer = (this.checkboxContainer = $('<ul />'))
				.addClass('ui-multiselect-checkboxes ui-helper-reset')
				.appendTo( menu );
		
		// perform event bindings
		this._bindEvents();
		
		// build menu
		this.refresh( true );
		
		// some addl. logic for single selects
		if( !o.multiple ){
			menu.addClass('ui-multiselect-single');
		}
	},
	
	_init: function(){
		if( this.options.header === false ){
			this.header.hide();
		}
		if( !this.options.multiple ){
			this.headerLinkContainer.find('.ui-multiselect-all, .ui-multiselect-none').hide();
		}
		if( this.options.autoOpen ){
			this.open();
		}
		if( this.element.is(':disabled') ){
			this.disable();
		}
	},
	
	refresh: function( init ){
		var el = this.element,
			o = this.options,
			menu = this.menu,
			checkboxContainer = this.checkboxContainer,
			optgroups = [],
			html = [],
			id = el.attr('id') || multiselectID++; // unique ID for the label & option tags
		
		// build items
		el.find('option').each(function( i ){
			var $this = $(this), 
				parent = this.parentNode,
				title = this.innerHTML,
				description = this.title,
				value = this.value,
				inputID = this.id || 'ui-multiselect-' + id + '-option-' + i,
				isDisabled = this.disabled,
				isSelected = this.selected,
				labelClasses = ['ui-corner-all'],
				optLabel;
			
			// is this an optgroup?
			if( parent.tagName.toLowerCase() === 'optgroup' ){
				optLabel = parent.getAttribute('label');
				
				// has this optgroup been added already?
				if( $.inArray(optLabel, optgroups) === -1 ){
					html.push('<li class="ui-multiselect-optgroup-label"><a href="#">' + optLabel + '</a></li>');
					optgroups.push( optLabel );
				}
			}
		
			if( isDisabled ){
				labelClasses.push('ui-state-disabled');
			}

			// browsers automatically select the first option
			// by default with single selects
			if( isSelected && !o.multiple ){
				labelClasses.push('ui-state-active');
			}
			
			html.push('<li class="' + (isDisabled ? 'ui-multiselect-disabled' : '') + '">');
			
			// create the label
			html.push('<label for="' + inputID + '" title="' + description + '" class="' + labelClasses.join(' ') + '">');
			html.push('<input id="' + inputID + '" name="multiselect_' + id + '" type="' + (o.multiple ? "checkbox" : "radio") + '" value="' + value + '" title="' + title + '"');

			// pre-selected?
			if( isSelected ){
				html.push(' checked="checked"');
				html.push(' aria-selected="true"');
			}

			// disabled?
			if( isDisabled ){
				html.push(' disabled="disabled"');
				html.push(' aria-disabled="true"');
			}

			// add the title and close everything off
			html.push(' /><span>' + title + '</span></label></li>');
		});
		
		// insert into the DOM
		checkboxContainer.html( html.join('') );

		// cache some moar useful elements
		this.labels = menu.find('label');
		
		// set widths
		this._setButtonWidth();
		this._setMenuWidth();
		
		// remember default value
		this.button[0].defaultValue = this.update();
		
		// broadcast refresh event; useful for widgets
		if( !init ){
			this._trigger('refresh');
		}
	},
	
	// updates the button text. call refresh() to rebuild
	update: function(){
		var o = this.options,
			$inputs = this.labels.find('input'),
			$checked = $inputs.filter('[checked]'),
			numChecked = $checked.length,
			value;
		
		if( numChecked === 0 ){
			value = o.noneSelectedText;
		} else {
			if($.isFunction( o.selectedText )){
				value = o.selectedText.call(this, numChecked, $inputs.length, $checked.get());
			} else if( /\d/.test(o.selectedList) && o.selectedList > 0 && numChecked <= o.selectedList){
				value = $checked.map(function(){ return $(this).next().text(); }).get().join(', ');
			} else {
				value = o.selectedText.replace('#', numChecked).replace('#', $inputs.length);
			}
		}
		
		this.buttonlabel.html( value );
		return value;
	},
	
	// binds events
	_bindEvents: function(){
		var self = this, button = this.button;
		
		function clickHandler(){
			self[ self._isOpen ? 'close' : 'open' ]();
			return false;
		}
		
		// webkit doesn't like it when you click on the span :(
		button
			.find('span')
			.bind('click.multiselect', clickHandler);
		
		// button events
		button.bind({
			click: clickHandler,
			keypress: function( e ){
				switch(e.which){
					case 27: // esc
					case 38: // up
					case 37: // left
						self.close();
						break;
					case 39: // right
					case 40: // down
						self.open();
						break;
				}
			},
			mouseenter: function(){
				if( !button.hasClass('ui-state-disabled') ){
					$(this).addClass('ui-state-hover');
				}
			},
			mouseleave: function(){
				$(this).removeClass('ui-state-hover');
			},
			focus: function(){
				if( !button.hasClass('ui-state-disabled') ){
					$(this).addClass('ui-state-focus');
				}
			},
			blur: function(){
				$(this).removeClass('ui-state-focus');
			}
		});

		// header links
		this.header
			.delegate('a', 'click.multiselect', function( e ){
				// close link
				if( $(this).hasClass('ui-multiselect-close') ){
					self.close();
			
				// check all / uncheck all
				} else {
					self[ $(this).hasClass('ui-multiselect-all') ? 'checkAll' : 'uncheckAll' ]();
				}
			
				e.preventDefault();
			});
		
		// optgroup label toggle support
		this.menu
			.delegate('li.ui-multiselect-optgroup-label a', 'click.multiselect', function( e ){
				e.preventDefault();
				
				var $this = $(this),
					$inputs = $this.parent().nextUntil('li.ui-multiselect-optgroup-label').find('input:visible:not(:disabled)'),
				    nodes = $inputs.get(),
				    label = $this.parent().text();
				
				// trigger event and bail if the return is false
				if( self._trigger('beforeoptgrouptoggle', e, { inputs:nodes, label:label }) === false ){
					return;
				}
				
				// toggle inputs
				self._toggleChecked(
					$inputs.filter('[checked]').length !== $inputs.length,
					$inputs
				);

				self._trigger('optgrouptoggle', e, {
				    inputs: nodes,
				    label: label,
				    checked: nodes[0].checked
				});
			})
			.delegate('label', 'mouseenter.multiselect', function(){
				if( !$(this).hasClass('ui-state-disabled') ){
					self.labels.removeClass('ui-state-hover');
					$(this).addClass('ui-state-hover').find('input').focus();
				}
			})
			.delegate('label', 'keydown.multiselect', function( e ){
				e.preventDefault();
				
				switch(e.which){
					case 9: // tab
					case 27: // esc
						self.close();
						break;
					case 38: // up
					case 40: // down
					case 37: // left
					case 39: // right
						self._traverse(e.which, this);
						break;
					case 13: // enter
						$(this).find('input')[0].click();
						break;
				}
			})
			.delegate('input[type="checkbox"], input[type="radio"]', 'click.multiselect', function( e ){
				var $this = $(this),
					val = this.value,
					checked = this.checked,
					tags = self.element.find('option');
				
				// bail if this input is disabled or the event is cancelled
				if( this.disabled || self._trigger('click', e, { value: val, text: this.title, checked: checked }) === false ){
					e.preventDefault();
					return;
				}

				// make sure the input has focus. otherwise, the esc key
				// won't close the menu after clicking an item.
				$this.focus();
				
				// toggle aria state
				$this.attr('aria-selected', checked);
				
				// change state on the original option tags
				tags.each(function(){
					if( this.value === val ){
						this.selected = checked;
					} else if( !self.options.multiple ){
						this.selected = false;
					}
				});
				
				// some additional single select-specific logic
				if( !self.options.multiple ){
					self.labels.removeClass('ui-state-active');
					$this.closest('label').toggleClass('ui-state-active', checked );
					
					// close menu
					self.close();
				}

				// fire change on the select box
				self.element.trigger("change");
				
				// setTimeout is to fix multiselect issue #14 and #47. caused by jQuery issue #3827
				// http://bugs.jquery.com/ticket/3827 
				setTimeout($.proxy(self.update, self), 10);
			});

		// close each widget when clicking on any other element/anywhere else on the page
		$(document).bind('mousedown.multiselect', function( e ){
			if(self._isOpen && !$.contains(self.menu[0], e.target) && !$.contains(self.button[0], e.target) && e.target !== self.button[0]){
				self.close();
			}
		});

		// deal with form resets.  the problem here is that buttons aren't
		// restored to their defaultValue prop on form reset, and the reset
		// handler fires before the form is actually reset.  delaying it a bit
		// gives the form inputs time to clear.
		$(this.element[0].form).bind('reset.multiselect', function(){
			setTimeout($.proxy(self.refresh, self), 10);
		});
	},

	// set button width
	_setButtonWidth: function(){
		var width = this.element.outerWidth(),
			o = this.options;
			
		if( /\d/.test(o.minWidth) && width < o.minWidth){
			width = o.minWidth;
		}
		
		// set widths
		this.button.width( width );
	},
	
	// set menu width
	_setMenuWidth: function(){
		var m = this.menu,
			width = this.button.outerWidth()-
				parseInt(m.css('padding-left'),10)-
				parseInt(m.css('padding-right'),10)-
				parseInt(m.css('border-right-width'),10)-
				parseInt(m.css('border-left-width'),10);
				
		m.width( width || this.button.outerWidth() );
	},
	
	// move up or down within the menu
	_traverse: function( which, start ){
		var $start = $(start),
			moveToLast = which === 38 || which === 37,
			
			// select the first li that isn't an optgroup label / disabled
			$next = $start.parent()[moveToLast ? 'prevAll' : 'nextAll']('li:not(.ui-multiselect-disabled, .ui-multiselect-optgroup-label)')[ moveToLast ? 'last' : 'first']();
		
		// if at the first/last element
		if( !$next.length ){
			var $container = this.menu.find('ul').last();
			
			// move to the first/last
			this.menu.find('label')[ moveToLast ? 'last' : 'first' ]().trigger('mouseover');
			
			// set scroll position
			$container.scrollTop( moveToLast ? $container.height() : 0 );
			
		} else {
			$next.find('label').trigger('mouseover');
		}
	},

	// This is an internal function to toggle the checked property and
	// other related attributes of a checkbox.
	//
	// The context of this function should be a checkbox; do not proxy it.
	_toggleState: function( prop, flag ){
		return function(){
			if( !this.disabled ) {
				this[ prop ] = flag;
			}

			if( flag ){
				this.setAttribute('aria-selected', true);
			} else {
				this.removeAttribute('aria-selected');
			}
		};
	},

	_toggleChecked: function( flag, group ){
		var $inputs = (group && group.length) ?
			group :
			this.labels.find('input'),

			self = this;

		// toggle state on inputs
		$inputs.each(this._toggleState('checked', flag));

		// give the first input focus
		$inputs.eq(0).focus();
		
		// update button text
		this.update();
		
		// gather an array of the values that actually changed
		var values = $inputs.map(function(){
			return this.value;
		}).get();

		// toggle state on original option tags
		this.element
			.find('option')
			.each(function(){
				if( !this.disabled && $.inArray(this.value, values) > -1 ){
					self._toggleState('selected', flag).call( this );
				}
			});

		// trigger the change event on the select
		if( $inputs.length ) {
			this.element.trigger("change");
		}
	},

	_toggleDisabled: function( flag ){
		this.button
			.attr({ 'disabled':flag, 'aria-disabled':flag })[ flag ? 'addClass' : 'removeClass' ]('ui-state-disabled');
		
		this.menu
			.find('input')
			.attr({ 'disabled':flag, 'aria-disabled':flag })
			.parent()[ flag ? 'addClass' : 'removeClass' ]('ui-state-disabled');
		
		this.element
			.attr({ 'disabled':flag, 'aria-disabled':flag });
	},
	
	// open the menu
	open: function( e ){
		var self = this,
			button = this.button,
			menu = this.menu,
			speed = this.speed,
			o = this.options;
		
		// bail if the multiselectopen event returns false, this widget is disabled, or is already open 
		if( this._trigger('beforeopen') === false || button.hasClass('ui-state-disabled') || this._isOpen ){
			return;
		}
		
		var $container = menu.find('ul').last(),
			effect = o.show,
			pos = button.offset();
		
		// figure out opening effects/speeds
		if( $.isArray(o.show) ){
			effect = o.show[0];
			speed = o.show[1] || self.speed;
		}
		
		// set the scroll of the checkbox container
		$container.scrollTop(0).height(o.height);
		
		// position and show menu
		if( $.ui.position && !$.isEmptyObject(o.position) ){
			o.position.of = o.position.of || button;
			
			menu
				.show()
				.position( o.position )
				.hide()
				.show( effect, speed );
		
		// if position utility is not available...
		} else {
			menu.css({ 
				top: pos.top + button.outerHeight(),
				left: pos.left
			}).show( effect, speed );
		}
		
		// select the first option
		// triggering both mouseover and mouseover because 1.4.2+ has a bug where triggering mouseover
		// will actually trigger mouseenter.  the mouseenter trigger is there for when it's eventually fixed
		this.labels.eq(0).trigger('mouseover').trigger('mouseenter').find('input').trigger('focus');
		
		button.addClass('ui-state-active');
		this._isOpen = true;
		this._trigger('open');
	},
	
	// close the menu
	close: function(){
		if(this._trigger('beforeclose') === false){
			return;
		}
	
		var o = this.options, effect = o.hide, speed = this.speed;
		
		// figure out opening effects/speeds
		if( $.isArray(o.hide) ){
			effect = o.hide[0];
			speed = o.hide[1] || this.speed;
		}
	
		this.menu.hide(effect, speed);
		this.button.removeClass('ui-state-active').trigger('blur').trigger('mouseleave');
		this._isOpen = false;
		this._trigger('close');
	},

	enable: function(){
		this._toggleDisabled(false);
	},
	
	disable: function(){
		this._toggleDisabled(true);
	},
	
	checkAll: function( e ){
		this._toggleChecked(true);
		this._trigger('checkAll');
	},
	
	uncheckAll: function(){
		this._toggleChecked(false);
		this._trigger('uncheckAll');
	},
	
	getChecked: function(){
		return this.menu.find('input').filter('[checked]');
	},
	
	destroy: function(){
		// remove classes + data
		$.Widget.prototype.destroy.call( this );
		
		this.button.remove();
		this.menu.remove();
		this.element.show();
		
		return this;
	},
	
	isOpen: function(){
		return this._isOpen;
	},
	
	widget: function(){
		return this.menu;
	},
	
	// react to option changes after initialization
	_setOption: function( key, value ){
		var menu = this.menu;
		
		switch(key){
			case 'header':
				menu.find('div.ui-multiselect-header')[ value ? 'show' : 'hide' ]();
				break;
			case 'checkAllText':
				menu.find('a.ui-multiselect-all span').eq(-1).text(value);
				break;
			case 'uncheckAllText':
				menu.find('a.ui-multiselect-none span').eq(-1).text(value);
				break;
			case 'height':
				menu.find('ul').last().height( parseInt(value,10) );
				break;
			case 'minWidth':
				this.options[ key ] = parseInt(value,10);
				this._setButtonWidth();
				this._setMenuWidth();
				break;
			case 'selectedText':
			case 'selectedList':
			case 'noneSelectedText':
				this.options[key] = value; // these all needs to update immediately for the update() call
				this.update();
				break;
			case 'classes':
				menu.add(this.button).removeClass(this.options.classes).addClass(value);
				break;
		}
		
		$.Widget.prototype._setOption.apply( this, arguments );
	}
});

})(jQuery);
/* jshint forin:true, noarg:true, noempty:true, eqeqeq:true, boss:true, undef:true, curly:true, browser:true, jquery:true */
/*
 * jQuery MultiSelect UI Widget Filtering Plugin 1.4
 * Copyright (c) 2011 Eric Hynds
 *
 * http://www.erichynds.com/jquery/jquery-ui-multiselect-widget/
 *
 * Depends:
 *   - jQuery UI MultiSelect widget
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 *
*/

(function($){
	var rEscape = /[\-\[\]{}()*+?.,\\\^$|#\s]/g;
	
	$.widget("ech.multiselectfilter", {
		
		options: {
			label: "Filter:",
			width: null, /* override default width set in css file (px). null will inherit */
			placeholder: "Enter keywords",
			autoReset: false
		},
		
		_create: function(){
			var self = this,
				opts = this.options,
				instance = (this.instance = $(this.element).data("multiselect")),
				
				// store header; add filter class so the close/check all/uncheck all links can be positioned correctly
				header = (this.header = instance.menu.find(".ui-multiselect-header").addClass("ui-multiselect-hasfilter")),
				
				// wrapper elem
				wrapper = (this.wrapper = $('<div class="ui-multiselect-filter">'+(opts.label.length ? opts.label : '')+'<input placeholder="'+opts.placeholder+'" type="search"' + (/\d/.test(opts.width) ? 'style="width:'+opts.width+'px"' : '') + ' /></div>').prependTo( this.header ));

			// reference to the actual inputs
			this.inputs = instance.menu.find('input[type="checkbox"], input[type="radio"]');
			
			// build the input box
			this.input = wrapper
			.find("input")
			.bind({
				keydown: function( e ){
					// prevent the enter key from submitting the form / closing the widget
					if( e.which === 13 ){
						e.preventDefault();
					}
				},
				keyup: $.proxy(self._handler, self),
				click: $.proxy(self._handler, self)
			});
			
			// cache input values for searching
			this.updateCache();
			
			// rewrite internal _toggleChecked fn so that when checkAll/uncheckAll is fired,
			// only the currently filtered elements are checked
			instance._toggleChecked = function(flag, group){
				var $inputs = (group && group.length) ?
						group :
						this.labels.find('input'),
					
					_self = this,

					// do not include hidden elems if the menu isn't open.
					selector = self.instance._isOpen ?
						":disabled, :hidden" :
						":disabled";

				$inputs = $inputs.not( selector ).each(this._toggleState('checked', flag));
				
				// update text
				this.update();
				
				// figure out which option tags need to be selected
				var values = $inputs.map(function(){
					return this.value;
				}).get();
				
				// select option tags
				this.element
					.find('option')
					.filter(function(){
						if( !this.disabled && $.inArray(this.value, values) > -1 ){
							_self._toggleState('selected', flag).call( this );
						}
					});
			};
			
			// rebuild cache when multiselect is updated
			var doc = $(document).bind("multiselectrefresh", function(){
				self.updateCache();
				self._handler();
			});

			// automatically reset the widget on close?
			if(this.options.autoReset) {
				doc.bind("multiselectclose", $.proxy(this._reset, this));
			}
		},
		
		// thx for the logic here ben alman
		_handler: function( e ){
			var term = $.trim( this.input[0].value.toLowerCase() ),
			
				// speed up lookups
				rows = this.rows, inputs = this.inputs, cache = this.cache;
			
			if( !term ){
				rows.show();

                this._trigger( "filter", e, null);

			} else {
				rows.hide();
				
				var regex = new RegExp(term.replace(rEscape, "\\$&"), 'gi');
				
				this._trigger( "filter", e, $.map(cache, function(v, i){
					if( v.search(regex) !== -1 ){
						rows.eq(i).show();
						return inputs.get(i);
					}
					
					return null;
				}));
			}

			// show/hide optgroups
			this.instance.menu.find(".ui-multiselect-optgroup-label").each(function(){
				var $this = $(this);
				var isVisible = $this.nextUntil('.ui-multiselect-optgroup-label').filter(function(){
				  return $.css(this, "display") !== 'none';
				}).length;
				
				$this[ isVisible ? 'show' : 'hide' ]();
			});
		},

		_reset: function() {
			this.input.val('').trigger('keyup');
		},
		
		updateCache: function(){
			// each list item
			this.rows = this.instance.menu.find(".ui-multiselect-checkboxes li:not(.ui-multiselect-optgroup-label)");
			
			// cache
			this.cache = this.element.children().map(function(){
				var self = $(this);
				
				// account for optgroups
				if( this.tagName.toLowerCase() === "optgroup" ){
					self = self.children();
				}
				
				return self.map(function(){
					return this.innerHTML.toLowerCase();
				}).get();
			}).get();
		},
		
		widget: function(){
			return this.wrapper;
		},
		
		destroy: function(){
			$.Widget.prototype.destroy.call( this );
			this.input.val('').trigger("keyup");
			this.wrapper.remove();
		}
	});
})(jQuery);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.panel.list.registerPage('activation_keys', { create : 'new_activation_key' });
KT.panel.set_expand_cb(function() {
    var children = $('#panel .menu_parent');
    $.each(children, function(i, item) {
        KT.menu.hoverMenu(item, { top : '75px' });
    });
    KT.system_groups_pane.register_multiselect();
});

$(document).ready(function() {

    KT.panel.set_expand_cb(function() {
        KT.activation_key.initialize_new();
        KT.activation_key.initialize_edit();

        if( $('#subscription_form').length > 0 ){
            KT.activation_key.subscription_setup();
        }
    });

    $('#save_key').live('submit', function(e) {
        e.preventDefault();
        KT.activation_key.save_key($(this));
    });

    $('#cancel_key').live('click', function(e) {
        e.preventDefault();
        KT.activation_key.cancel_key($(this));
    });

    //Set the callback on the environment selector
    env_select.click_callback = function(env_id) {
        KT.activation_key.save_selected_environment(env_id);
        KT.activation_key.get_content_views();
        KT.activation_key.get_products();
    };

    $('#activation_key_content_view_id').live('change', function() {
        KT.activation_key.remove_content_view_highlight(false);
        KT.activation_key.get_products();
    });

    $('.clickable.product_family').live('click', function() {
        KT.activation_key.toggle_family($(this));
    });

    // the parent (product family) was checked, so check all children
    $('.family_checkbox').live('click', function() {
        KT.activation_key.toggle_family_checkboxes($(this), this.checked);
    });

    // a child was checked, so update the parent (product family), if needed
    $('#subscription_form input[type="checkbox"]').live('click', function() {
        KT.activation_key.toggle_parent_checkbox($(this));
    });

     $('input[id^=filter]').live('change, keyup', function(){
         // if the user has cleared the filter box, locate all parents and if a parent is collapsed, hide the children
         if ($.trim($(this).val()).length === 0) {
             var parents = $('tr[data-family_begin]');
             parents.each(function(){
                 // if the parent is collapsed, hide the children
                 var arrow = $(this).find('a img');
                 if(arrow.attr("src").indexOf("collapsed") !== -1){
                     var family = $(this).attr('data-family_begin');
                     $('tr[data-in_family="'+family+'"]').slideToggle();
                 }
             });
         }
     });

    KT.system_groups_pane.register_events();
});

KT.activation_key = (function($) {
    var subscription_setup = function(){
        var subbutton = $('#subscription_submit_button');
        var fakesubbutton = $('#fake_subscription_submit_button');
        var subcheckboxes = $('#subscription_form input[type="checkbox"]');
        var checked = 0;

        subcheckboxes.each(function(){
            $(this).change(function(){
                if($(this).is(":checked")){
                    checked += 1;
                    subbutton.removeAttr('disabled');
                } else {
                    checked -= 1;
                    if (checked === 0) {
                        subbutton.attr('disabled', 'disabled');
                    }
                }
            });
        });
    },
    initialize_new = function() {
        $('#usage_limit_checkbox').live('click', function() {
            KT.activation_key.toggle_usage_limit($(this));
        });
    },
    initialize_edit = function() {
        reset_env_select();
        enable_buttons();
        remove_content_view_highlight();
    },
    reset_env_select = function() {
        $('#path-expanded').hide();
        env_select.reset_hover();
        env_select.recalc_scroll();
    },
    refresh_list_item = function(id) {
        var elem_id = "activation_key_" + id;
        KT.panel.list.refresh(elem_id, $("#" + elem_id).data("ajax_url"));
    },
    save_key = function(data) {
        disable_buttons();

        data.ajaxSubmit({
            success: function(data) {
                remove_content_view_highlight();
                enable_buttons();
                refresh_list_item(this.url.match(/\d+/)[0]);
            }, error: function(e) {
                remove_content_view_highlight();
                enable_buttons();
        }});
    },
    cancel_key = function(data) {
        var url = $('#cancel_key').attr('data-url');
        if (url !== undefined) {
            disable_buttons();

            $.ajax({
                type: "GET",
                url: url,
                cache: false,
                success: function(response) {
                    $('.panel-content').html(response);
                    initialize_edit();
                },
                error: function(data) {
                    initialize_edit();
                }
            });
        }
    },
    toggle_family = function(data) {
        // user clicked a product family
        var family = data.closest('tr').attr('data-family_begin');

        // show/hide the elements that are part of the family
        $('tr[data-in_family="'+family+'"]').slideToggle();

        // toggle the expand/collapse arrow
        var arrow = data.find('img');
        if(arrow.attr("src").indexOf("collapsed") === -1){
            arrow.attr("src", "assets/icons/expander-collapsed.png");
        } else {
            arrow.attr("src", "assets/icons/expander-expanded.png");
        }
    },
    toggle_usage_limit = function(checkbox) {
        var tb = $("#activation_key_usage_limit");
        if (checkbox.is(":checked")) {
            tb.val('');
            tb.attr("disabled", true);
        } else {
            tb.val('');
            tb.removeAttr('disabled');
        }
    },
    toggle_family_checkboxes = function(data, checked) {
        var family = data.closest('tr'),
            family_name = family.attr('data-family_begin');
        family.siblings('tr[data-in_family="'+family_name+'"]').find('input:checkbox').attr('checked', checked);
    },
    toggle_parent_checkbox = function(data) {
        // if all children are checked, the parent should be checked... otherwise, it should be unchecked...
        // so toggle the parent checkbox, if needed...

        // check to see if the child is evan part of a product family...
        var sub = data.closest('tr'),
            in_family = sub.attr('data-in_family');
        if (in_family !== undefined) {
            // locate the family
            var family = sub.prevAll('tr[data-family_begin="'+in_family+'"]');
            if (family !== undefined) {
                // retrieve the checkboxes for the family and siblings
                var family_cbx = family.find('input:checkbox'),
                    sibling_cbxs = family.siblings('tr[data-in_family="'+in_family+'"]').find('input:checkbox'),
                    total = sibling_cbxs.length,
                    num_checked = 0;

                sibling_cbxs.each( function() {
                    if (this.checked) {
                        num_checked += 1;
                    }
                });
                if (total === num_checked) {
                    family_cbx.attr('checked', true);
                }
                else if (num_checked > 0) {
                    family_cbx.attr('checked', false);
                }
                else {
                    family_cbx.attr('checked', false);
                }
            }
        }
    },
    get_products = function() {
        // this function will retrieve the products associated with a given environment and
        // update the products box with the results
        var url = $('.path_link.active').attr('data-products_url');

        if (url !== undefined) {
            if($("#activation_key_content_view_id").val()) {
                url += ("?content_view_id=" + $("#activation_key_content_view_id").val());
            }

            disable_buttons();
            $.ajax({
                type: "GET",
                url: url,
                cache: false,
                success: function(response) {
                    $('.productsbox').html(response);
                    enable_buttons();
                },
                error: function(data) {
                    enable_buttons();
                }
            });
        }
    },
    get_content_views = function() {
        // this function will retrieve the views associated with a given environment and
        // update the views box with the results
        var url = $('.path_link.active').attr('data-content_views_url');
        if (url !== undefined) {
            disable_buttons();
            $.ajax({
                type: "GET",
                url: url,
                cache: false,
                success: function(response) {
                    // update the appropriate content on the page
                    var options = '';
                    var opt_template = KT.utils.template("<option value='<%= key %>'><%= text %></option>");

                    if (response.length > 0) {
                        // create an html option list using the response
                        $.each(response, function(key, item) {
                            options += opt_template({key: item.id, text: item.name});
                        });
                        highlight_content_views(i18n.select_content_view);
                    } else {
                        // the user selected an environment that has not views, warn them
                        highlight_content_views(i18n.no_content_views_available);
                    }
                    $("#activation_key_content_view_id").html(options);

                    enable_buttons();
                },
                error: function(data) {
                    enable_buttons();
                }
            });
        }
    },
    save_selected_environment = function(env_id) {
        // save the id of the env selected
        $("#activation_key_environment_id").attr('value', env_id);
    },
    disable_buttons = function() {
        $('#cancel_key').attr("disabled","disabled");
        $('input[id^=save_key]').attr("disabled","disabled");
    },
    enable_buttons = function() {
        $('#cancel_key').removeAttr('disabled');
        $('input[id^=save_key]').removeAttr('disabled');
    },
    highlight_content_views = function(text){
        var select_input = $("#activation_key_content_view_id"),
            highlight_text = select_input.next('span.highlight_input_text');

        select_input.addClass('highlight_input');
        if (highlight_text.length > 0) {
            highlight_text.html(text);
        } else {
            select_input.after('<span class ="highlight_input_text">' + text + '</span>');
        }
    },
    remove_content_view_highlight = function() {
        var select_input = $("#activation_key_content_view_id");
        select_input.removeClass('highlight_input');
        select_input.next('span.highlight_input_text').remove();
    };

    return {
        subscription_setup: subscription_setup,
        initialize_new: initialize_new,
        initialize_edit: initialize_edit,
        reset_env_select: reset_env_select,
        save_key: save_key,
        cancel_key: cancel_key,
        toggle_family: toggle_family,
        toggle_usage_limit: toggle_usage_limit,
        toggle_family_checkboxes: toggle_family_checkboxes,
        toggle_parent_checkbox: toggle_parent_checkbox,
        get_content_views: get_content_views,
        get_products: get_products,
        save_selected_environment: save_selected_environment,
        disable_buttons: disable_buttons,
        enable_buttons: enable_buttons,
        highlight_content_views: highlight_content_views,
        remove_content_view_highlight: remove_content_view_highlight,
        refresh_list_item: refresh_list_item
    };
}(jQuery));

/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.

 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.system_groups_pane = (function() {
    var system_groups = $("#system_groups"),
        multiselect_widget,

    add_groups = function(e){
        var checked = $("#system_group").multiselect("getChecked"),
            group_ids = checked.map(function(){return this.value;}).get();

        e.preventDefault();
        disable_inputs();

        $.ajax({
            type: "PUT",
            url: $("input#add_groups").data("url"),
            data: {group_ids:group_ids},
            cache: false,
            success: function(data) {

                // for each group that was added to the system, remove it from the multiselect
                checked.each(function(index, item){
                    var v = $(item).val(),
                        opt = $('option[value="'+v+'"]');

                    opt.remove();
                    multiselect_widget.multiselect('refresh');

                });

                // The response will include a block of html representing the list of groups successfully added.
                // Add that response to the top of the table.
                $("tr#empty_row").hide();
                $("tr#add_groups").after(data);

                enable_inputs();
            },
            error: function() {
                enable_inputs();
            }
        });
    },
    remove_groups = function() {
        var btn = $("input#remove_groups"),
            groups = [],
            checked = $("input.group_select:checked");

        checked.each(function(index, item){
            groups.push($(item).val());
        });
        if (groups.length === 0){
            return;
        }
        disable_inputs();

        $.ajax({
            type: "PUT",
            url: btn.data("url"),
            data: {group_ids:groups},
            cache: false,
            success: function(data) {
                // for each group that was removed from the system, add it to the multiselect
                checked.each(function(index, item){
                    var v = $(item).val(),
                        t = $(item).attr('name'),
                        opt = $('<option />', {value: v, text: t});

                    opt.appendTo(multiselect_widget);
                    multiselect_widget.multiselect('refresh');
                });

                checked.parents("tr").remove();
                if ($("input.group_select").length === 0) {
                    $("tr#empty_row").show();
                }
                enable_inputs();
            },
            error: function() {
                enable_inputs();
            }
        });
    },
    disable_inputs = function(){
        $("input#add_groups").attr("disabled", true);
        $("input#remove_groups").attr("disabled", true);
        $("input.group_select").attr("disabled", true);
        $("#system_group").multiselect('disable');
    },
    enable_inputs = function(){
        $("input#add_groups").removeAttr("disabled");
        $("input#remove_groups").removeAttr("disabled");
        $("input.group_select").removeAttr("disabled");
        $("#system_group").multiselect('enable');
    },
    register_events = function() {
        $("input#add_groups").live('click', add_groups);
        $("input#remove_groups").live('click', remove_groups);
    },
    register_multiselect = function() {
        multiselect_widget = $("#system_group").multiselect({
            noneSelectedText: i18n.select_system_groups,
            selectedList: 4
        }).multiselectfilter();
    };
    return {
        add_groups: add_groups,
        remove_groups: remove_groups,
        register_events: register_events,
        register_multiselect: register_multiselect
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/





;
