/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.i18n
 *
 * @description
 *   Module for internationalization.
 */

angular.module('Katello.i18n', []);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/*global i18n*/

/**
 * @ngdoc filter
 * @name  Katello.i18n:i18n
 *
 * @description
 *   Looks up a key in the i18n dictionary and returns the translated string.
 *
 * @example
 *   {{ "some_i18n_code" | i18n }}
 *   {{ "some_i18n_code_with_replacements" | i18n: ["why", "hello"] }}
 */

angular.module('Katello.i18n').filter('i18n', function() {
    return function(i18nKey, replacements) {
        var translation;
        var translated = i18nKey;

        if (i18n.hasOwnProperty(i18nKey)) {
            translation = i18n[i18nKey];
            translated = translation;
            if (typeof translation === 'function') {
                translated = translation.apply(this, replacements);
            }
        }
        return translated;
    };
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.systems
 *
 * @description
 *   Module for systems related functionality.
 */

angular.module('Katello.systems', ['alchemy', 'alch-templates', 'ui.compat', 'Katello.widgets']);

/**
 * @ngdoc config
 * @name  Katello.config
 *
 * @requires $stateProvider
 *
 * @description
 *   Used for systems level configuration such as setting up the ui state machine.
 */
angular.module('Katello.systems').config(['$stateProvider', function($stateProvider){
    $stateProvider.state('systems', {
        views: {
            '@': {
                controller: 'SystemsController'
            }
        }
    });

    $stateProvider.state('systems.alter-content', {
        views: {
            '@': {
                controller: 'SystemsBulkActionController',
                templateUrl: 'views/systems/alter-content-bulk.html'
            }
        }
    });

    $stateProvider.state('systems.alter-system-groups', {
        views: {
            '@': {
                controller: 'SystemsBulkActionController',
                templateUrl: 'assets/views/systems/alter-systems-group-bulk.html'
            }
        }
    });

    $stateProvider.state('systems.bulk-delete', {
        views: {
            '@': {
                controller: 'SystemsBulkActionController',
                templateUrl: 'views/systems/systems-delete-bulk.html'
            }
        }
    });
}]);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.system-groups
 *
 * @description
 *   Module for system groups related functionality.
 */

angular.module('Katello.system-groups', ['ngResource']);
/**
 * Copyright 2013 Red Hat, Inc.

 * This software is licensed to you under the GNU General Public
 * License as published by the Free Software Foundation; either version
 * 2 of the License (GPLv2) or (at your option) any later version.
 * There is NO WARRANTY for this software, express or implied,
 * including the implied warranties of MERCHANTABILITY,
 * NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 * have received a copy of GPLv2 along with this software; if not, see
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc factory
 * @name  Katello.system-groups.factory:SystemGroups
 *
 * @requires $resource
 * @requires Routes
 * @requires CurrentOrganization
 *
 * @description
 *   Provides a $resource for system groups.
 */

angular.module('Katello.system-groups').factory('SystemGroups',
    ['$resource', 'Routes', 'CurrentOrganization', function($resource, Routes, CurrentOrganization) {
        var SystemGroups = $resource(Routes.api_organization_system_groups_path(CurrentOrganization) + '/:systemGroupId',
                            {systemGroupId: '@systemGroupId'});

        return SystemGroups;
}]);
/**
 * Copyright 2013 Red Hat, Inc.
 *
 * This software is licensed to you under the GNU General Public
 * License as published by the Free Software Foundation; either version
 * 2 of the License (GPLv2) or (at your option) any later version.
 * There is NO WARRANTY for this software, express or implied,
 * including the implied warranties of MERCHANTABILITY,
 * NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 * have received a copy of GPLv2 along with this software; if not, see
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 */

/**
 * @ngdoc module
 * @name  Katello.factory:Routes
 *
 * @description
 *   Provides a wrapper factory around KT.routes.
 */

angular.module('Katello').factory('Routes', [function() {
    var Routes = KT.routes;

    return Routes;
}]);
/* Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/







;
