/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.panel.list.registerPage('content_view_definitions', { create : 'new_content_view_definition' });

KT.panel.set_expand_cb(function() {
    $('a.remove.disabled').tipsy({ fade:true, gravity:'s', delayIn:500, html:true, className:'content_definition-tipsy',
        title:function() { return $('.hidden-text.hidden').html();} });

    KT.object.label.initialize();
    KT.content_view_definition.initialize();
    KT.content_view_definition.initialize_views();
    KT.content_view_definition.initialize_composite_content();
    KT.content_view_definition.initialize_create();
    KT.content_view_definition_filters.initialize();
});

KT.content_view_definition = (function(){
    var status_updater,
        view_repos,
        view_conflicts = [],

    initialize = function() {
        $("#view_definitions").treeTable({
            expandable: true,
            initialState: "expanded",
            clickableNodeNames: true,
            onNodeShow: function(){$.sparkline_display_visible();}
        });
    },
    initialize_create = function() {
        var pane = $("#content_view_definition_create");
        if (pane.length === 0) {
            return;
        }
        $("#content_view_definition_composite").change(function() {
            // If the definition is a composite, show the list of views; otherwise, do not.
            if ($(this).is(":checked")) {
                $("#select_views").show();
            } else {
                $("#select_views").hide();
            }
        });
        initialize_view_checkboxes();
    },
    initialize_composite_content = function() {
        var pane = $("#composite_definition_content");
        if (pane.length === 0) {
            return;
        }
        enable_component_view_content_save();
        initialize_view_checkboxes();
    },
    initialize_views = function() {
        var pane = $("#content_view_definition_views");
        if (pane.length === 0) {
            return;
        }

        $('.repo_conflict').tipsy({fade : true, gravity : 'e', live : true, delayIn : 500,
                                   hoverable : true, delayOut : 50 });
        $('.repo_conflict').click(function(e) {e.preventDefault();});

        enable_remove_view();
        enable_refresh();
        initialize_views_treetable();
        start_updater();
    },
    initialize_view_checkboxes = function() {
        $('.view_checkbox').tipsy({fade : true, gravity : 'e', live : true, delayIn : 500,
                                  html: true, hoverable : true, delayOut : 50 });

        view_conflicts = [];

        $("input[id^='content_views_']").change(function() {
            var clicked_view_id = $(this).data('view_id').toString();

            if ($(this).is(":checked")) {
                // if the user selected a view, look to see if it has at least 1 repo in common
                // with the other enabled views and if so, disable the checkboxes on those views
                var enabled_views = $("input[id^='content_views_']:not(disabled)");
                KT.utils.each(enabled_views, function(enabled_view) {
                    var enabled_view_id = $(enabled_view).data('view_id').toString();
                    if (enabled_view_id !== clicked_view_id) {
                        if (repo_in_common(clicked_view_id, enabled_view_id)) {
                            $(enabled_view).attr("disabled", "true");
                            $(enabled_view).parent().attr("original-title", i18n.repos_in_common);
                        }
                    }
                });

            } else {
                // if the user unselected a view, look to see if it has at least 1 repo in common
                // with any disabled views and if so, enable the checkboxes on those views
                // (if they do not have a repo in common with any other selected view)
                var disabled_views = $("input[id^='content_views_']:disabled");
                KT.utils.each(disabled_views, function(disabled_view) {
                    var disabled_view_id = $(disabled_view).data('view_id').toString();
                    if (disabled_view_id !== clicked_view_id) {
                        // does the clicked view have any repos in common with the disabled view?
                        if (repo_in_common(clicked_view_id, disabled_view_id)) {
                            // does the disabled view have any repos in common with other selected views?
                            var selected_views = $("input[id^='content_views_']:checked"),
                                common_with_selected = false;

                            common_with_selected = KT.utils.find(selected_views, function(selected_view) {
                                var selected_view_id = $(selected_view).data('view_id').toString();
                                return repo_in_common(disabled_view_id, selected_view_id);
                            });
                            if (!common_with_selected) {
                                $(disabled_view).parent().removeAttr("original-title");
                                $(disabled_view).removeAttr("disabled");
                            }
                        }
                    }
                });

                remove_view_conflicts($(this));
            }
        });

        // As part of initializing, we need to determine if there are any views
        // with repos in common.  If there are and if more than one of the views is
        // selected, the views need to be highlighted as errors for the user to
        // address; otherwise, if only one of the views is selected, the others
        // should be disabled to avoid errors.
        var selected_views = $("input[id^='content_views_']:checked"),
            enabled_views = $("input[id^='content_views_']:not(disabled)");

        KT.utils.each(selected_views, function(selected_view) {
            var selected_view_id = $(selected_view).data('view_id').toString();
            KT.utils.each(enabled_views, function(enabled_view) {
                var enabled_view_id = $(enabled_view).data('view_id').toString();
                if (enabled_view_id !== selected_view_id) {
                    if (repo_in_common(selected_view_id, enabled_view_id)) {
                        if ($(enabled_view).is(":checked")) {
                            // both views are selected, so we've got a conflict that needs to be resolved
                            disable_component_view_content_save();

                            add_view_conflict(enabled_view, selected_view);
                            add_view_conflict(selected_view, enabled_view);

                            display_view_conflict(enabled_view_id, $(enabled_view).parent());
                            display_view_conflict(selected_view_id, $(selected_view).parent());
                        } else {
                            $(enabled_view).attr("disabled", "true");
                            $(enabled_view).parent().attr("original-title", i18n.repos_in_common);
                        }
                    }
                }
            });
        });
    },
    disable_component_view_content_save = function() {
        var saveButton = $("#update_component_views");
        saveButton.addClass("disabled");
        saveButton.unbind("click");
    },
    enable_component_view_content_save = function() {
        var saveButton = $("#update_component_views");
        saveButton.click(function(){
            var btn = $(this);
            if(btn.hasClass("disabled")){
                return;
            }
            btn.addClass("disabled");

            $("#component_views_form").ajaxSubmit({
                type: "POST",
                url: btn.data("url"),
                cache: false,
                success: function(){
                    $("#update_component_views").removeClass("disabled");
                },
                error: function(){
                    $("#update_component_views").removeClass("disabled");
                }
            });
        });
        saveButton.removeClass("disabled");
    },
    add_view_conflict = function(view, conflict_view) {
        var view_id = $(view).data('view_id').toString(),
            conflict_view_id = $(conflict_view).data('view_id').toString();

        if (view_conflicts[view_id] === undefined) {
            view_conflicts[view_id] = [];
        }
        if (KT.utils.indexOf(view_conflicts[view_id], view_repos[conflict_view_id]['name']) === -1) {
            view_conflicts[view_id].push(view_repos[conflict_view_id]['name']);
        }
        $(view).closest("tr").addClass("error");
    },
    remove_view_conflicts = function(unselected_view) {
        var rows_with_error = $("tr.error"),
            unselected_view_id = $(unselected_view).data('view_id').toString();

        KT.utils.each(rows_with_error, function(row) {
            var conflicted_view = $(row).find("input[id^='content_views_']"),
                conflicted_view_id = conflicted_view.data('view_id').toString(),
                unselected_index;

            if (view_conflicts[conflicted_view_id].length > 0) {
                if (unselected_view_id === conflicted_view_id) {
                    // the current element is the one that was unselected... so we can clear
                    // conflicts completely from that view
                    view_conflicts[unselected_view_id] = [];
                    $(unselected_view).closest("tr").removeClass("error");

                    $(unselected_view).attr("disabled", "true");
                    $(unselected_view).parent().attr("original-title", i18n.repos_in_common);

                } else {
                    // the current element has conflicts, but let's see if it has the
                    // unselected element as a conflict...
                    unselected_index = KT.utils.indexOf(view_conflicts[conflicted_view_id],
                                                        view_repos[unselected_view_id]['name']);
                    if (unselected_index !== -1) {
                        view_conflicts[conflicted_view_id].splice(unselected_index, 1);
                    }

                    if (view_conflicts[conflicted_view_id].length === 0) {
                        $(conflicted_view).closest("tr").removeClass("error");
                        $(conflicted_view).parent().removeAttr("original-title");
                    }
                }
            }
        });

        if ($("tr.error").length === 0) {
            enable_component_view_content_save();
        }

    },
    display_view_conflict = function(view_id, element) {
        element.attr("original-title", i18n.view_conflicts(view_conflicts[view_id].join(", ")));
    },
    repo_in_common = function(view_id_1, view_id_2) {
        // Does view 1 have any repos in common with view 2?
        var in_common = false,
            view_1_repos = view_repos[view_id_1]['repos'],
            view_2_repos = view_repos[view_id_2]['repos'];

        KT.utils.each(view_1_repos, function(view_1_repo) {
            if (KT.utils.contains(view_2_repos, view_1_repo)) {
                in_common = true;
            }
        });
        return in_common;
    },
    initialize_views_treetable = function() {
        $("#content_views").treeTable({
            expandable: true,
            initialState: "expanded",
            clickableNodeNames: false,
            onNodeShow: function(){$.sparkline_display_visible();}
        });
    },
    start_updater = function () {
        var timeout = 8000,
            pending_tasks = [];

        $('.view_version[data-pending_task_id]').each(function(i) {
            pending_tasks[i] = $(this).data("pending_task_id");
        });

        if (pending_tasks.length > 0) {
            if (status_updater !== undefined) {
                status_updater.stop();
            }
            status_updater = $.PeriodicalUpdater($('#content_view_definition_views').data('status_url'), {
                method: 'get',
                type: 'json',
                data: function() {return {task_ids: pending_tasks};},
                global: false,
                minTimeout: timeout,
                maxTimeout: timeout
            }, update_status);
        }
    },
    update_status = function(data) {
        // For each action that the user has initiated (e.g. refresh), update the status.
        var task_statuses = data["task_statuses"] || [],
            status_updated = false;

        $.each(task_statuses, function(index, status) {
            var node;

            if(!status["pending?"]) {
                node = $('.view_version[data-pending_task_id=' + status['id'] + ']');
                if(node !== undefined) {
                    node.prevAll('.parent:first').removeClass('initialized');
                    node.replaceWith(status["status_html"]);
                    status_updated = true;
                }
            }
        });

        if (status_updated === true) {
            initialize_views_treetable();
        }

        if($('.view_version[data-pending_task_id]').length === 0) {
            status_updater.stop();
            enable_refresh();
            enable_remove_view();
        }
    },
    disable = function(element) {
        element.attr('disabled', 'disabled').addClass('disabled');
    },
    enable = function(element) {
        element.removeAttr('disabled').removeClass('disabled');
    },
    disable_remove_view = function(view_id) {
        var view = $('#' + view_id),
            remove_link = view.find('a.remove_view');

        remove_link.unbind('click').click(function(event){event.preventDefault();});
        disable(remove_link);
    },
    enable_remove_view = function(view_id) {
        // This will enable the 'remove' for views that do not have a pending task
        // (e.g. publish, refresh).  If the user provides a view_id, only that view
        // will be evaluated; otherwise, if view_id is undefined, all views will be
        // evaluated.
        var views;
        if (view_id === undefined) {
            views = $('tr.view');
        } else {
            views = $('#' + view_id);
        }

        views.each(function() {
            // only enable remove, if the view does not have a task pending
            var view = $(this),
                task_pending = $('tr.child-of-' + view.attr('id')).data('pending_task_id');

            if (task_pending === undefined) {
                var remove_links = view.find('a.remove_view');

                remove_links.unbind('click').click(function(event){
                    event.preventDefault();
                    var remove_link = $(this),
                        view = remove_link.closest('tr');

                    KT.common.customConfirm({
                        message: i18n.confirm_request,
                        yes_callback: function(){

                            var view_id = view.attr('id'),
                                view_versions = $('tr.child-of-' + view_id);

                            // disable links associated with the view
                            disable_remove_view(view_id);
                            disable_refresh(view_id);

                            $.ajax({
                                type: "DELETE",
                                url: remove_link.data("url"),
                                cache: false,
                                success: function(){
                                    // on success, remove all rows associated w/ the view from the pane
                                    view_versions.remove();
                                    view.remove();
                                },
                                error: function(){
                                    // enable links associated with the view
                                    enable_remove_view(view_id);
                                    enable_refresh(view_id);
                                }
                            });
                        }
                    });
                    return false;
                });
                enable(remove_links);
            }
        });
    },
    disable_refresh = function(view_id) {
        var view_versions = $('tr.child-of-' + view_id),
            refresh_links = view_versions.find('a.refresh_action');

        refresh_links.unbind('click').click(function(event){event.preventDefault();});
        disable(refresh_links);
    },
    enable_refresh = function(view_id) {
        // If the user provides a view_id, enable the refresh for only that view; however,
        // if view_id is undefined, enable the refresh for all views.
        var refresh_links;
        if (view_id === undefined) {
            refresh_links = $('a.refresh_action');
        } else {
            refresh_links = $('tr.child-of-' + view_id).find('a.refresh_action');
        }

        refresh_links.unbind('click').bind('click', function(event) {
            event.preventDefault();
            var view_id = $(this).closest('tr.view_version').prev('tr.view').attr('id');
            disable_remove_view(view_id);

            $.ajax({
                type: 'POST',
                url: $(this).data('url'),
                cache: false,
                success: function(response) {
                    // the response contains the html for the view and all versions
                    var view_id = $(response).first('tr').attr('id');

                    $('.child-of-'+view_id).remove();
                    $('#'+view_id).replaceWith(response);

                    initialize_views_treetable();
                    disable_remove_view(view_id);
                    start_updater();
                },
                error: function() {
                    KT.panel.panelAjax('', $('#content_view_definition_views').data('views_url'), $('#panel'), false);
                }
            });
        });

        enable(refresh_links);
    };
    return {
        initialize                   : initialize,
        initialize_composite_content : initialize_composite_content,
        initialize_create            : initialize_create,
        initialize_views             : initialize_views,
        set_view_repos               : function(vp) {view_repos = vp;}
    };
}());

KT.content_view_definition_filters = (function(){
    var initialize = function() {
        initialize_filters();
        initialize_filter();
        initialize_rule();
    },
    initialize_filters = function() {
        var pane = $("#filters");
        if (pane.length === 0) {
            return;
        }
        register_remove($("#filters_form"));
        initialize_checkboxes($("#filters_form"));
    },
    initialize_filter = function() {
        var pane = $("#filter");
        if (pane.length === 0) {
            return;
        }
        $("#filter_tabs").tabs().show();
        register_remove($("#rules_form"));
        initialize_checkboxes($("#rules_form"));
    },
    initialize_rule = function() {
        var pane = $("#rule");
        if (pane.length === 0) {
            return;
        }
        $('.inclusion').unbind('change').change(function(){
            $('#update_form').ajaxSubmit({
                type: "PUT",
                cache: false,
                success: function(new_value) {
                    // Update the "Specifying included/excluded" statement on the pane
                    var element = $('#inclusion');
                    element.html(element.html().replace(element.data('initial_value'), new_value));
                    element.data('initial_value', new_value);
                }
            });
        });

        $('.filter_method').unbind('change').change(function() {
            $.ajax({
                type: 'GET',
                url: $(this).data('url'),
                cache: false,
                success: function(html) {
                    $('.rule_parameters').html(html);
                    initialize_common_rule_params();
                    initialize_errata_rule_params();
                }
            });
        });

        initialize_version_save($('.save_version'));
        initialize_version_select($('.version_type'));
        initialize_version_input($('.input.input'));

        initialize_common_rule_params();
        initialize_errata_rule_params();
    },
    initialize_version_save = function(version_save_button) {
        version_save_button.unbind('click').click(function(e) {
            // user clicked save to commit some changes to a pkg filter rule
            e.preventDefault();
            var parameter_name,
                version_selector = $(this).parent('.version_selector'),
                version,
                min_version,
                max_version,
                range_inputs;

            parameter_name = $(this).closest('tr').find('td.parameter_name').find('.parameter_checkbox').data('id');

            type = version_selector.find('.version_type').val();
            if (type === 'version') {
                version = version_selector.find('input.version').val();
            } else if (type === 'min_version') {
                min_version = version_selector.find('input.version').val();
            } else if (type === 'max_version') {
                max_version = version_selector.find('input.version').val();
            } else if (type === 'version_range') {
                range_inputs = version_selector.find('input.range');
                min_version = range_inputs.first().val();
                max_version = range_inputs.last().val();
            }

            disable_version_selector(version_selector);

            $.ajax({
                type: 'PUT',
                url: $(this).attr('href'),
                data:
                { 'parameter':
                    { 'unit' :
                        { 'name' : $(this).closest('tr').find('td.parameter_name').find('.parameter_checkbox').data('id'),
                          'version' : version,
                          'min_version' : min_version,
                          'max_version' : max_version
                        }
                    }
                },
                cache: false,
                success: function(html) {
                    version_selector.find('.save_version').hide();
                    if (type === 'all_versions') {
                        version_selector.find('input.input').val('');
                    } else if (type === 'version_range') {
                        version_selector.find('input.version').val('');
                    } else {
                        version_selector.find('input.range').val('');
                    }
                    enable_version_selector(version_selector);
                },
                error: function() {
                    enable_version_selector(version_selector);
                }
            });
        });
    },
    initialize_version_select = function(version_select) {
        version_select.unbind('change').change(function() {
            // user changed the version type (e.g. all, older than..) on a pkg filter rule
            var version_selector = $(this).parent('.version_selector');

            if ($(this).val() === 'all_versions') {
                version_selector.find('.input').hide();
            } else if ($(this).val() === 'version_range') {
                version_selector.find('.version').hide();
                version_selector.find('.range').show();
            } else {
                version_selector.find('.range').hide();
                version_selector.find('.version').show();
            }
            version_selector.find('.save_version').show();
        });
    },
    initialize_version_input = function(version_input) {
        version_input.unbind('keypress').keypress(function() {
            var version_selector = $(this).parent('.version_selector');
            version_selector.find('.save_version').show();
        });
    },
    initialize_common_rule_params = function() {
        var pane = $("#parameter_list"),
            rule_type = $("#add_rule").data("rule_type"),
            filter_id = $("#add_rule").data("filter_id");
        if (pane.length === 0) {
            return;
        }

        $("#rule_input").unbind('keypress').keypress(function(event) {
            if(event.keyCode === 13) {
                event.preventDefault();
                $("#add_rule").click();
            }
        });

        // setup autocomplete
        if (rule_type === "erratum") {
            $("#rule_input").autocomplete({
                source: KT.routes.auto_complete_errata_path({filter_id: filter_id}),
                minLength: 3
            });
        } else if (rule_type === "rpm") {
            $("#rule_input").autocomplete({
                source: KT.routes.auto_complete_packages_path({filter_id: filter_id})
            });
        }

        // setup the tipsy
        $(".rule-search-tipsy").tipsy();

        $('#add_rule').unbind('click').click(function() {
            var rule_input = $('input#rule_input').val(),
                data;

            if ($(this).data('rule_type') === 'erratum') {
                data = {'parameter[unit][id]': rule_input};
            } else {  // this is for a package or package group rule
                data = {'parameter[unit][name]': rule_input};
            }
            if (rule_input.length > 0) {
                $.ajax({
                    type: 'PUT',
                    url: $(this).data('url'),
                    data: data,
                    cache: false,
                    success: function(html) {
                        var empty_row = $("tr#empty_row");
                        empty_row.after(html);
                        empty_row.hide();
                        initialize_checkboxes($("#parameters_form"));

                        var new_parameter = $('.parameter_checkbox[data-id=' + rule_input + ']').closest('tr');
                        initialize_version_save(new_parameter.find('.save_version'));
                        initialize_version_select(new_parameter.find('.version_type'));
                        initialize_version_input(new_parameter.find('.input.input'));
                    }
                });
            }
        });
        register_remove_filter_rule_param($("#parameters_form"));
        initialize_checkboxes($("#parameters_form"));
    },
    initialize_errata_rule_params = function() {
        var pane = $("#errata_parameters");
        if (pane.length === 0) {
            return;
        }
        KT.editable.initialize_datepicker();
        KT.editable.initialize_multiselect();
    },
    register_remove = function(form) {
        var remove_button = form.find("#remove_button");
        remove_button.unbind('click');
        remove_button.click(function(){
            var btn = $(this);
            if(btn.hasClass("disabled")){
                return;
            }
            disable(remove_button);

            form.ajaxSubmit({
                type: "DELETE",
                url: btn.data("url"),
                cache: false,
                success: function(){
                    // remove the deleted filters from the table and show the 'empty' message
                    // if all filters have been deleted
                    $('input[type="checkbox"]:checked').closest('tr').remove();
                    if ($('input[type="checkbox"]').length === 0) {
                        $('tr#empty_row').show();
                    }
                    disable(remove_button);
                },
                error: function(){
                    enable(remove_button);
                }
            });
        });
        disable(remove_button);
    },
    register_remove_filter_rule_param = function(form) {
        var remove_button = form.find("#remove_button");
        remove_button.unbind('click').click(function(){
            var btn = $(this), parameters = [];
            if(btn.hasClass("disabled")){
                return;
            }
            disable(remove_button);

            $('input.parameter_checkbox:checked').each(function() {
                parameters.push($(this).val());
            });

            $.ajax({
                type: "DELETE",
                url: btn.data("url"),
                cache: false,
                data: {'units': parameters},
                success: function(){
                    // remove the deleted parameters from the table and show the 'empty' message
                    // if all have been deleted
                    $('input[type="checkbox"]:checked').closest('tr').remove();
                    if ($('input[type="checkbox"]').length === 0) {
                        $('tr#empty_row').show();
                    }
                    disable(remove_button);
                },
                error: function(){
                    enable(remove_button);
                }
            });
        });
        disable(remove_button);
    },
    disable_version_selector = function(selector) {
        disable(selector.find('select.version_type'));
        disable(selector.find('input.input'));
        disable(selector.find('a.save_version'));
    },
    enable_version_selector = function(selector) {
        enable(selector.find('select.version_type'));
        enable(selector.find('input.input'));
        enable(selector.find('a.save_version'));
    },
    disable = function(button) {
        button.attr('disabled', 'disabled').addClass('disabled');
    },
    enable = function(button) {
        button.removeAttr('disabled').removeClass('disabled');
    },
    initialize_checkboxes = function(form) {
        var checkboxes = $('input[type="checkbox"]'),
            button = form.find("#remove_button");

        checkboxes.unbind('change').each(function(){
            $(this).change(function(){
                if($(this).is(":checked")) {
                    enable(button);
                } else if($('input[type="checkbox"]:checked').length === 0) {
                    disable(button);
                }
            });
        });
    };
    return {
        initialize : initialize
    };
}());
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

$(document).ready(function() {
    $(".toggle").live('click', function(){
       var btn = $(this);
       var parent = btn.parents(".product_entry");
       if(parent.hasClass("disabled")){
           return;
       }

       if (btn.hasClass("collapsed")){
        btn.addClass("expanded").removeClass("collapsed");
       }
       else {
        btn.removeClass("expanded").addClass("collapsed");
       }
       btn.parent().find(".options").toggle();

    });

    KT.panel.set_expand_cb(function(){
        KT.product_input.register();
    });
});

KT.product_input = (function(){

    var register = function() {

        var select = $('#product_select');
        if (select.length === 0){
            return;
        }

        KT.filter_renderer.render_products_repos();

        select.html(KT.filter_renderer.product_select_template());

        select.chosen({
            custom_compare:function(search, name, value){
                if (name.toUpperCase().indexOf(search.toUpperCase()) > -1){
                 return true;
                }
                else if(value.indexOf("PROD-") === 0)  {
                    var prod_id = value.split("-")[1];
                    var prod = KT.products[prod_id];
                    if(prod && prod.name === name){
                        var match = false;
                        $.each(prod.repos, function(index, item){
                             if (item.name.toUpperCase().indexOf(search.toUpperCase()) > -1){
                                 match = true;
                                 return false;
                             }
                        });
                        return match;
                    }
                }
            }
        });

        $("#add_product").click(function(e){
            var value;
            var add_btn = $("add_product");
            e.preventDefault();
            if (add_btn.hasClass("disabled")){
                return;
            }
            add_btn.addClass("disabled");

            value = select.val();
            if (value.indexOf("PROD-") === 0){
                value = value.substring(5);
                KT.filters.add_product(value);
            }
            else if(value.indexOf("REPO-") === 0){
                value = value.substring(5);
                KT.filters.add_repo(value, KT.filters.lookup_repo_product(value), true);
            }
        });

        $("#update_products").click(function(){
            var btn = $(this);
            if(btn.hasClass("disabled")){
                return;
            }
            btn.addClass("disabled");
            KT.filters.update_product_repos();
        });

        $("#revert_products").click(function(){
            KT.filters.revert_products();
        });

    },
    post_render_register = function() {
        $(".product_radio").unbind().change(function(){
            var radio = $(this),
                value = radio.val(),
                parent = radio.parents(".product_entry"),
                prod_id = parseInt(parent.attr("data-id"), 10),
                list = parent.find(".repos_list"),
                search = parent.find(".repos_search"),
                filter = KT.filters.get_current_filter(),
                repos;

            //if 'all was selected'
            if (value === "all"){
                list.hide(); //hide list and re-render message
                search.hide();
                KT.filter_renderer.render_product_message(prod_id, true);
                if (KT.utils.indexOf(filter.products, prod_id) === -1){
                    filter.products.push(prod_id); //add product to filter
                }
                //remove product from repos and cache it
                repos = filter.repos[prod_id];
                delete filter.repos[prod_id];
                if (repos){
                    KT.filters.get_repo_cache()[prod_id] = repos;
                }
            }
            else { //else 'select repos' was selected
                list.show(); //show list and rerender message
                search.show();
                KT.filters.pop_product(prod_id);

                filter.repos[prod_id] = [];
                repos = KT.filters.get_repo_cache()[prod_id];
                if (repos && repos.length > 0){
                    filter.repos[prod_id] =  repos;
                    delete KT.filters.get_repo_cache()[prod_id];
                }
                KT.filter_renderer.render_product_message(prod_id, false, KT.filters.get_current_filter().repos[prod_id]);

            }
        });

        $('.repo_select').chosen();

        $(".add_repo").unbind().click(function(){
            var select,id, repo_id, prod_id;
            select = $(this).siblings("select");
            repo_id = select.val();
            prod_id = select.attr("data-prod_id");
            KT.filters.add_repo(repo_id, prod_id);
        });

        $(".remove_repo").unbind().click(function(){
            var repo = $(this).parent(),
                repo_id = repo.attr("data-id"),
                parent = repo.parents('.product_entry'),
                prod_id = parseInt(parent.attr("data-id"), 10),
                filter = KT.filters.get_current_filter();

            KT.filters.pop_repo(prod_id, repo_id);
            repo.remove();
            KT.filter_renderer.rerender_repo_select(prod_id);
            KT.filter_renderer.render_product_message(prod_id, false, filter.repos[prod_id]);

        });

        $(".remove_product").unbind().click(function(){
            var parent = $(this).parents('.product_entry');
            var prod_id = parseInt(parent.attr("data-id"), 10);
            var filter = KT.filters.get_current_filter();
            KT.filters.pop_product(prod_id);
            delete filter.repos[prod_id];
            parent.remove();
            KT.filter_renderer.empty_check();
        });
    };

    return {
        register:register,
        post_render_register: post_render_register
    };
})();


KT.filter_renderer = (function(){
    var render_products_repos =  function(){
        var div = $("#product_list");
        div.html(products_template());
        KT.product_input.post_render_register();

        div.find("tr").not(".no_sort").sortElements(function(a,b){
                var a_html = $.trim($(a).find('.text').text());
                var b_html = $.trim($(b).find('.text').text());
                if (a_html && b_html ) {
                    return  a_html.toUpperCase() >
                            b_html.toUpperCase() ? 1 : -1;
                }
        });
        empty_check();

    },
    render_single_product = function(prod_id){
        prod_id = parseInt(prod_id, 10);
        $('.product_entry[data-id=' + prod_id + ']').remove();
        $("#product_list").prepend(single_product(prod_id));
        $('.product_entry[data-id=' + prod_id + ']').hide();
        $('.product_entry[data-id=' + prod_id + ']').fadeIn(500);
        KT.product_input.post_render_register();
        empty_check();
    },
    render_single_repo = function(prod_id, repo_id){
        var list = $('.product_entry[data-id=' + prod_id + ']').find(".repos_list");
        list.append(repo_template(prod_id, repo_id, true));
        KT.product_input.post_render_register();
        render_product_message(prod_id, false);
        rerender_repo_select(prod_id);
        empty_check();
    },
    render_product_message = function(prod_id, is_full) {
        var msg = product_message(prod_id, is_full, KT.filters.get_current_filter().repos[prod_id]);
        $(".product_entry[data-id=" + prod_id + "]").find(".prod_message").text(msg);
    },
    rerender_repo_select = function(prod_id){
        var select = $('.product_entry[data-id=' + prod_id + ']').find("select");
        select.html(repo_search_items(prod_id));
        select.trigger("liszt:updated"); //update the chosen select
    },
    product_select_template = function() {
        var html = "";
        $.each(KT.editable_products, function(id, prod){
            html += '<option value="PROD-' + prod.id+'">' + prod.name +'</option>';
            if (prod.repos.length > 0){
                $.each(prod.repos, function(index, repo){
                    html += '<option value="REPO-' + repo.id +'"> - ' + repo.name + " </option>";
                });
            }
        });
        return html;
    },
    product_options = function(id, name, editable, is_full, repos) {
        var style = is_full ? 'style="display:none;"' : '';
        var html_name = "PROD-" + id;
        var search_box = editable ? repo_search(id) : "";
        var html = '<div class="options">';
            html += '<div>' + product_radio('all' + html_name, html_name, i18n.all_repos, editable, is_full, 'all') + '</div>';
            html += '<div>' + product_radio('sel' + html_name, html_name, i18n.select_repos, editable, !is_full, 'sel') +
                        '<span ' + style + 'class="repos_search">' + search_box +  '</span>' +
                    '</div>';
            html += '<div class="repos_list">';
            $.each(repos, function(index, repo_id){
                html += repo_template(id, repo_id, editable);
            });
            html += '</div></div>';
        return html;
    },
    product_radio = function(id, name, label, editable, is_checked, value){
      var checked = is_checked ? "checked" : "";
      var disabled = editable ? "" : "disabled=disabled" ;
      var html = "";
      html += '<input type="radio" ' + disabled + " " + checked + ' id="'+ id +'" name="' +name+ '" ' +
          'value="' + value + '" class="product_radio"/>';
      html += '<label for="' + id + '">' + label + '</label>';
      return html;
    },
    product_message = function(prod_id, is_full, repos){
        var message = i18n.entire_selected;
        if (!is_full){
            message = i18n.x_of_y_repos(KT.utils.size(repos), KT.utils.size(KT.products[prod_id].repos));
        }
        return message;
    },
    repo_template = function(prod_id, repo_id, editable) {
        var name, repo,
            html = '';

        repo = KT.utils.find(KT.products[prod_id].repos, function(repo){
            return repo.id.toString() === repo_id;
        });

        html += '<div class="repo" data-id="'  + repo_id + '">';
        html += repo.name;

        if (editable){
            html += '<a class="remove_repo"> &nbsp;' + i18n.remove + '</a>';
        }

        html += '</div>';
        return html;
    },
    repo_search = function(prod_id){
        var html = '<select style="width: 250px;" class="repo_select" data-prod_id="' + prod_id + '">';
        html += repo_search_items(prod_id);
        html += '</select>';
        html += '<a class="add_repo"> &nbsp;' + i18n.add_plus + '</a>';
        return html;
    },
    repo_search_items = function(prod_id){
        var html = "";
        $.each(KT.products[prod_id].repos, function(index, item){
            var used = KT.filters.get_current_filter().repos[prod_id] || [];
            if (used[item.id] === undefined) {
                html+= '<option value="' + item.id + '">' + item.name + '</option>';
            }
        });
        return html;
    },
    single_product = function(prod_id) {
        var filter = KT.filters.get_current_filter();
        var repos = [];
        var editable = KT.utils.indexOf(Object.keys(KT.editable_products), prod_id + "") > -1;
        if (KT.utils.indexOf(filter.products, prod_id) > -1){
            //render the cached repos if we want to
            if (KT.filters.get_repo_cache()[prod_id]){
                repos = KT.filters.get_repo_cache()[prod_id];
            }
            return product_template(prod_id, KT.products[prod_id].name, editable,  true, repos);
        }
        else {
            repos = filter.repos[prod_id];
            return product_template(prod_id, KT.products[prod_id].name, editable, false, repos);
        }
    },
    product_template = function(id, name, editable, is_full, repos){
        var html = '<tr><td class="no_padding"><div data-id="' + id + '" class="product_entry">';
        html += '<div  class="small_col toggle collapsed" data-id="' + id +'"></div>';
        html += '<div class="large_col">';
            html += '<span class="text">' + name + " <span class='prod_message'>" + product_message(id, is_full, repos) + '</span>';
            if (editable){
                html += '<a class="remove_product">&nbsp;' + i18n.remove + '</a>';
            }
            html += '</span>';
            html += product_options(id, name, editable, is_full, repos);
        html += '</div>';
        html += "</div></td></tr>";
        return html;
    },
    empty_check = function(){
        var filter = KT.filters.get_current_filter();
        if (filter.products.length === 0 && Object.keys(filter.repos).length === 0){
            var html = '<tr id="empty_message"><td>' + i18n.no_products_repos  + '</td></tr>';
            $("#product_list").html(html);
        }
        else {
            $("#empty_message").remove();
        }
    },
    products_template = function(){
        var html = "";
        var filter = KT.filters.get_current_filter();
        if (!filter){return "";}
        var all_products = filter.products.concat(Object.keys(filter.repos));
        $.each(all_products , function(index, id){
            html += single_product(id);
        });
        return html;
    };

    return {
        render_products_repos:render_products_repos,
        product_select_template: product_select_template,
        render_product_message: render_product_message,
        render_single_product : render_single_product,
        render_single_repo : render_single_repo,
        rerender_repo_select: rerender_repo_select,
        empty_check: empty_check
    };

})();


KT.filters = (function(){
    var current_filter,
    saved_filter,
    repo_cache = {};

    add_package = function(name, item_id, cleanup_cb){
        var input = $("#package_input");

        //verify the package isn't already displayed
        if ($(".package_select[value=" + KT.common.escapeId(name) + "]").length !== 0){
            cleanup_cb();
            return;
        }

        disable_package_inputs();

        $.ajax({
            type: "PUT",
            url: input.data("url"),
            data: {packages:[name]},
            cache: false,
            success: function(data) {
                var table = $("#package_filter").find("table");
                $(".empty_message").remove();
                $.each(data, function(index, item){
                    var html = "<tr><td>";
                    html+= '<input type="checkbox" id="' + item + '" class="package_select" value="' + item + '">';
                    html += '&nbsp;<label for="' + item + '">' + item + '</label></td></tr>';
                    table.append(html);
                });
                table.find("tr").not(".no_sort").sortElements(function(a,b){
                        var a_html = $.trim($(a).find('td').text());
                        var b_html = $.trim($(b).find('td').text());
                        if (a_html && b_html ) {
                            return  a_html.toUpperCase() >
                                    b_html.toUpperCase() ? 1 : -1;
                        }
                });
                cleanup_cb();
                enable_package_inputs();
            }
        });
    },
    remove_packages = function() {
        var btn = $("#remove_packages"),
        pkgs = [],
        checked = $(".package_select:checked");

        if (btn.hasClass("disabled")){
            return;
        }

        checked.each(function(index, item){
            pkgs.push($(item).val());
        });
        if (pkgs.length === 0){
            return;
        }
        disable_package_inputs();

        $.ajax({
            type: "PUT",
            url: btn.data("url"),
            data: {packages:pkgs},
            cache: false,
            success: function(data) {
                var html = '<tr class="empty_message"><td>' + i18n.no_packages + '</td></tr>';
                checked.parents("tr").remove();
                if ($("#package_filter").find("tr").not(".no_sort").length === 0){
                    $("#package_filter").find("table").append(html);
                }
                enable_package_inputs();
            }
        });
    },
    disable_package_inputs = function(){
        $("#package_filter").find("input").addClass("disabled");

    },
    enable_package_inputs = function(){
        $("#package_filter").find("input").removeClass("disabled");
    },
    get_current_filter = function(){
        return current_filter;
    },
    set_current_filter = function(filter_in) {
        current_filter = filter_in;
        saved_filter = $.parseJSON(JSON.stringify(filter_in));
    },
    revert_products = function() {
      current_filter =   $.parseJSON(JSON.stringify(saved_filter));
      KT.filter_renderer.render_products_repos();
    },
    update_product_repos = function() {
        var repos = [];
        $.ajax({
            type: "PUT",
            contentType: "application/json",
            url: $("#update_products").data("url"),
            data: JSON.stringify({products:current_filter.products, repos:current_filter.repos}),
            cache: false,
            success: function(){
                repo_cache = []; //clear repo cache
                $("#update_products").removeClass("disabled");
                KT.filter_renderer.render_products_repos();
                saved_filter = $.parseJSON(JSON.stringify(current_filter));
            }
        });
    },
    add_product = function(prod_id){
        prod_id = parseInt(prod_id, 10);
        if ($.inArray( prod_id, current_filter.products) === -1 &&
             current_filter.repos[prod_id] === undefined){
            repo_cache[prod_id] = [];
            current_filter.products.push(prod_id);
            KT.filter_renderer.render_single_product(prod_id);
            expand_product(prod_id);
        }
    },
    pop_product = function(prod_id){
      var index = KT.utils.indexOf(current_filter.products, parseInt(prod_id, 10));
        if (index > -1) {
            current_filter.products.splice(index, 1);
        }

    },
    pop_repo = function(prod_id, repo_id){
        var repos = current_filter.repos[parseInt(prod_id, 10)],
        index;
        if(repos) {
            index = KT.utils.indexOf(repos, repo_id);
            repos.splice(index, 1);
        }

    },
    add_repo = function(repo_id, prod_id, bump_up){
        if(repo_id === null || repo_id === undefined) {
            return;
        }
        prod_id = parseInt(prod_id, 10);
        if ($.inArray( prod_id, current_filter.products) > -1){
            KT.filters.pop_product(prod_id);
        }

        //clear the repo cache
        if (repo_cache[prod_id] !== undefined){
            repo_cache[prod_id] = [];
        }
        if (current_filter.repos[prod_id] === undefined){
            current_filter.repos[prod_id] = [];
        }

        if (KT.utils.indexOf(current_filter.repos[prod_id], repo_id) === -1) {
            current_filter.repos[prod_id].push(repo_id);
            if (bump_up){
                KT.filter_renderer.render_single_product(prod_id);
            }
            else {
                KT.filter_renderer.render_single_repo(prod_id, repo_id);
            }
        }
        expand_product(prod_id);
    },
    lookup_repo_product = function(repo_id){
      var found;

      $.each(KT.products, function(index, prod){
        $.each(prod.repos, function(index, repo){
           if (repo.id.toString() === repo_id){
               found = prod.id;
               return false;
           }
        });
      });
      return found;
    },
    expand_product = function(id){
        $(".product_entry").find('.collapsed.toggle[data-id='  + id + ']').click();
    },
    collapse_product = function(id){
        $(".product_entry").find('.expanded.toggle[data-id='  + id + ']').click();
    },
    get_repo_cache = function(){
        return repo_cache;
    };

    return {
        add_package     : add_package,
        remove_packages : remove_packages,
        get_current_filter: get_current_filter,
        set_current_filter: set_current_filter,
        add_repo        : add_repo,
        pop_repo        : pop_repo,
        add_product     : add_product,
        pop_product     : pop_product,
        lookup_repo_product: lookup_repo_product,
        expand_product  : expand_product,
        collapse_product: collapse_product,
        get_repo_cache  : get_repo_cache,
        update_product_repos: update_product_repos,
        revert_products : revert_products
    };
})();
/**
 * jQuery.fn.sortElements
 * --------------
 * @author James Padolsey (http://james.padolsey.com)
 * @version 0.11
 * @updated 18-MAR-2010
 * --------------
 * @param Function comparator:
 *   Exactly the same behaviour as [1,2,3].sort(comparator)
 *   
 * @param Function getSortable
 *   A function that should return the element that is
 *   to be sorted. The comparator will run on the
 *   current collection, but you may want the actual
 *   resulting sort to occur on a parent or another
 *   associated element.
 *   
 *   E.g. $('td').sortElements(comparator, function(){
 *      return this.parentNode; 
 *   })
 *   
 *   The <td>'s parent (<tr>) will be sorted instead
 *   of the <td> itself.
 *
 * Copyright (c) 2010 James Padolsey
 * -------------------------------------------------------
 * Dual licensed under the MIT and GPL licenses.
 *    - http://www.opensource.org/licenses/mit-license.php
 *    - http://www.gnu.org/copyleft/gpl.html
 */

jQuery.fn.sortElements = (function(){
    
    var sort = [].sort;
    
    return function(comparator, getSortable) {
        
        getSortable = getSortable || function(){return this;};
        
        var placements = this.map(function(){
            
            var sortElement = getSortable.call(this),
                parentNode = sortElement.parentNode,
                
                // Since the element itself will change position, we have
                // to have some way of storing it's original position in
                // the DOM. The easiest way is to have a 'flag' node:
                nextSibling = parentNode.insertBefore(
                    document.createTextNode(''),
                    sortElement.nextSibling
                );
            
            return function() {
                
                if (parentNode === this) {
                    throw new Error(
                        "You can't sort elements if any one is a descendant of another."
                    );
                }
                
                // Insert before flag:
                parentNode.insertBefore(this, nextSibling);
                // Remove flag:
                parentNode.removeChild(nextSibling);
                
            };
            
        });
       
        return sort.call(this, comparator).each(function(i){
            placements[i].call(getSortable.call(this));
        });
        
    };
    
})();
/*
 * jQuery treeTable Plugin unreleased
 * http://ludo.cubicphuse.nl/jquery-plugins/treeTable/doc/
 *
 * Copyright 2011, Ludo van den Boom
 * Dual licensed under the MIT or GPL Version 2 licenses.
 */

(function($) {
  // Helps to make options available to all functions
  // TODO: This gives problems when there are both expandable and non-expandable
  // trees on a page. The options shouldn't be global to all these instances!
  var options;
  var defaultPaddingLeft;

  $.fn.treeTable = function(opts) {
    options = $.extend({}, $.fn.treeTable.defaults, opts);

    return this.each(function() {
      $(this).addClass("treeTable").find("tbody tr").each(function() {
        // Initialize root nodes only if possible
        if(!options.expandable || $(this)[0].className.search(options.childPrefix) == -1) {
          // To optimize performance of indentation, I retrieve the padding-left
          // value of the first root node. This way I only have to call +css+
          // once.
          if (isNaN(defaultPaddingLeft)) {
            defaultPaddingLeft = parseInt($($(this).children("td")[options.treeColumn]).css('padding-left'), 10);
          }

          initialize($(this));
        } else if(options.initialState == "collapsed") {
          this.style.display = "none"; // Performance! $(this).hide() is slow...
        }
      });
    });
  };

  $.fn.treeTable.defaults = {
    childPrefix: "child-of-",
    clickableNodeNames: false,
    expandable: true,
    indent: 19,
    initialState: "collapsed",
    onNodeShow: null,
    treeColumn: 0
  };

  // Recursively hide all node's children in a tree
  $.fn.collapse = function() {
    $(this).addClass("collapsed");

    childrenOf($(this)).each(function() {
      if(!$(this).hasClass("collapsed")) {
        $(this).collapse();
      }

      this.style.display = "none"; // Performance! $(this).hide() is slow...
    });

    return this;
  };

  // Recursively show all node's children in a tree
  $.fn.expand = function() {
    $(this).removeClass("collapsed").addClass("expanded");

    childrenOf($(this)).each(function() {
      initialize($(this));

      if($(this).is(".expanded.parent")) {
        $(this).expand();
      }

      // this.style.display = "table-row"; // Unfortunately this is not possible with IE :-(
      $(this).show();

      if($.isFunction(options.onNodeShow)) {
        options.onNodeShow.call();
      }
    });

    return this;
  };

  // Reveal a node by expanding all ancestors
  $.fn.reveal = function() {
    $(ancestorsOf($(this)).reverse()).each(function() {
      initialize($(this));
      $(this).expand().show();
    });

    return this;
  };

  // Add an entire branch to +destination+
  $.fn.appendBranchTo = function(destination) {
    var node = $(this);
    var parent = parentOf(node);

    var ancestorNames = $.map(ancestorsOf($(destination)), function(a) { return a.id; });

    // Conditions:
    // 1: +node+ should not be inserted in a location in a branch if this would
    //    result in +node+ being an ancestor of itself.
    // 2: +node+ should not have a parent OR the destination should not be the
    //    same as +node+'s current parent (this last condition prevents +node+
    //    from being moved to the same location where it already is).
    // 3: +node+ should not be inserted as a child of +node+ itself.
    if($.inArray(node[0].id, ancestorNames) == -1 && (!parent || (destination.id != parent[0].id)) && destination.id != node[0].id) {
      indent(node, ancestorsOf(node).length * options.indent * -1); // Remove indentation

      if(parent) { node.removeClass(options.childPrefix + parent[0].id); }

      node.addClass(options.childPrefix + destination.id);
      move(node, destination); // Recursively move nodes to new location
      indent(node, ancestorsOf(node).length * options.indent);
    }

    return this;
  };

  // Add reverse() function from JS Arrays
  $.fn.reverse = function() {
    return this.pushStack(this.get().reverse(), arguments);
  };

  // Toggle an entire branch
  $.fn.toggleBranch = function() {
    if($(this).hasClass("collapsed")) {
      $(this).expand();
    } else {
      $(this).removeClass("expanded").collapse();
    }

    return this;
  };

  // === Private functions

  function ancestorsOf(node) {
    var ancestors = [];
    while(node = parentOf(node)) {
      ancestors[ancestors.length] = node[0];
    }
    return ancestors;
  };

  function childrenOf(node) {
    return $("table.treeTable tbody tr." + options.childPrefix + node[0].id);
  };

  function getPaddingLeft(node) {
    var paddingLeft = parseInt(node[0].style.paddingLeft, 10);
    return (isNaN(paddingLeft)) ? defaultPaddingLeft : paddingLeft;
  }

  function indent(node, value) {
    var cell = $(node.children("td")[options.treeColumn]);
    cell[0].style.paddingLeft = getPaddingLeft(cell) + value + "px";

    childrenOf(node).each(function() {
      indent($(this), value);
    });
  };

  function initialize(node) {
    if(!node.hasClass("initialized")) {
      node.addClass("initialized");

      var childNodes = childrenOf(node);

      if(!node.hasClass("parent") && childNodes.length > 0) {
        node.addClass("parent");
      }

      if(node.hasClass("parent")) {
        var cell = $(node.children("td")[options.treeColumn]);
        var padding = getPaddingLeft(cell) + options.indent;

        childNodes.each(function() {
          $(this).children("td")[options.treeColumn].style.paddingLeft = padding + "px";
        });

        if(options.expandable) {
          cell.prepend('<span style="margin-left: -' + options.indent + 'px; padding-left: ' + options.indent + 'px" class="expander"></span>');
          $(cell[0].firstChild).click(function() { node.toggleBranch(); });

          if(options.clickableNodeNames) {
            cell[0].style.cursor = "pointer";
            $(cell).click(function(e) {
              // Don't double-toggle if the click is on the existing expander icon
              if (e.target.className != 'expander') {
                node.toggleBranch();
              }
            });
          }

          // Check for a class set explicitly by the user, otherwise set the default class
          if(!(node.hasClass("expanded") || node.hasClass("collapsed"))) {
            node.addClass(options.initialState);
          }

          if(node.hasClass("expanded")) {
            node.expand();
          }
        }
      }
    }
  };

  function move(node, destination) {
    node.insertAfter(destination);
    childrenOf(node).reverse().each(function() { move($(this), node[0]); });
  };

  function parentOf(node) {
    var classNames = node[0].className.split(' ');

    for(var key=0; key<classNames.length; key++) {
      if(classNames[key].match(options.childPrefix)) {
        return $("#" + classNames[key].substring(9));
      }
    }

    return null;
  };
})(jQuery);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

KT.auto_complete_box = function(params) {

    var settings = {
        values: undefined, //either a url, an array, or a callback of items for auto_completion
        default_text: undefined, //default text to go into the search box if desired
        comma_separated_input: false,
        input_id: undefined,
        input: undefined,
        selected_input_id: undefined,
        selected_input: undefined,
        form_id: undefined,
        add_btn_id: undefined,
        add_btn: undefined,
        add_text: i18n.add_plus,
        require_select: false, //disable the add button if proper autocompleted item isn't selected
        add_cb: function(item, item_id, cb){cb();}
    };
    $.extend( settings, params );

    var add_item_from_input = function(e) {
        var item = get_input().attr("value"),
            item_id = get_selected_input().val();


        e.preventDefault();
        if (item.length === 0 || item === settings.default_text){
                return;
        }
            get_add_btn().addClass("working");
        add_item_base(item, item_id, true);
    },
    get_input = function(){
        return settings.input || $("#" + settings.input_id);
    },
    get_selected_input = function(){
        return settings.selected_input || $("#" + settings.selected_input_id);
    },
    get_add_btn = function(){
        return settings.add_btn || $("#" + settings.add_btn_id);
    },
    add_item_base = function(item, item_id, focus) {
        var input = get_input(),
            add_btn = get_add_btn();

        input.removeClass("input_error");
        add_btn.addClass("working");
        add_btn.html("<img  src='assets/icons/spinner.gif'>");
        input.attr("disabled", "disabled");
        input.autocomplete('disable');
        input.autocomplete('close');

        if (settings.comma_separated_input) {
            // convert the item string to an array
            item = split(item);
        }
        settings.add_cb(item, item_id, function(){
            reset_input();
            if (focus) {
                get_input().focus();
            }
        });
    },
    reset_input = function() {
        //re-lookup all items, since a redraw may have happened
        var input = get_input(),
            add_btn = get_add_btn();
        add_btn.removeClass('working');
        if (add_btn.text() === "") {
            add_btn.html(settings.add_text);
        }
        input.removeAttr('disabled');
        input.autocomplete('enable');
        input.val('');
    },
    manually_add = function(item, item_id) {
        add_item_base(item, item_id, false);
    },
    error = function() {
        var input = get_input();
        input.addClass("input_error");
    },
    split = function(val) {
        return val.split(/,\s*/);
    },
    extractLast = function(term) {
        return split(term).pop();
    };

    //initialization
    var input = get_input(),
        form = $("#" + settings.form_id),
        add_btn = get_add_btn();

    if (settings.default_text) {
        input.val(settings.default_text);
        input.focus(function() {
            if (input.val() === settings.default_text) {
                input.val("");
            }
        });
        input.blur(function() {
            if(input.val() === "") {
                input.val(settings.default_text);
            }
        });
    }

    if (!settings.comma_separated_input) {
        input.autocomplete({
            source: function (request, response) {
                $.ajax({
                    url: settings.values,
                    data: request,
                    dataType: 'json',
                    success: function (data) {
                        response(data);
                        KT.utils.each(data, function(item){
                            if(request.term === item.value ||
                               request.term === item.label)
                            {
                                get_selected_input().val(item.id);
                                add_btn.removeAttr("disabled");
                            }
                        });
                    },
                    error: function () {
                        response([]);
                    }
                });
            },
            search: function(){
                get_selected_input().val('');
                if(settings.require_select){
                    add_btn.attr("disabled", "disabled");
                }
            },
            select: function (event, ui) {
                get_input().val(ui.item.value);
                get_selected_input().val(ui.item.id);
                add_btn.removeAttr("disabled");
                return false;
            }
        });
    } else {
        input.autocomplete({
            source: function(request, response) {
                $.getJSON( settings.values, {
                    term: extractLast(request.term)
                }, response);
            },
            search: function () {
                // custom minLength
                var term = extractLast(this.value);
                if (term.length < 1) {
                    return false;
                }
            },
            focus: function () {
                // prevent value inserted on focus
                return false;
            },
            select: function (event, ui) {
                var terms = split(this.value);
                // remove the current input
                terms.pop();
                // add the selected item
                terms.push(ui.item.value);
                // add placeholder to get the comma-and-space at the end
                terms.push("");
                this.value = terms.join(", ");
                return false;
            }
        });
    }
    add_btn.bind('click', add_item_from_input);
    form.submit(add_item_from_input);

    return {
        manually_add: manually_add,
        error: error,
        reset_input: reset_input
    };
};


/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/






;
