/*
 * Copyright (c) 2008 Greg Weber greg at gregweber.info
 * Dual licensed under the MIT and GPLv2 licenses just as jQuery is:
 * http://jquery.org/license
 *
 * documentation at http://gregweber.info/projects/uitablefilter
 *
 * allows table rows to be filtered (made invisible)
 * <code>
 * t = $('table')
 * $.uiTableFilter( t, phrase )
 * </code>
 * arguments:
 *   jQuery object containing table rows
 *   phrase to search for
 *   optional arguments:
 *     column to limit search too (the column title in the table header)
 *     ifHidden - callback to execute if one or more elements was hidden
 *
 * If a user has a table where certain rows should not be filtered,
 * simply add a class of 'not_filtered' to those rows and they will
 * be skipped during filtering.
 */

(function($) {
  $.uiTableFilter = function(jq, phrase, column, ifHidden){
    var new_hidden = false;
    if( this.last_phrase === phrase ) return false;

    var phrase_length = phrase.length;
    var words = phrase.toLowerCase().split(" ");

    // these function pointers may change
    var matches = function(elem) { elem.show() }
    var noMatch = function(elem) { elem.hide(); new_hidden = true }
    var getText = function(elem) { return elem.text() }

    if( column ) {
      var index = null;
      jq.find("thead > tr:last > th").each( function(i){
        if( $.trim($(this).text()) == column ){
          index = i; return false;
        }
      });
      if( index == null ) throw("given column: " + column + " not found")

      getText = function(elem){ return $(elem.find(
        ("td:eq(" + index + ")")  )).text()
      }
    }

    // if added one letter to last time,
    // just check newest word and only need to hide
    if( (words.size > 1) && (phrase.substr(0, phrase_length - 1) ===
          this.last_phrase) ) {

      if( phrase[-1] === " " )
      { this.last_phrase = phrase; return false; }

      var words = words[-1]; // just search for the newest word

      // only hide visible rows
      matches = function(elem) {;}
      var elems = jq.find("tbody:first > tr:visible").not(".not_filtered")
    }
    else {
      new_hidden = true;
      var elems = jq.find("tbody:first > tr").not(".not_filtered")
    }

    elems.each(function(){
      var elem = $(this);
      $.uiTableFilter.has_words( getText(elem), words, false ) ?
        matches(elem) : noMatch(elem);
    });

    last_phrase = phrase;
    if( ifHidden && new_hidden ) ifHidden();
    return jq;
  };

  // caching for speedup
  $.uiTableFilter.last_phrase = ""

  // not jQuery dependent
  // "" [""] -> Boolean
  // "" [""] Boolean -> Boolean
  $.uiTableFilter.has_words = function( str, words, caseSensitive )
  {
    var text = caseSensitive ? str : str.toLowerCase();
    for (var i=0; i < words.length; i++) {
      if (text.indexOf(words[i]) === -1) return false;
    }
    return true;
  }
}) (jQuery);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

var filtertable = (function() {
    return {
        initialize : function() {
            var theTable = $('table.filter_table');
            var filter = $('#filter');

            filter.live('change, keyup', function(){
                $.uiTableFilter(theTable, this.value);
            });

            //override the submit so it doesn't try to push a form
            $('#filter_form').submit(function () {
                filter.keyup();
                return false;
            }).focus(); //Give focus to input field
            $('.filter_button').click(function(){filter.change()});
        }
    };
})();

$(document).ready(function() {
    // initialize the filter table
    filtertable.initialize();
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/



;

/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/
/**
 * This file is for use with the events subnav within systems page.
 * It is also used on the events subnav of the distributors page.
 */



KT.events = (function() {

    var total_events = 0,
    total_loaded = 0,
    actions_updater,
    loaded_summary = $('#loaded_summary'),
    more_button = $('#more'),
    page_size = 0,
    moreEvents = function() {
        var list = $('.events'),
        spinner = $('#list-spinner'),
        dataScrollURL = more_button.data("scroll_url") + "?offset=" + total_loaded +"&";
        spinner.fadeIn();
        $.ajax({
            type: "GET",
            url: dataScrollURL,
            cache: false,
            success: function(data) {
                spinner.fadeOut();
                list.append(data);
                $('#filter').keyup();
                $('.scroll-pane').jScrollPane().data('jsp').reinitialise();
                updateLoadedSummary();

                if (total_loaded === total_events) {
                    more_button.empty().remove();
                }
            },
            error: function() {
                spinner.fadeOut();
            }
        });
    },
    updateStatus = function(data) {
        // For each action that the user has initiated, update the status.
        var jobs = data["jobs"],
            tasks = data["tasks"];

        if (tasks) {
            $.each(tasks, function(index, status) {
                var node,
                    msg;

                if(!status["pending?"]) {
                    node = $('.event_name[data-pending-task-id=' + status['id'] + ']');
                    if(node !== undefined) {
                        node.parent().html(status["status_html"]);
                    }
                }
            });
        }

        if (jobs) {
            $.each(jobs, function(index, status) {
                var node,
                    msg;

                if(!status["pending?"]) {
                    node = $('.event_name[data-pending-job-id=' + status['id'] + ']');
                    if(node !== undefined) {
                        node.parent().html(status["status_html"]);
                    }
                }
            });
        }

        if(($('.event_name[data-pending-task-id]').length === 0) && ($('.event_name[data-pending-job-id]').length === 0)) {
            actions_updater.stop();
        }
    },
    startUpdater = function () {
        var timeout = 8000,
            pending_jobs = [],
            pending_tasks = [];

        $('.event_name[data-pending-job-id]').each(function(i) {
            pending_jobs[i] = $(this).data("pending-job-id");
        });
        $('.event_name[data-pending-task-id]').each(function(i) {
            pending_tasks[i] = $(this).data("pending-task-id");
        });

        if(pending_jobs.length > 0 || pending_tasks.length > 0) {
            if (actions_updater !== undefined) {
                actions_updater.stop();
            }
            actions_updater = $.PeriodicalUpdater($('.events').data('url'), {
                method: 'get',
                type: 'json',
                data: function() {return {task_id: pending_tasks, job_id: pending_jobs};},
                global: false,
                minTimeout: timeout,
                maxTimeout: timeout
            }, updateStatus);
        }
    },
    initEvents = function() {
        total_events = $('.events').data('total_events');
        if(total_events !== undefined) {
            total_events = parseInt(total_events, 10);
        } else {
            total_events = 0;
        }
        if(total_events > 0) {
            if(more_button !== undefined) {
                page_size = parseInt(more_button.data("page_size"),10);
                more_button.bind('click', moreEvents);
            } else {
                page_size = 0;
            }

            updateLoadedSummary();

            var  search_button = $('#event_filter_button'),
                    search_field = $('#event_search_filter'),
                spinner = $('#list-spinner');

            search_button.live('click', function(evt){
                evt.preventDefault();
                $.ajax({
                    type: "GET",
                    url: search_field.data('search_url'),
                    data: {search: search_field.val()},
                    cache: false,
                    success: function(data) {
                        spinner.fadeOut();
                        $("#event_items").html(data["html"]);
                        $('#filter').keyup();
                        $('.scroll-pane').jScrollPane().data('jsp').reinitialise();
                        updateLoadedSummary();
                        if (total_loaded === total_events) {
                            more_button.empty().remove();
                        }
                    },
                    error: function() {
                        spinner.fadeOut();
                    }
                });
            });
        }
        if(($('.event_name[data-pending-task-id]').length > 0) || ($('.event_name[data-pending-job-id]').length > 0)) {
            startUpdater();
        }
    },
    updateLoadedSummary = function() {
        var more_size = page_size;
        total_loaded = $('tr.tasks').length;
        loaded_summary.html(i18n.x_of_y(total_loaded, total_events));

        if(more_size > (total_events - total_loaded)) {
           more_size = total_events - total_loaded;
        }
        more_button.text(i18n.x_more(more_size));
    };
    return {
        initEvents: initEvents
    };
})();

$(document).ready(function() {
    KT.events.initEvents();
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/



;
