/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/



KT.panel.list.registerPage('system_groups', { create : 'new_system_group' });

$(document).ready(function() {
    KT.panel.set_expand_cb(function(){
        var children = $('#panel .menu_parent');
        $.each(children, function(i, item) {
            KT.menu.hoverMenu(item, { top : '75px' });
        });

        KT.system_groups.initialize();
        KT.system_groups.new_setup();
        KT.system_groups.details_setup();
        KT.system_groups.env_content_view_setup();
        KT.system_groups.selected_systems_setup();
    });
});


KT.sg_table = (function(){
    var add_system = function(html){
        var tbody =  $("#systems_table").find("tbody");
        tbody.prepend(html);
        tbody.find(".empty_row").hide();
        recalc_rows();
        KT.system_groups.refresh_list_item();
    },
    remove_system = function(ids){
        var table = $("#systems_table"),
            rows;
        $.each(ids, function(index, value){
            table.find("tr[data-id=" + value +"]").remove();
        });
        //show the empty message if there are no entries
        rows = table.find('tbody').find('tr').not('.empty_row');
        if(rows.length === 0){
            table.find(".empty_row").show();
        }
        recalc_rows();
        KT.system_groups.refresh_list_item();
    },
    recalc_rows = function(){
        /*
         * ensures that there are minimum number of rows in the table
         * for aesthetics
         */
        var min_rows = 12,
            tbody = $("#systems_table").find("tbody"),
            rows,
            missing;

        tbody.find('.stub').remove();
        rows = tbody.find('tr').not('.empty_row');
        missing = min_rows - rows.length;
        if (missing > 0){
            for(var i = 0; i < missing; i += 1){
                tbody.append("<tr class='stub'><td><br></td><td></td></tr>");
            }
        }
        rows = tbody.find('tr').not('.empty_row').not('.stub');
        var alt = 1;
        rows.each(function(index, value){
            if(alt % 2 === 0){
                $(value).addClass('alt');
            }
            else {
                $(value).removeClass('alt');
            }
            alt += 1;
        });

    };

    return {
        add_system: add_system,
        remove_system: remove_system,
        recalc_rows: recalc_rows
    };
}());

KT.system_groups = (function(){
    var current_system_input,
        current_max_systems,
        systems_deletable = false,
        selected_systems,
    refresh_list_item = function(){
        var id = $('#system_group_id');
        list.refresh(id.val(), id.data('ajax_url'));
    },
    quota_setup = function() {
        // quota_setup is used for both the 'new' and 'edit' panes.  While the logic is nearly the same
        // there are slight differences, since the 'edit' uses inline editing, but the 'new' does not.
        var unlimited = '-1',
            initial_max;

        if ($('system_group_new').length > 0) {
            // user is creating a group
            initial_max = $('#system_group_max_systems').val();
        } else {
            // user is editing a group
            initial_max = $('#system_group_max_systems').html();
        }
        current_max_systems = initial_max.length === 0 ? unlimited : initial_max;

        $('input.unlimited_members').unbind('click');
        $('input.unlimited_members').bind('click', function(){
            var max_systems_element = $('.limit'),
                max_systems = $('#system_group_max_systems');

            if($(this).is(":checked")){
                // user checked unlimited
                max_systems_element.hide();
                max_systems.val(unlimited);

                if (max_systems.hasClass('editable')) {
                    // user is editing an existing group
                    if (max_systems.val() !== current_max_systems) {
                        // user has changed the value since toggling unlimited on/off/on, so send request to server to set max_systems to unlimited
                        $.ajax({
                            type: "PUT",
                            url: max_systems.data("url"),
                            data: {system_group:{max_systems:unlimited}},
                            cache: false,
                            success: function(data) {
                                max_systems.html(i18n.clickToEdit); // reset the jeditable input
                                current_max_systems = unlimited;
                            },
                            error: function() {
                            }
                        });
                    }
                }
            } else {
                // user unchecked unlimited
                max_systems.val('');
                max_systems_element.show();

                if (max_systems.hasClass('editable')) {
                    // user is editing an existing group, send a click event to jeditable to open, so the user doesn't need to
                    max_systems.click();
                }
            }
        });
    },
    initialize = function(){
        $('.pane_action.remove').bind('click', prompt_to_destroy_group);
    },
    prompt_to_destroy_group = function(e) {
        e.preventDefault();
        KT.common.customConfirm({
            message: i18n.delete_system_group_confirm,
            yes_callback: function(){
                if (systems_deletable) {
                    // User selected "Yes", ask if they also want to delete the systems
                    prompt_to_destroy_systems();
                } else {
                    // User selected "No", they only want to delete the group
                    destroy_group();
                }
            }
        });
        return false;
    },
    prompt_to_destroy_systems = function() {
        KT.common.customConfirm({
            message: i18n.delete_systems_confirm,
            warning_message: i18n.delete_systems_warning,
            yes_callback: function(){
                // User selected "Yes"
                destroy_systems_and_group();
            },
            no_callback: function(){
                // User selected "No"
                destroy_group();
            },
            no_text: i18n.delete_system_group_continue,
            include_cancel: true
        });
        return false;
    },
    destroy_systems_and_group = function() {
        var id = $('#system_group_id').attr('name');
        $.ajax({
            type: "DELETE",
            url: KT.routes.destroy_systems_system_group_path(id),
            cache: false,
            success: function(data) {
                /*jslint evil: true */
                eval(data);
            }
        });
    },
    destroy_group = function() {
        var id = $('#system_group_id').attr('name');
        $.ajax({
            type: "DELETE",
            url: KT.routes.system_group_path(id),
            cache: false,
            success: function(data) {
                /*jslint evil: true */
                eval(data);
            }
        });
    },
    new_setup = function(){
        var pane = $("#system_group_new");
        if (pane.length === 0){
            return;
        }
        quota_setup();
    },
    details_setup = function(){
        var pane = $("#system_group_edit");
        if (pane.length === 0){
            return;
        }
        pane.find(".edit_name").each(function(){
            $(this).editable($(this).data("url"), {
                type        :  'text',
                width       :  250,
                method      :  'PUT',
                name        :  $(this).attr('name'),
                cancel      :  i18n.cancel,
                submit      :  i18n.save,
                indicator   :  i18n.saving,
                tooltip     :  i18n.clickToEdit,
                placeholder :  i18n.clickToEdit,
                submitdata  :  $.extend({ authenticity_token: AUTH_TOKEN }, KT.common.getSearchParams()),
                onsuccess   :  refresh_list_item
            });
        });
        pane.find(".edit_max_systems").each(function(){
            $(this).editable($(this).data("url"), {
                type        :  'text',
                width       :  250,
                method      :  'PUT',
                name        :  $(this).attr('name'),
                cancel      :  i18n.cancel,
                submit      :  i18n.save,
                indicator   :  i18n.saving,
                tooltip     :  i18n.clickToEdit,
                placeholder :  i18n.clickToEdit,
                submitdata  :  $.extend({ authenticity_token: AUTH_TOKEN }, KT.common.getSearchParams()),
                onsuccess   :  function(data){
                    $(this).html(data);
                    current_max_systems = data;
                },
                onresetcomplete :  function(settings, element){
                    // This event is invoked on cancel, esc and click outside of the element.
                    // When this occurs, if the user hasn't changed the value, reset to 'unlimited', hiding
                    // the limit element.
                    if ($(element).html() === i18n.clickToEdit) {
                        $('.limit').hide();
                        $('input.unlimited_members').attr('checked', 'checked');
                    }
                }
            });
        });
        quota_setup();
    },
    systems_setup = function(systems_deletable_perm){
        var pane = $("#system_group_systems");
        if (pane.length === 0){
            return;
        }
        systems_deletable = systems_deletable_perm;

        KT.sg_table.recalc_rows();
        $('#remove_systems').click(function(){
            var sys_ids = $('.system_checkbox:checked').map(function(){
                return $(this).data('id');
            }).get();
            if(sys_ids.length > 0){
                remove_systems(sys_ids, $(this));
            }
        });

        current_system_input = KT.auto_complete_box({
            values:       KT.routes.auto_complete_systems_path(),
            input_id:     "add_system_input",
            form_id:      "system_form",
            add_btn_id:   "add_system",
            selected_input_id: 'add_system_input_id',
            add_cb:       add_system
        });

        $("input.system_checkbox").unbind("change");
        $("input.system_checkbox").change(function() {
            var checked_systems = $("input.system_checkbox:checked");
            if (checked_systems.length === 0) {
                $("a.edit_systems").html(i18n.edit_all_systems);
            }
            else if (checked_systems.length > 0) {
                $("a.edit_systems").html(i18n.edit_selected_systems);
           }
        });
    },
    selected_systems_setup = function() {
        // This function will recheck any system checkboxes that were selected
        // before the systems pane was reloaded.  This is to handle the case
        // where user selects a group of systems, edits them and immediately
        // is returned to the systems list.
        var pane = $("#system_group_systems");
        if (pane.length === 0){
            return;
        }
        KT.utils.each(selected_systems, function(system) {
            var system_id = $(system).data('id');
            checkbox = $("input.system_checkbox[data-id=" + system_id + "]").prop('checked', true);
        });
        selected_systems = undefined;
    },
    add_system = function(string, item_id, cb){
        var grp_id = $("#system_group_systems").data('id'),
        add_funct = function(id){
            if(id){
                submit_change(grp_id, [id], true,
                    function(content){
                        KT.sg_table.add_system(content);
                        $("#add_system_input").val('');
                        cb();
                    },
                    function() {
                        // error_cb - handle scenario where error is returned from the server during 'add'
                        // reset the autocomplete input (e.g. remove the spinner, re-add add button..etc)
                        current_system_input.reset_input();
                        current_system_input.error();
                        cb();
                    });
            }
            else {
                current_system_input.error();
                cb();
            }
        };
        if (item_id) {
            add_funct(item_id);
        }
        else {
            //User did not select from the list, so we must search
            $.get(KT.routes.auto_complete_systems_path(), {term:string}, function(data){
                var found = false;
                $.each(data, function(index, element){
                    if (element.value === string){
                        found = element.id;

                        return false;
                    }
                });
                add_funct(found);
            });
        }

    },
    remove_systems = function(sys_ids, btn){
        var grp_id = $("#system_group_systems").data('id'),
            cleanup;

        btn.attr('disabled', 'true');
        $('.system_checkbox').attr('disabled', 'true');

        cleanup = function(){
            btn.removeAttr('disabled');
            $('.system_checkbox').removeAttr('disabled');
        };

        submit_change(grp_id, sys_ids, false,
            function(){
                KT.sg_table.remove_system(sys_ids);
                cleanup();
            },
            function(){
                cleanup();
            });
    },
    submit_change = function(grp_id, sys_ids, add, cb, error_cb){
      var url = add ? KT.routes.add_systems_system_group_path(grp_id) :
                        KT.routes.remove_systems_system_group_path(grp_id);
      $.post(url, {'system_ids':sys_ids}, cb).error(error_cb);
    },
    env_content_view_setup = function(){
        var pane = $(".env_content_view_selector");
        if (pane.length === 0){
            return;
        }

        KT.env_content_view_selector.initialize(
            {
                before_serialize_cb: add_selected_systems,
                save_success_cb: updated_env_content_view,
                override_save: true
            }
        );

        $('#update_form').unbind('submit');
        $('#update_form').submit(function(e) {
            var form = $(this);
            e.preventDefault();
            KT.common.customConfirm({
                message: i18n.change_systems_confirm,
                yes_callback: function(){
                    KT.env_content_view_selector.save(form);
                }
            });
        });
    },
    add_selected_systems = function(form) {
        // This will add the systems that are currently selected to the form provided.
        var selected_systems = $("input.system_checkbox:checked").clone().removeClass('system_checkbox').addClass('hidden');
        form.find('#content_view_id').after(selected_systems);
        return true;
    },
    updated_env_content_view = function (){
        // save the systems currently selected.  we'll use these to
        // reselect the systems after the pane reloads
        selected_systems = $("input.system_checkbox:checked");

        KT.panel.panelAjax('', KT.routes.systems_system_group_path($('#system_group_systems').data('id')) ,$('#panel'), true);
        KT.panel.closeSubPanel($('#subpanel'));
    };

    return {
        initialize: initialize,
        new_setup: new_setup,
        details_setup: details_setup,
        systems_setup: systems_setup,
        selected_systems_setup: selected_systems_setup,
        env_content_view_setup: env_content_view_setup,
        add_system : add_system,
        refresh_list_item: refresh_list_item
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

KT.system_group = KT.system_group || {};

KT.system_group.events = (function (){
    var init = function() {
        KT.tipsy.custom.tooltip('.tipsy-icon.task-info');
        KT.events.initEvents();
    };
    return {
        init: init
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.system = KT.system || {};

KT.system.errata = (function() {
    var errata_container,
        table_body,
        load_more,
        task_list = {},
        actions_updater,
        parent_id,  // e.g. system id, system group id
        items_url,
        install_url,
        status_url,
        errata_for,
        update_function,

        init = function(editable, page){
            errata_container = $('#errata_container');
            table_body = errata_container.find('tbody');
            load_more = $('#load_more_errata');

            // This component is shared by the systems and system groups pages; however, some of the behavior is
            // different depending on the page.  For example, the URLs to interact with the server.
            errata_for = page;
            parent_id = errata_container.data('parent_id');
            if (page === 'system') {
                items_url = KT.routes.items_system_errata_path(parent_id);
                install_url = KT.routes.install_system_errata_path(parent_id);
                status_url = KT.routes.status_system_errata_path(parent_id);
                update_function = update_status_for_system;
                KT.tipsy.custom.tooltip($('.tipsy-icon.errata-info'));
            } else {
                // errata_for === 'system_group'
                items_url = KT.routes.items_system_group_errata_path(parent_id);
                install_url = KT.routes.install_system_group_errata_path(parent_id);
                status_url = KT.routes.status_system_group_errata_path(parent_id);
                update_function = update_status_for_group;
                KT.tipsy.custom.tooltip($('.tipsy-icon.systems, .tipsy-icon.errata-info'));
            }

            register_events();

            // Not all users have permission to interact with errata; those that don't
            // cannot check their status either.
            if(editable) {
                init_status_check();
            }
            show_spinner(true);
            fetch_errata({ data : { clear_items : false }});
        },
        register_events = function(){
            $('#display_errata_type').bind('change', { clear_items : true }, fetch_errata);
            $('#select_all_errata').bind('change', select_all_errata);
            $('#errata_state_radio_applied').bind('change', fetch_errata);
            $('#errata_state_radio_outstanding').bind('change', fetch_errata);
        $('#run_errata_button').bind('click', add_errata);
        $('#select_all_button').bind('click', select_all);
        $('#deselect_all_button').bind('click', deselect_all);
            load_more.bind('click', { clear_items : false }, fetch_errata);
        },
        init_status_check = function(){
            var timeout = 8000;

            actions_updater = $.PeriodicalUpdater(status_url, {
                method: 'get',
                type: 'json',
                data: function() {return {id: Object.keys(task_list)};},
                global: false,
                minTimeout: timeout,
                maxTimeout: timeout
            }, update_function);
        },
        fetch_errata = function(event){
            var type = get_current_filter(),
                state = get_current_state(),
                offset,
                clear_items = event.data ? event.data.clear_items : false;

            if( clear_items ){
                insert_data({ "html" : "", "results_count" : 0, "total_count" : 0, "current_count" : 0}, false);
                show_spinner(true);
            }

            offset = clear_items ? 0 : $('#loaded_summary').data('current_count');

            $.ajax({
                method    : 'get',
                url        : items_url,
                data    : { filter_type : type, offset : offset, errata_state : state }
            }).success(function(data){
                insert_data(data, !clear_items);
                show_spinner(false);
            });
        },
        get_current_filter = function(){
            return $('#display_errata_type').val();
        },
        get_current_state = function(){
            return $('input[@name=errata_state_radio]:checked').val();
        },
        select_all_errata = function(){
            var checkboxes = table_body.find(':checkbox');

            if( $('#select_all_errata').attr('checked') ){
                checkboxes.attr('checked', true);
            } else {
                checkboxes.attr('checked', false);
            }
        },
        select_all = function(){
            var checkboxes = table_body.find(':checkbox');
            checkboxes.attr('checked', true);
        },
        deselect_all = function(){
            var checkboxes = table_body.find(':checkbox');
            checkboxes.attr('checked', false);
        },
        add_errata = function(){
            var selected_errata = errata_container.find(':checkbox:checked'),
                errata_ids = [],
                params = {};

            if ($('#run_errata_button').hasClass("disabled") || selected_errata.length === 0){
                return;
            }

            $.each(selected_errata, function(index, value){
                errata_ids.push($(value).val());
            });

            set_status(errata_ids, 'installing');

            params["errata_ids"] = errata_ids;
            params = $.param(params);

            $.ajax({
                url : install_url,
                type : 'POST',
                data : params
            }).success(function(data){
                if( Object.keys(task_list).length === 0 ){
                    actions_updater.restart();
                }
                task_list[data] = errata_ids;
                if(errata_for === 'system') {
                    set_status(errata_ids, 'installing', undefined);
                } else {
                    set_status(errata_ids, 'installing', KT.routes.system_group_event_path(parent_id, data));
                }
            });
        },
        update_status_for_system = function(data){
            var i = 0, length = data.length, task, url;

            for(i; i < length; i += 1){
                task = data[i];
                url = KT.routes.system_event_path(parent_id, task.id);

                if( task['state'] === 'finished' ){
                    set_status(task_list[task['id']], 'finished', url);
                    delete task_list[task['id']];
                } else if( task['state'] === 'running' ){
                    if( !task_list.hasOwnProperty(task['id']) ){
                        task_list[task['id']] = task['parameters']['errata_ids'];
                        set_status(task['parameters']['errata_ids'], 'installing', url);
                    }
                } else if( task['state'] === 'error' ){
                    set_status(task_list[task['id']], 'failed', url);
                    delete task_list[task['id']];
                }
            }

            if( Object.keys(task_list).length === 0 ){
                actions_updater.stop();
            }
        },
        update_status_for_group = function(data){
            var i = 0, num_jobs = data.length, job, num_tasks, task, url;

            for(i; i < num_jobs; i += 1){
                job = data[i];
                num_tasks = job.tasks.length;
                url = KT.routes.system_group_event_path(parent_id, job.id);

                var j = 0, running = 0, error = 0;
                for(j; j < num_tasks; j += 1){
                    task = job.tasks[j];
                    if ((task.state === 'waiting') || (task.state === 'running')) {
                        running += 1;
                    } else if (task.state === 'error') {
                        error += 1;
                    }
                }
                if (running > 0) {
                    // still have a task running... status should remain installing
                    if( !task_list.hasOwnProperty(job.id) ){
                        task_list[job.id] = job['parameters']['errata_ids'];
                        set_status(task_list[job.id], 'installing', url);
                    }
                } else if (error > 0) {
                    // no tasks are still running, but at least 1 error was encountered
                    set_status(task_list[job.id], 'failed', url);
                    delete task_list[job.id];
                } else {
                    // looks like all tasks completed...
                    set_status(task_list[job.id], 'finished', url);
                    delete task_list[job.id];
                }
            }

            if( Object.keys(task_list).length === 0 ){
                actions_updater.stop();
            }
        },
        insert_data = function(data, append){
            var html = data["html"];

            if(data.total_count > 0){
                $('#run_errata_button').removeClass("disabled");
            }
            if( append ){
                table_body.append(html);
            } else {
                table_body.html(html);
            }
            update_counts(data);
        },
        update_counts = function(data){
            var current_count = data["current_count"],
                total_count = data["total_count"],
                results_count = data["results_count"];

            $('#loaded_summary').data('current_count', current_count),
            $('#loaded_summary').html(i18n.x_of_y_total(current_count, results_count, total_count));

            if( current_count === results_count ){
                load_more.hide();
            } else {
                load_more.show();
            }
        },
        show_spinner = function(show){
            if( show ){
                $('#list-spinner').show();
            } else {
                $('#list-spinner').hide();
            }
        },
        set_status = function(errata_ids, status, url){
            var rows = get_rows(errata_ids),
                errata_row, i, length;

            length = rows.length;

            for( i = 0; i < length; i += 1){
                errata_row = rows[i];

                if( status === 'installing' ){
                    errata_row.find('.errata_status_text').html(get_status_block(i18n.installing, url));
                    errata_row.find('img').show();
                } else if( status === 'finished' ){
                    errata_row.find('img').hide();
                    errata_row.find('.errata_status_text').html(get_status_block(i18n.install_finished, url));
                } else if ( status === 'failed' ) {
                    errata_row.find('img').hide();
                    errata_row.find('.errata_status_text').html(get_status_block(i18n.install_error, url));
                }
                errata_row.find('.errata_status').show();
            }
        },
        get_status_block = function(status, url){
            if(url === undefined) {
                return status;
            } else {
                var html = '<a data-url="' + url + '" class="subpanel_element">' + status + '</a>';
                return html;
            }
        },
        get_rows = function(errata_ids){
            var i = 0,
                length = errata_ids.length,
                rows = [];

            for( i; i < length; i += 1){
                rows.push($('#errata_' + KT.common.escapeId(errata_ids[i])));
            }

            return rows;
        };

    return {
        init    : init
    };

})();

// Call this init() from a location where the 'editable' flag can be set
// appropriately. In this case it is called from errata/_index.html.haml
//$(document).ready(function() {
//    KT.system.errata.init(editable);
//});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.packages = (function() {
    var valid_package_list_format = function(packages){
        var length = packages.length;

        for (var i = 0; i < length; i += 1){
            if( !valid_package_name(packages[i]) ){
                return false;
            }
        }
        return true;
    },
    valid_package_name = function(package_name){
        var is_match = package_name.match(/[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789\-\.\_\+\,]+/);

        return is_match === null ? true : false;
    };
    return {
        valid_package_list_format : valid_package_list_format,
        valid_package_name : valid_package_name
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


KT.package_action_types = (function() {
    return {
        PKG : "pkg",
        PKG_INSTALL : "pkg_install",
        PKG_UPDATE : "pkg_update",
        PKG_REMOVE : "pkg_remove",
        PKG_GRP : "pkg_grp",
        PKG_GRP_INSTALL : "pkg_grp_install",
        PKG_GRP_UPDATE : "pkg_grp_update",
        PKG_GRP_REMOVE : "pkg_grp_remove"
    };
})();

KT.package_actions = (function() {
    var packages_container,
    content_add_url,
    content_update_url,
    content_remove_url,
    status_url,
    packages_top,
    content_form,
    content_input,
    add_content_button,
    update_content_button,
    update_all_content_button,
    remove_content_button,
    error_message,
    add_row_shading = true,
    actions_updater,
    actions_updater_running = false,

    init = function(editable, page) {
        packages_container = $('#packages_container');
        packages_top = $('.packages').find('tbody');
        content_form = $('#content_form');
        content_input = $('#content_input');
        add_content_button = $('#add_content');
        update_content_button = $('#update_content');
        update_all_content_button = $('#update_all_content');
        remove_content_button = $('#remove_content');
        error_message = $('#error_message');

        // This component could be shared by multiple pages (e.g. systems & system groups); however, some of the
        // behavior is different depending on the page.  For example, the URLs to interact with the server.
        packages_for = page;
        var id = packages_container.data('parent_id');
        if (packages_for === 'system') {
            // TODO: update for system packages when it is used
            var todo = 1;
        } else {
            // packages_for === 'system_group'
            content_add_url = KT.routes.add_system_group_packages_path(id);
            content_update_url = KT.routes.system_group_packages_path(id);
            content_remove_url = KT.routes.remove_system_group_packages_path(id);
            status_url = KT.routes.status_system_group_packages_path(id);
        }

        disableContentButtons();
        enableUpdateAll();

        content_input.bind('change, keyup', refreshButtons);
        content_input.bind('keypress', function(event) {
            // if the user presses enter, ignore it... do not submit the form...
            if( event.which === 13) {
                event.preventDefault();
            }
        });

        $('.perform_action').change(refreshButtons);
        KT.tipsy.custom.system_packages_tooltips();
    },
    refreshButtons = function() {
        if (content_input.val().length === 0) {
            disableContentButtons();

            var selected_action = $("input[name=perform_action]:checked").attr('id');
            if (selected_action === 'perform_action_packages') {
                enableUpdateAll();
            } else {  // perform_action_package_groups
                disableUpdateAll();
            }

        } else {
            disableUpdateAll();
            enableContentButtons();
        }
    },
    disableContentButtons = function() {
        add_content_button.unbind().attr('disabled', 'disabled').addClass('disabled');
        update_content_button.unbind().attr('disabled', 'disabled').addClass('disabled');
        remove_content_button.unbind().attr('disabled', 'disabled').addClass('disabled');
    },
    enableContentButtons = function() {
        if (add_content_button.hasClass('disabled')) {
            add_content_button.unbind().bind('click', addContent).removeAttr('disabled').removeClass('disabled');
            add_content_button.bind('keypress', function(event) {
                if( event.which === 13) {
                    event.preventDefault();
                    KT.package_actions.addContent(event);
                }
            });
        }

        if (update_content_button.hasClass('disabled')) {
            update_content_button.unbind().bind('click', updateContent).removeAttr('disabled').removeClass('disabled');
            update_content_button.bind('keypress', function(event) {
                if( event.which === 13) {
                    event.preventDefault();
                    KT.package_actions.updateContent(event);
                }
            });
        }

        if (remove_content_button.hasClass('disabled')) {
            remove_content_button.unbind().bind('click', removeContent).removeAttr('disabled').removeClass('disabled');
            remove_content_button.bind('keypress', function(event) {
                if( event.which === 13) {
                    event.preventDefault();
                    KT.package_actions.removeContent(event);
                }
            });
        }
    },
    enableUpdateAll = function() {
        update_all_content_button.unbind().bind('click', updateContent).removeAttr('disabled').removeClass('disabled');
        update_all_content_button.bind('keypress', function(event) {
            if( event.which === 13) {
                event.preventDefault();
                KT.package_actions.updateContent(event);
            }
        });
    },
    disableUpdateAll = function() {
        update_all_content_button.unbind().attr('disabled', 'disabled').addClass('disabled');
    },
    addContent = function(data) {
        data.preventDefault();
        performAction(content_add_url);
    },
    updateContent = function(data) {
        data.preventDefault();
        performAction(content_update_url);
    },
    removeContent = function(data) {
        data.preventDefault();
        performAction(content_remove_url);
    },
    performAction = function(url) {
        var selected_action = $("input[name=perform_action]:checked").attr('id'),
            content_string = content_form.find('#content_input').val(),
            content_array = content_string.split(/ *, */),
            content,
            validation_error;

        if (selected_action === 'perform_action_packages') {
            validation_error = validate_action_requested(content_array, KT.package_action_types.PKG);
            content = {'packages':content_string};
        } else {
            // perform_action_package_groups
            validation_error = validate_action_requested(content_array, KT.package_action_types.PKG_GRP);
            content = {'groups':content_string};
        }

        if (validation_error === undefined) {
            disableContentButtons();
            disableUpdateAll();
            $.ajax({
                url: url,
                type: 'PUT',
                data: content,
                cache: false,
                success: function(data) {
                    $(data).each( function(index, element) {
                        // The response will have the html to be rendered for the action scheduled, which
                        // includes 1 row for each package or group included in the action.
                        if ($(element).is('tr')) {
                            var name = $(element).find('td').data('name'),
                                existing;

                            // check to see if there was an previous/existing action for the package or group
                            if ($(element).hasClass('package')) {
                                existing = $("tr.package > td[data-name='"+name+"']");
                            } else {
                                existing = $("tr.group > td[data-name='"+name+"']");
                            }

                            if (existing.length > 0) {
                                // there was an action, so just update the status on the existing row
                                existing.next('td.status').replaceWith($(element).find('td.status'));
                            } else {
                                // there was no action, so render the new row
                                if (add_row_shading) {
                                    add_row_shading = false;
                                    $(element).addClass('alt');
                                } else {
                                    add_row_shading = true;
                                }
                                packages_top.prepend(element);
                            }
                        }
                    });
                    refreshButtons();
                    show_validation_error(false);
                    startUpdater();  // ensure polling is enabled for updates on the action
                },
                error: function() {
                    refreshButtons();
                }
            });
        } else {
            show_validation_error(true, validation_error);
        }
    },
    validate_action_requested = function(content, content_type) {
        // validate the action being requested and return a validation error, if an error is found
        var validation_error;

        // validate the package list format
        if ((content_type === KT.package_action_types.PKG) && !KT.packages.valid_package_list_format(content)) {
            validation_error = i18n.validation_error_name_format;

        // validate that no actions pending on same package or group
        } else {
            var item;
            $.each(content, function(index, content_item) {
                var status;
                item = $.trim(content_item);
                switch (content_type) {
                    case KT.package_action_types.PKG:
                        // look for an existing node for the package, where there is an action pending
                        status = $("tr.package > td[data-name='"+item+"'] + td.status[data-pending-action-id]");
                        if (status.length > 0) {
                            validation_error = i18n.validation_error_package_pending;
                        }
                        break;
                    case KT.package_action_types.PKG_GRP:
                        // look for an existing node for the group, where there is an action pending
                        status = $("tr.group > td[data-name='"+item+"'] + td.status[data-pending-action-id]");
                        if (status.length > 0) {
                            validation_error = i18n.validation_error_group_pending;
                        }
                        break;
                }
            });
        }
        return validation_error;
    },
    show_validation_error = function(show, validation_error){
        var input = content_form.find('#content_input');
        if( show ){
            input.addClass('validation_error_input');
            error_message.html(validation_error);
            error_message.show();
        } else {
            input.removeClass('validation_error_input');
            error_message.hide();
        }
    },
    startUpdater = function () {
        if (!actions_updater_running) {
            var timeout = 8000;

            actions_updater = $.PeriodicalUpdater(status_url, {
                method: 'get',
                type: 'json',
                data: function() {return {id: getActionsInProgress()};},
                global: false,
                minTimeout: timeout,
                maxTimeout: timeout
            }, updateStatus);

            actions_updater_running = true;
        }
    },
    updateStatus = function(data) {
        // update the status for each action retrieved
        $.each(data, function(index, status) {
            var node = $('td.status[data-pending-action-id=' + status['id'] + ']');
            if(node !== undefined) {
                if (status['status_html'] !== undefined) {
                    node.replaceWith(status['status_html']);
                }
            }
        });
        if (getActionsInProgress().length === 0) {
            actions_updater.stop();
            actions_updater_running = false;
        }
    },
    getActionsInProgress = function() {
        var pending_actions = [];
        $('td.status[data-pending-action-id]').each(function(i, element) {
            pending_actions[i] = $(element).data("pending-action-id");
        });
        return pending_actions;
    };
    return {
        init: init,
        addContent: addContent,
        removeContent: removeContent,
        updateContent: updateContent
    };
})();

/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

KT.auto_complete_box = function(params) {

    var settings = {
        values: undefined, //either a url, an array, or a callback of items for auto_completion
        default_text: undefined, //default text to go into the search box if desired
        comma_separated_input: false,
        input_id: undefined,
        input: undefined,
        selected_input_id: undefined,
        selected_input: undefined,
        form_id: undefined,
        add_btn_id: undefined,
        add_btn: undefined,
        add_text: i18n.add_plus,
        require_select: false, //disable the add button if proper autocompleted item isn't selected
        add_cb: function(item, item_id, cb){cb();}
    };
    $.extend( settings, params );

    var add_item_from_input = function(e) {
        var item = get_input().attr("value"),
            item_id = get_selected_input().val();


        e.preventDefault();
        if (item.length === 0 || item === settings.default_text){
                return;
        }
            get_add_btn().addClass("working");
        add_item_base(item, item_id, true);
    },
    get_input = function(){
        return settings.input || $("#" + settings.input_id);
    },
    get_selected_input = function(){
        return settings.selected_input || $("#" + settings.selected_input_id);
    },
    get_add_btn = function(){
        return settings.add_btn || $("#" + settings.add_btn_id);
    },
    add_item_base = function(item, item_id, focus) {
        var input = get_input(),
            add_btn = get_add_btn();

        input.removeClass("input_error");
        add_btn.addClass("working");
        add_btn.html("<img  src='assets/icons/spinner.gif'>");
        input.attr("disabled", "disabled");
        input.autocomplete('disable');
        input.autocomplete('close');

        if (settings.comma_separated_input) {
            // convert the item string to an array
            item = split(item);
        }
        settings.add_cb(item, item_id, function(){
            reset_input();
            if (focus) {
                get_input().focus();
            }
        });
    },
    reset_input = function() {
        //re-lookup all items, since a redraw may have happened
        var input = get_input(),
            add_btn = get_add_btn();
        add_btn.removeClass('working');
        if (add_btn.text() === "") {
            add_btn.html(settings.add_text);
        }
        input.removeAttr('disabled');
        input.autocomplete('enable');
        input.val('');
    },
    manually_add = function(item, item_id) {
        add_item_base(item, item_id, false);
    },
    error = function() {
        var input = get_input();
        input.addClass("input_error");
    },
    split = function(val) {
        return val.split(/,\s*/);
    },
    extractLast = function(term) {
        return split(term).pop();
    };

    //initialization
    var input = get_input(),
        form = $("#" + settings.form_id),
        add_btn = get_add_btn();

    if (settings.default_text) {
        input.val(settings.default_text);
        input.focus(function() {
            if (input.val() === settings.default_text) {
                input.val("");
            }
        });
        input.blur(function() {
            if(input.val() === "") {
                input.val(settings.default_text);
            }
        });
    }

    if (!settings.comma_separated_input) {
        input.autocomplete({
            source: function (request, response) {
                $.ajax({
                    url: settings.values,
                    data: request,
                    dataType: 'json',
                    success: function (data) {
                        response(data);
                        KT.utils.each(data, function(item){
                            if(request.term === item.value ||
                               request.term === item.label)
                            {
                                get_selected_input().val(item.id);
                                add_btn.removeAttr("disabled");
                            }
                        });
                    },
                    error: function () {
                        response([]);
                    }
                });
            },
            search: function(){
                get_selected_input().val('');
                if(settings.require_select){
                    add_btn.attr("disabled", "disabled");
                }
            },
            select: function (event, ui) {
                get_input().val(ui.item.value);
                get_selected_input().val(ui.item.id);
                add_btn.removeAttr("disabled");
                return false;
            }
        });
    } else {
        input.autocomplete({
            source: function(request, response) {
                $.getJSON( settings.values, {
                    term: extractLast(request.term)
                }, response);
            },
            search: function () {
                // custom minLength
                var term = extractLast(this.value);
                if (term.length < 1) {
                    return false;
                }
            },
            focus: function () {
                // prevent value inserted on focus
                return false;
            },
            select: function (event, ui) {
                var terms = split(this.value);
                // remove the current input
                terms.pop();
                // add the selected item
                terms.push(ui.item.value);
                // add placeholder to get the comma-and-space at the end
                terms.push("");
                this.value = terms.join(", ");
                return false;
            }
        });
    }
    add_btn.bind('click', add_item_from_input);
    form.submit(add_item_from_input);

    return {
        manually_add: manually_add,
        error: error,
        reset_input: reset_input
    };
};


/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/







;
