/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


/*
 * Katello Global JavaScript File
 * Author: @jrist
 * Date: 09/01/2010
 */

//Katello global object namespace that all others should be attached to
var KT = {};
KT.widget = {};

KT.utils = _.noConflict();

// load angular module to Katello
var Katello = angular.module('Katello', ['alchemy', 'alch-templates', 'ngSanitize']);

// Must be at the top to prevent AngularJS unnecessary digest operations
// And to handle the hashPrefix that AngularJS adds that confuses BBQ
$(window).bind("hashchange", function(event) {
// refresh the favicon to make sure it shows up
    $('link[type*=icon]').detach().appendTo('head');
});

if ($.bbq !== undefined) {
    $.bbq.pushState('!', '');
}

//i18n global variable
var i18n = {};

function localize(data) {
    for (var key in data) {
        if(data.hasOwnProperty(key)) {
            i18n[key] = data[key];
        }
    }
}

function update_status() {
  var statusElement = $(".status");
  var i = setInterval(function() {
      $.ajax({
          url: "#{@_request.fullpath}",
          dataType: 'json',
          success: function (json, status, xhr) {
              statusElement.text(json.status);
              if (xhr.status === 200) {
                  clearInterval(i);
              }
          },
          error: function (xhr, status, error) {
              statusElement.text(jQuery.parseJSON(xhr.responseText).message);
              clearInterval(i);
          }
      });
  }, 1000);
}

// Common functionality throughout Katello

// Simple function to dump a message to the browser error log
function log(msg) {
    setTimeout(function() {
        throw new Error(msg);
    }, 0);
}

// Workaround for a bug in chosen
// When a chosen select receives focus() invoked from js code
// it ends up in an infinite loop of displaying and hiding the options.
// 2pane gives automatically focus to a first enabled visible input on the panel.
// Therefore we disable the inner input at the beginning and enable it
// after 400ms.
//
// requires jQuery
$.fn.delayed_chosen = function(options, delay_time) {
    var chzn_input;

    $(this).chosen(options);

    delay_time = (delay_time === undefined) ? 400 : delay_time;
    chzn_input = $(this).parent().find('.chzn-container :input');
    chzn_input.prop('disabled', true);
    chzn_input.delay(delay_time).queue(function() {
        $(this).prop('disabled', false);
        $(this).dequeue();
    });
};

//requires jQuery
KT.getData = function(fieldNames) {
    var data = {},
        value;

    $.each(fieldNames, function(i, fieldName){
        value = $('#'+fieldName).val();
        if (value !== undefined) {
            data[fieldName] = value;
        }
    });
    return data;
};


KT.helptip =  (function($) {
    var enable = function(key, url) {
        $.ajax({
          type: "POST",
          url: url,
          data: { "key":key},
          cache: false
          });
        },
        disable = function(key, url) {
          $.ajax({
            type: "POST",
            url: url,
            data: { "key":key},
            cache: false
           });
        },
        handle_close = function(){
          var key = this.id.split("helptip-opened_")[1],
              url = $(this).attr('data-url');

          $("#helptip-opened_" + key).hide();
          $("#helptip-closed_" + key).show();

          $(document).trigger('helptip-closed');

          disable(key, url);
        },
        handle_open = function(){
          var key = this.id.split("helptip-closed_")[1],
              url = $(this).attr('data-url');

          $("#helptip-opened_" + key).show();
          $("#helptip-closed_" + key).hide();

          $(document).trigger('helptip-opened');

          enable(key, url);
        };

    return {
        handle_close    :    handle_close,
        handle_open     :    handle_open
    };
})(jQuery);


//Add backwards compatible version of Object.keys
// https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Object/keys
if(!Object.keys) {
    Object.keys = function(o){
     if (o !== Object(o)) {
        throw new TypeError('Object.keys called on non-object');
     }
     var ret=[],p;
     for(p in o) {
       if(Object.prototype.hasOwnProperty.call(o,p)){
         ret.push(p);
       }
     }
     return ret;
  };
}


//override the jQuery UJS $.rails.allowAction
$.rails.allowAction = function(element) {
    var message = element.data('confirm'),
    answer = false, callback;
    if (!message) { return true; }

    if ($.rails.fire(element, 'confirm')) {
        KT.common.customConfirm({
            message: message,
            yes_callback: function() {
                callback = $.rails.fire(element, 'confirm:complete', [answer]);
                if(callback) {
                    var oldAllowAction = $.rails.allowAction;
                    $.rails.allowAction = function() { return true; };
                    element.trigger('click');
                    $.rails.allowAction = oldAllowAction;
                }
            }
        });
    }
    return false;
};

//make jQuery Contains case insensitive
$.expr[':'].Contains = function(a, i, m) {
  return $(a).text().toUpperCase().indexOf(m[3].toUpperCase()) >= 0;
};
$.expr[':'].contains = function(a, i, m) {
  return $(a).text().toUpperCase().indexOf(m[3].toUpperCase()) >= 0;
};

//requires jQuery
KT.common = (function() {
    var root_url;
    return {
        height: function() {
            return $(window).height();
        },
        width: function() {
            return $(window).width();
        },
        scrollTop: function() {
            return $(window).scrollTop();
        },
        scrollLeft: function() {
            return $(window).scrollLeft();
        },
        decode: function(value){
            var decoded = decodeURIComponent(value);
            return decoded.replace(/\+/g, " ");
        },
        escapeId: function(myid) {
            return myid.replace(/([ #;&,.%+*~\':"!\^$\[\]()=>|\/])/g,'\\$1');
        },
        customConfirm : function (params) {
          var settings = {
              message: undefined,
              warning_message: undefined,
              yes_text: i18n.yes,
              no_text: i18n.no,
              yes_callback: function(){},
              no_callback: function(){},
              include_cancel: false
          },
          confirmTrue = true,
          confirmFalse = false;

          $.extend(settings, params);

          var message = "<div style='margin:20px;'><span class='status_confirm_icon'/><div style='margin-left: 24px; display:table;height:1%;'>" + settings.message + "</div></div>",
              warning_message = (settings.warning_message === undefined) ? undefined : "<div style='margin:20px;'><span class='status_warning_icon'/><div style='margin-left: 24px; display:table;height:1%;color:red;'>" + settings.warning_message + "</div></div>",
              html = (warning_message === undefined) ? message : "<div>"+message+warning_message+"</div>",
              buttons = {
                "Yes": {
                  click : function () {
                    $(this).dialog("close");
                    $(this).dialog("destroy");
                    settings.yes_callback();
                    return confirmTrue;
                  },
                  'class' : 'button',
                  'text' : settings.yes_text
                },
                "No": {
                  click:function () {
                    $(this).dialog("close");
                    $(this).dialog("destroy");
                    settings.no_callback();
                    return confirmFalse;
                  },
                  'class' : 'button',
                  'text' : settings.no_text
                }
              };

          if(settings.include_cancel === true) {
              buttons["Cancel"] = {
                click:function () {
                  $(this).dialog("close");
                  $(this).dialog("destroy");
                  return confirmFalse;
                },
                'class' : 'button',
                'text' : i18n.cancel
              };
          }

          $(html).dialog({
            closeOnEscape: true,
            open: function (event, ui) {
                $('.ui-dialog-titlebar-close').hide();
                $('.confirmation').find('.ui-button')[1].focus();
            },
            modal: true,
            resizable: false,
            width: 450,
            title: i18n.confirmation,
            dialogClass: "confirmation",
            buttons: buttons
          });
        },
        customAlert : function(message) {
          var html = "<div style='margin:20px;'><span class='status_exclamation_icon'/><div style='margin-left: 24px; display:table;height:1%;'>" + message + "</div></div>";
          $(html).dialog({
            closeOnEscape: true,
            open: function (event, ui) { $('.ui-dialog-titlebar-close').hide(); },
            modal: true,
            resizable: false,
            width: 300,
            title: i18n.alert,
            dialogClass: "alert",
            stack: false,
            buttons: {
                "Ok": {
                  click : function () {
                    $(this).dialog("close");
                    $(this).dialog("destroy");
                    return false;
                  },
                  'class' : 'button',
                  'text' : i18n.ok
                }
            }
          });
        },
        orgSwitcherSetup : function() {
             $('.favorite').live('click', function(e) {
                KT.orgswitcher.checkboxChanged($(this).parent().find('.default_org'));
            });
        },
        rootURL : function() {
            if (root_url === undefined) {
                //root_url = $('#root_url').attr('data-url');
                root_url = KT.config['root_url'];
            }
            return root_url;
        },
        getSearchParams : function(val) {
            var search_string;

            if ($.bbq) {
                search_string = $.bbq.getState('list_search');
            }

            if( search_string ){
                return { 'search' : search_string };
            } else {
                return false;
            }
        },
        spinner_path : function() {
          return KT.common.rootURL() + "assets/icons/spinner.gif";
        },
        jscroll_init: function(element) {
            element.jScrollPane({ hideFocus: true });
        },
        jscroll_resize: function(element) {
            element.resize(function(event){
                var element = $('.scroll-pane');
                if (element.length){
                    element.data('jsp').reinitialise();
                }
            });
        },
        to_human_readable_bytes : function(bytes) {
            var sizes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB'],
                i;

            if (bytes === 0) {
                return '0';
            } else {
                i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)), 10);
                return ((i === 0) ? (bytes / Math.pow(1024, i)) : (bytes / Math.pow(1024, i)).toFixed(1)) + ' ' + sizes[i];
            }
        },
        icon_hover_change : function(element, shade){
            var background_position,
                icon = element.find('i[data-change_on_hover="' + shade + '"]'),
                shade_position;

            if( icon.length > 0 ){
                background_position = icon.css('background-position');

                if( background_position !== undefined ){
                    background_position = background_position.split(" ");

                    shade_position = (background_position[1] === "0" || background_position[1] === "0px") ? ' -16px' : ' 0';
                    background_position = background_position[0] + shade_position;

                    icon.css({ 'background-position' : background_position });
                }
            }
        },
        link_hover_setup : function(shade){
            $('a').live('mouseenter',
                function(){ KT.common.icon_hover_change($(this), shade); }
            ).live('mouseleave',
                function(){ KT.common.icon_hover_change($(this), shade); }
            );
        }
    };
})(jQuery);

KT.orgswitcher = (function($) {
    var checkboxChanged = function(checkbox) {
      var this_checkbox = $(checkbox);
      this_checkbox.attr("disabled", "disabled");
      //var for all favorite icons to clear them
      var all_favorites = $('.favorite');
      //current favorite
      var this_favorite = this_checkbox.parent().find('.favorite');
      var this_spinner = this_checkbox.parent().find('.fav_spinner');
      var name = this_checkbox.attr("name");

      //extract the URL for the preference change
      var url = checkbox.data("url");

      var selected_org_id = this_checkbox.attr("value");
      var checked = this_checkbox.attr("checked");
      var options = {};

      if (checked){
        options = {user_id : $('#user_id').data("user_id")};
      } else {
        options = {org : selected_org_id, user_id : $('#user_id').data("user_id")};
      }

      //hide the favorite icon temporarily while the ajax operation occurs
      this_favorite.hide();

      //show the spinner while waiting
      this_spinner.removeClass('hidden').show();

      $.ajax({
          type: "PUT",
          url: url,
          data: options,
          cache: false,
          success: function(data, textStatus, jqXHR){
            //hide spinner
            this_spinner.addClass('hidden').hide();
            if(checked){
              this_checkbox.attr("checked", false);
              this_favorite.addClass("favorites_icon-grey");
              this_favorite.removeClass("favorites_icon-black");
              this_favorite.attr("title", i18n.make_default_org);
              if(this_favorite.parent().find('label').length){
                this_favorite.parent().find('label').html(i18n.make_default_org);
              }
            } else {
              this_checkbox.attr("checked", true);
              all_favorites.removeClass("favorites_icon-black");
              all_favorites.attr("title", i18n.make_default_org);
              $('.favorite').addClass("favorites_icon-grey");
              this_favorite.removeClass("favorites_icon-grey").addClass("favorites_icon-black");
              this_favorite.attr("title", i18n.current_default_org);
              if(this_favorite.parent().find('label').length){
                this_favorite.parent().find('label').html(i18n.current_default_org);
              }
            }
            this_favorite.show();

            this_checkbox.removeAttr("disabled");
          },
          error: function(data, textStatus, jqXHR){
            //hide the spinner and show the favorite is not selected
            this_spinner.addClass('hidden').hide();
            this_checkbox.attr("checked", false);
            this_favorite.show();
            this_checkbox.removeAttr("disabled");
          }
      });
      return false;
    };
    return {
        checkboxChanged: checkboxChanged
    };
}(jQuery));

var client_common = {
    create: function(data, url, on_success, on_error) {
      $.ajax({
        type: "POST",
        url: url,
        data: data,
        cache: false,
        success: on_success,
        error: on_error
      });
    },
    destroy: function(url, on_success, on_error) {
      $.ajax({
        type: "DELETE",
        url: url,
        cache: false,
        success: on_success,
        error: on_error
      });
    }
};

/**
 * Document Ready function
 */
$(document).ready(function (){
    KT.common.link_hover_setup('dark');

    //Add a handler so that if any input has focus
    //   our keyboard shortcuts don't steal it
    $(":input").focus(function() {
        onInputField = true;
    }).blur(function() {
        onInputField = false;
    });

    //Add a handler for helptips
    $(".helptip-open").live('click', KT.helptip.handle_close);
    $(".helptip-close").live('click', KT.helptip.handle_open);

    // Add a handler for ellipsis
    $(".one-line-ellipsis").ellipsis(true);
    $(".tipsify").tipsy({ live : true, gravity: 's', fade: true, delayIn : 350 });
    $(".tipsify-west").tipsy({ gravity: 'w', hoverable : 'true' });

    KT.tipsy.custom.enable_forms_tooltips();

    KT.common.orgSwitcherSetup();
});

/**
 * Window Ready function
 */
$(window).ready(function(){
    $('.fc').parent().css({"text-align":"center"});
    //all purpose display loading icon for ajax calls
    $("#loading").bind("ajaxSend", function(){
      $(this).show();
      $('body').css('cursor', 'wait');
    }).bind("ajaxComplete", function(){
      $(this).hide();
      $('body').css('cursor', 'default');
    });
    $().UItoTop({ easingType: 'easeOutQuart' });

    //allow all buttons with class .button to be clicked via enter or space button
    $('.button').live('keyup', function(e){
        if(e.which === 13 || e.which === 32)
        {
            $(this).click();
        }
    });

    window.alert = function(message){KT.common.customAlert(message);return false;};
    $.rails.confirm = function(message) {
        KT.common.customConfirm({message: message}); return false;
    };
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 */

/*
 * Katello JavaScript Menu File
 * Author: jrist
 * Date: 10/25/11
 * Time: 10:19 AM
 */

/**
 * Document Ready function
 */

$(document).ready(function (){
  KT.menu.menuSetup();
});

KT.menu = (function(){
  return {
    menuSetup: function(){
      //set some useful vars
      var topLevel = $('#appnav li.top_level');
      var secondLevel = $('#subnav').find('ul');
      var activeSubnav = secondLevel.find(".selected").first().parent();
      var subnav = $('#subnav');

      var activeTab = topLevel.filter('.selected:visible');

      //hide the secondlevel before prepending it to subnav
      secondLevel.not(activeSubnav).hide();
      //set the current tab to active so we can check it later
      activeTab.addClass('active');
      activeTab = topLevel.filter('.active');

      //append the arrow icon to all menus with a third level
      var parents = $('li.menu_parent');
      parents.prepend($('<div class="arrow_icon_menu"></div>'));
      parents.each(function(i, a_parent){
        var this_parent = $(a_parent);
        var parent_menu_name = this_parent.data('dropdown');
        $("li[dropdown='" + parent_menu_name +"']").first().parent().appendTo(this_parent);
        KT.menu.hoverMenu(a_parent);
      });


      //some settings for the hoverable top level tabs
      var hoverSettings = {
        sensitivity: 2,
        interval: 50,
        timeout: 350,
        over: function(){
          $(this).trigger("open");
        },
        out: function(){
          $(this).trigger("close.menu");
        }
      };

      //for each top level menu item tab, attach a hoverIntent event and bind another event
      topLevel.each(function(){
        var topLevelTab = $(this);
        var tabType = topLevelTab.attr('data-menu');
        var currentSubnav = subnav.find('.' + tabType + '.second_level').parent();
        var enter = function(){topLevelTab.trigger("mouseenter");};
        var leave = function(){topLevelTab.delay("800").trigger("mouseout")};
        currentSubnav.hover(function(){enter()},function(){leave()});

        topLevelTab.bind("open", function(){
            //make the tab "highlight" on hover
            $(this).addClass('selected');
            activeSubnav.hide();
            //show the current subnav and trigger it to stay open
            currentSubnav.slideDown('fast');
            topLevelTab.trigger("hovering");
          }).bind("hovering", function(){
            currentSubnav.show();
          })
          .bind("close.menu", function(){
            //take away tab highlight
            if(!$(this).hasClass('active')) {
                $(this).removeClass('selected');
                activeTab.addClass('selected');
                activeSubnav.show();
              //the stuff to do to if it's not the current tab
              currentSubnav.slideUp('fast');
            }
          }).hoverIntent(hoverSettings);

      });
    },
    hoverMenu : function(element, options){
      var li = $(element);
      if (!(li.children().first().hasClass('arrow_icon_menu'))){
        //append the arrow icon to all menus with a third level
        li.prepend($('<div class="arrow_icon_menu"></div>'));
      }
      var ul = li.find('ul');
      options = options || {};

      if( options.top ){
        ul.css('top', options.top);
      }

      li.hoverIntent(
          function(){
            $(document).mouseup();
            ul.addClass("third_level").slideDown('fast');
          },
          function(){
            ul.slideUp('fast').removeClass("third_level");
      });
      li.mouseenter().mouseleave();
    }
  };
})();
(function() {
  var NodeTypes, ParameterMissing, Utils, defaults,
    __hasProp = {}.hasOwnProperty;

  ParameterMissing = function(message) {
    this.message = message;
  };

  ParameterMissing.prototype = new Error();

  defaults = {
    prefix: "",
    default_url_options: {}
  };

  NodeTypes = {"GROUP":1,"CAT":2,"SYMBOL":3,"OR":4,"STAR":5,"LITERAL":6,"SLASH":7,"DOT":8};

  Utils = {
    serialize: function(object, prefix) {
      var element, i, key, prop, result, s, _i, _len;

      if (prefix == null) {
        prefix = null;
      }
      if (!object) {
        return "";
      }
      if (!prefix && !(this.get_object_type(object) === "object")) {
        throw new Error("Url parameters should be a javascript hash");
      }
      if (window.jQuery) {
        result = window.jQuery.param(object);
        return (!result ? "" : result);
      }
      s = [];
      switch (this.get_object_type(object)) {
        case "array":
          for (i = _i = 0, _len = object.length; _i < _len; i = ++_i) {
            element = object[i];
            s.push(this.serialize(element, prefix + "[]"));
          }
          break;
        case "object":
          for (key in object) {
            if (!__hasProp.call(object, key)) continue;
            prop = object[key];
            if (!(prop != null)) {
              continue;
            }
            if (prefix != null) {
              key = "" + prefix + "[" + key + "]";
            }
            s.push(this.serialize(prop, key));
          }
          break;
        default:
          if (object) {
            s.push("" + (encodeURIComponent(prefix.toString())) + "=" + (encodeURIComponent(object.toString())));
          }
      }
      if (!s.length) {
        return "";
      }
      return s.join("&");
    },
    clean_path: function(path) {
      var last_index;

      path = path.split("://");
      last_index = path.length - 1;
      path[last_index] = path[last_index].replace(/\/+/g, "/").replace(/.\/$/m, "");
      return path.join("://");
    },
    set_default_url_options: function(optional_parts, options) {
      var i, part, _i, _len, _results;

      _results = [];
      for (i = _i = 0, _len = optional_parts.length; _i < _len; i = ++_i) {
        part = optional_parts[i];
        if (!options.hasOwnProperty(part) && defaults.default_url_options.hasOwnProperty(part)) {
          _results.push(options[part] = defaults.default_url_options[part]);
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    },
    extract_anchor: function(options) {
      var anchor;

      anchor = "";
      if (options.hasOwnProperty("anchor")) {
        anchor = "#" + options.anchor;
        delete options.anchor;
      }
      return anchor;
    },
    extract_options: function(number_of_params, args) {
      var ret_value;

      ret_value = {};
      if (args.length > number_of_params) {
        ret_value = args.pop();
      }
      return ret_value;
    },
    path_identifier: function(object) {
      var property;

      if (object === 0) {
        return "0";
      }
      if (!object) {
        return "";
      }
      property = object;
      if (this.get_object_type(object) === "object") {
        property = object.to_param || object.id || object;
        if (this.get_object_type(property) === "function") {
          property = property.call(object);
        }
      }
      return property.toString();
    },
    clone: function(obj) {
      var attr, copy, key;

      if ((obj == null) || "object" !== this.get_object_type(obj)) {
        return obj;
      }
      copy = obj.constructor();
      for (key in obj) {
        if (!__hasProp.call(obj, key)) continue;
        attr = obj[key];
        copy[key] = attr;
      }
      return copy;
    },
    prepare_parameters: function(required_parameters, actual_parameters, options) {
      var i, result, val, _i, _len;

      result = this.clone(options) || {};
      for (i = _i = 0, _len = required_parameters.length; _i < _len; i = ++_i) {
        val = required_parameters[i];
        result[val] = actual_parameters[i];
      }
      return result;
    },
    build_path: function(required_parameters, optional_parts, route, args) {
      var opts, parameters, result, url, url_params;

      args = Array.prototype.slice.call(args);
      opts = this.extract_options(required_parameters.length, args);
      if (args.length > required_parameters.length) {
        throw new Error("Too many parameters provided for path");
      }
      parameters = this.prepare_parameters(required_parameters, args, opts);
      this.set_default_url_options(optional_parts, parameters);
      result = "" + (this.get_prefix()) + (this.visit(route, parameters));
      url = Utils.clean_path("" + result + (this.extract_anchor(parameters)));
      if ((url_params = this.serialize(parameters)).length) {
        url += "?" + url_params;
      }
      return url;
    },
    visit: function(route, parameters, optional) {
      var left, left_part, right, right_part, type, value;

      if (optional == null) {
        optional = false;
      }
      type = route[0], left = route[1], right = route[2];
      switch (type) {
        case NodeTypes.GROUP:
          return this.visit(left, parameters, true);
        case NodeTypes.STAR:
          return this.visit_globbing(left, parameters, true);
        case NodeTypes.LITERAL:
        case NodeTypes.SLASH:
        case NodeTypes.DOT:
          return left;
        case NodeTypes.CAT:
          left_part = this.visit(left, parameters, optional);
          right_part = this.visit(right, parameters, optional);
          if (optional && !(left_part && right_part)) {
            return "";
          }
          return "" + left_part + right_part;
        case NodeTypes.SYMBOL:
          value = parameters[left];
          if (value != null) {
            delete parameters[left];
            return this.path_identifier(value);
          }
          if (optional) {
            return "";
          } else {
            throw new ParameterMissing("Route parameter missing: " + left);
          }
          break;
        default:
          throw new Error("Unknown Rails node type");
      }
    },
    visit_globbing: function(route, parameters, optional) {
      var left, right, type, value;

      type = route[0], left = route[1], right = route[2];
      value = parameters[left];
      if (value == null) {
        return this.visit(route, parameters, optional);
      }
      parameters[left] = (function() {
        switch (this.get_object_type(value)) {
          case "array":
            return value.join("/");
          default:
            return value;
        }
      }).call(this);
      return this.visit(route, parameters, optional);
    },
    get_prefix: function() {
      var prefix;

      prefix = defaults.prefix;
      if (prefix !== "") {
        prefix = (prefix.match("/$") ? prefix : "" + prefix + "/");
      }
      return prefix;
    },
    _classToTypeCache: null,
    _classToType: function() {
      var name, _i, _len, _ref;

      if (this._classToTypeCache != null) {
        return this._classToTypeCache;
      }
      this._classToTypeCache = {};
      _ref = "Boolean Number String Function Array Date RegExp Undefined Null".split(" ");
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        name = _ref[_i];
        this._classToTypeCache["[object " + name + "]"] = name.toLowerCase();
      }
      return this._classToTypeCache;
    },
    get_object_type: function(obj) {
      var strType;

      if (window.jQuery && (window.jQuery.type != null)) {
        return window.jQuery.type(obj);
      }
      strType = Object.prototype.toString.call(obj);
      return this._classToType()[strType] || "object";
    },
    namespace: function(root, namespaceString) {
      var current, parts;

      parts = (namespaceString ? namespaceString.split(".") : []);
      if (!parts.length) {
        return;
      }
      current = parts.shift();
      root[current] = root[current] || {};
      return Utils.namespace(root[current], parts.join("."));
    }
  };

  Utils.namespace(window, "KT.routes");

  window.KT.routes = {
// about => /about(.:format)
  about_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"about",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// activation_key => /activation_keys/:id(.:format)
  activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// activation_keys => /activation_keys(.:format)
  activation_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"activation_keys",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_parameter_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/add_parameter(.:format)
  add_parameter_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_parameter",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_subscriptions_activation_key => /activation_keys/:id/add_subscriptions(.:format)
  add_subscriptions_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_system_group_packages => /system_groups/:system_group_id/packages/add(.:format)
  add_system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"add",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_system_groups_activation_key => /activation_keys/:id/add_system_groups(.:format)
  add_system_groups_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_system_groups_system => /systems/:id/add_system_groups(.:format)
  add_system_groups_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_system_system_packages => /systems/:system_id/system_packages/add(.:format)
  add_system_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"system_packages",false]],[7,"/",false]],[6,"add",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_systems_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/add_systems(.:format)
  add_systems_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_systems_api_system_group => /api/system_groups/:id/add_systems(.:format)
  add_systems_api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// add_systems_system_group => /system_groups/:id/add_systems(.:format)
  add_systems_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"add_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// all_api_content_view_definition_products => /api/content_view_definitions/:content_view_definition_id/products/all(.:format)
  all_api_content_view_definition_products_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[6,"all",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// all_api_organization_content_view_definition_products => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/products/all(.:format)
  all_api_organization_content_view_definition_products_path: function(_organization_id, _content_view_definition_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[6,"all",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// allowed_orgs_user_session => /user_session/allowed_orgs(.:format)
  allowed_orgs_user_session_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"user_session",false]],[7,"/",false]],[6,"allowed_orgs",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api => /api(.:format)
  api_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"api",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_about_index => /api/about(.:format)
  api_about_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"about",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_activation_key => /api/activation_keys/:id(.:format)
  api_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_activation_keys => /api/activation_keys(.:format)
  api_activation_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset => /api/changesets/:id(.:format)
  api_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_content_view => /api/changesets/:changeset_id/content_views/:id(.:format)
  api_changeset_content_view_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_content_views => /api/changesets/:changeset_id/content_views(.:format)
  api_changeset_content_views_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_distribution => /api/changesets/:changeset_id/distributions/:id(.:format)
  api_changeset_distribution_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"distributions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_distributions => /api/changesets/:changeset_id/distributions(.:format)
  api_changeset_distributions_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"distributions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_errata => /api/changesets/:changeset_id/errata(.:format)
  api_changeset_errata_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_erratum => /api/changesets/:changeset_id/errata/:id(.:format)
  api_changeset_erratum_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_package => /api/changesets/:changeset_id/packages/:id(.:format)
  api_changeset_package_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_packages => /api/changesets/:changeset_id/packages(.:format)
  api_changeset_packages_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_product => /api/changesets/:changeset_id/products/:id(.:format)
  api_changeset_product_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_products => /api/changesets/:changeset_id/products(.:format)
  api_changeset_products_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_repositories => /api/changesets/:changeset_id/repositories(.:format)
  api_changeset_repositories_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_repository => /api/changesets/:changeset_id/repositories/:id(.:format)
  api_changeset_repository_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_template => /api/changesets/:changeset_id/templates/:id(.:format)
  api_changeset_template_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"templates",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_changeset_templates => /api/changesets/:changeset_id/templates(.:format)
  api_changeset_templates_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"templates",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_consumer => /api/consumers/:id(.:format)
  api_consumer_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_consumers => /api/consumers(.:format)
  api_consumers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view => /api/content_views/:id(.:format)
  api_content_view_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition => /api/content_view_definitions/:id(.:format)
  api_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_content_views => /api/content_view_definitions/:content_view_definition_id/content_views(.:format)
  api_content_view_definition_content_views_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_filter => /api/content_view_definitions/:content_view_definition_id/filters/:id(.:format)
  api_content_view_definition_filter_path: function(_content_view_definition_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_filter_products => /api/content_view_definitions/:content_view_definition_id/filters/:filter_id/products(.:format)
  api_content_view_definition_filter_products_path: function(_content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_filter_repositories => /api/content_view_definitions/:content_view_definition_id/filters/:filter_id/repositories(.:format)
  api_content_view_definition_filter_repositories_path: function(_content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_filter_rule => /api/content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id(.:format)
  api_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_filter_rules => /api/content_view_definitions/:content_view_definition_id/filters/:filter_id/rules(.:format)
  api_content_view_definition_filter_rules_path: function(_content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_filters => /api/content_view_definitions/:content_view_definition_id/filters(.:format)
  api_content_view_definition_filters_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_products => /api/content_view_definitions/:content_view_definition_id/products(.:format)
  api_content_view_definition_products_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_content_view_definition_repositories => /api/content_view_definitions/:content_view_definition_id/repositories(.:format)
  api_content_view_definition_repositories_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_create_custom_info => /api/custom_info/:informable_type/:informable_id(.:format)
  api_create_custom_info_path: function(_informable_type, _informable_id, options) {
  return Utils.build_path(["informable_type","informable_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"custom_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[3,"informable_id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_crls => /api/crls(.:format)
  api_crls_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"crls",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_custom_info => /api/custom_info/:informable_type/:informable_id(.:format)
  api_custom_info_path: function(_informable_type, _informable_id, options) {
  return Utils.build_path(["informable_type","informable_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"custom_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[3,"informable_id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_destroy_custom_info => /api/custom_info/:informable_type/:informable_id/:keyname(.:format)
  api_destroy_custom_info_path: function(_informable_type, _informable_id, _keyname, options) {
  return Utils.build_path(["informable_type","informable_id","keyname"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"custom_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[3,"informable_id",false]],[7,"/",false]],[3,"keyname",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_distributor => /api/distributors/:id(.:format)
  api_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_distributor_subscription => /api/distributors/:distributor_id/subscriptions/:id(.:format)
  api_distributor_subscription_path: function(_distributor_id, _id, options) {
  return Utils.build_path(["distributor_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"subscriptions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_distributor_subscriptions => /api/distributors/:distributor_id/subscriptions(.:format)
  api_distributor_subscriptions_path: function(_distributor_id, options) {
  return Utils.build_path(["distributor_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_distributors => /api/distributors(.:format)
  api_distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"distributors",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment => /api/environments/:id(.:format)
  api_environment_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment_activation_keys => /api/environments/:environment_id/activation_keys(.:format)
  api_environment_activation_keys_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"activation_keys",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment_changesets => /api/environments/:environment_id/changesets(.:format)
  api_environment_changesets_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"changesets",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment_content_views => /api/environments/:environment_id/content_views(.:format)
  api_environment_content_views_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment_distributors => /api/environments/:environment_id/distributors(.:format)
  api_environment_distributors_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"distributors",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment_products => /api/environments/:environment_id/products(.:format)
  api_environment_products_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_environment_systems => /api/environments/:environment_id/systems(.:format)
  api_environment_systems_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_errata => /api/errata(.:format)
  api_errata_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_gpg_key => /api/gpg_keys/:id(.:format)
  api_gpg_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"gpg_keys",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_hypervisors => /api/hypervisors(.:format)
  api_hypervisors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"hypervisors",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization => /api/organizations/:id(.:format)
  api_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_activation_key => /api/organizations/:organization_id/activation_keys/:id(.:format)
  api_organization_activation_key_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_activation_keys => /api/organizations/:organization_id/activation_keys(.:format)
  api_organization_activation_keys_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"activation_keys",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_apply_default_info => /api/organizations/:organization_id/default_info/:informable_type/apply(.:format)
  api_organization_apply_default_info_path: function(_organization_id, _informable_type, options) {
  return Utils.build_path(["organization_id","informable_type"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"default_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[6,"apply",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view => /api/organizations/:organization_id/content_views/:id(.:format)
  api_organization_content_view_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition => /api/organizations/:organization_id/content_view_definitions/:id(.:format)
  api_organization_content_view_definition_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_filter => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/filters/:id(.:format)
  api_organization_content_view_definition_filter_path: function(_organization_id, _content_view_definition_id, _id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_filter_products => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/filters/:filter_id/products(.:format)
  api_organization_content_view_definition_filter_products_path: function(_organization_id, _content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_filter_repositories => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/filters/:filter_id/repositories(.:format)
  api_organization_content_view_definition_filter_repositories_path: function(_organization_id, _content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_filter_rule => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id(.:format)
  api_organization_content_view_definition_filter_rule_path: function(_organization_id, _content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_filter_rules => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/filters/:filter_id/rules(.:format)
  api_organization_content_view_definition_filter_rules_path: function(_organization_id, _content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_filters => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/filters(.:format)
  api_organization_content_view_definition_filters_path: function(_organization_id, _content_view_definition_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_products => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/products(.:format)
  api_organization_content_view_definition_products_path: function(_organization_id, _content_view_definition_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definition_repositories => /api/organizations/:organization_id/content_view_definitions/:content_view_definition_id/repositories(.:format)
  api_organization_content_view_definition_repositories_path: function(_organization_id, _content_view_definition_id, options) {
  return Utils.build_path(["organization_id","content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_view_definitions => /api/organizations/:organization_id/content_view_definitions(.:format)
  api_organization_content_view_definitions_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_content_views => /api/organizations/:organization_id/content_views(.:format)
  api_organization_content_views_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_create_default_info => /api/organizations/:organization_id/default_info/:informable_type(.:format)
  api_organization_create_default_info_path: function(_organization_id, _informable_type, options) {
  return Utils.build_path(["organization_id","informable_type"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"default_info",false]],[7,"/",false]],[3,"informable_type",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_destroy_default_info => /api/organizations/:organization_id/default_info/:informable_type/:keyname(.:format)
  api_organization_destroy_default_info_path: function(_organization_id, _informable_type, _keyname, options) {
  return Utils.build_path(["organization_id","informable_type","keyname"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"default_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[3,"keyname",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_distributors => /api/organizations/:organization_id/distributors(.:format)
  api_organization_distributors_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"distributors",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_environment => /api/organizations/:organization_id/environments/:id(.:format)
  api_organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_environment_changesets => /api/organizations/:organization_id/environments/:environment_id/changesets(.:format)
  api_organization_environment_changesets_path: function(_organization_id, _environment_id, options) {
  return Utils.build_path(["organization_id","environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"changesets",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_environments => /api/organizations/:organization_id/environments(.:format)
  api_organization_environments_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_gpg_keys => /api/organizations/:organization_id/gpg_keys(.:format)
  api_organization_gpg_keys_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"gpg_keys",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_product => /api/organizations/:organization_id/products/:id(.:format)
  api_organization_product_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_product_repository_sets => /api/organizations/:organization_id/products/:product_id/repository_sets(.:format)
  api_organization_product_repository_sets_path: function(_organization_id, _product_id, options) {
  return Utils.build_path(["organization_id","product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repository_sets",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_product_sync_index => /api/organizations/:organization_id/products/:product_id/sync(.:format)
  api_organization_product_sync_index_path: function(_organization_id, _product_id, options) {
  return Utils.build_path(["organization_id","product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"sync",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_products => /api/organizations/:organization_id/products(.:format)
  api_organization_products_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_providers => /api/organizations/:organization_id/providers(.:format)
  api_organization_providers_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"providers",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_sync_plan => /api/organizations/:organization_id/sync_plans/:id(.:format)
  api_organization_sync_plan_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"sync_plans",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_sync_plans => /api/organizations/:organization_id/sync_plans(.:format)
  api_organization_sync_plans_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"sync_plans",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_system_group => /api/organizations/:organization_id/system_groups/:id(.:format)
  api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_system_group_errata => /api/organizations/:organization_id/system_groups/:system_group_id/errata(.:format)
  api_organization_system_group_errata_path: function(_organization_id, _system_group_id, options) {
  return Utils.build_path(["organization_id","system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_system_group_packages => /api/organizations/:organization_id/system_groups/:system_group_id/packages(.:format)
  api_organization_system_group_packages_path: function(_organization_id, _system_group_id, options) {
  return Utils.build_path(["organization_id","system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_system_groups => /api/organizations/:organization_id/system_groups(.:format)
  api_organization_system_groups_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_systems => /api/organizations/:organization_id/systems(.:format)
  api_organization_systems_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_tasks => /api/organizations/:organization_id/tasks(.:format)
  api_organization_tasks_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"tasks",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organization_uebercert => /api/organizations/:organization_id/uebercert(.:format)
  api_organization_uebercert_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"uebercert",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_organizations => /api/organizations(.:format)
  api_organizations_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_ping_index => /api/ping(.:format)
  api_ping_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"ping",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_pool => /api/activation_keys/:id/pools/:id(.:format)
  api_pool_path: function(_id, _id, options) {
  return Utils.build_path(["id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"pools",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_pools => /api/activation_keys/:id/pools(.:format)
  api_pools_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_product => /api/products/:id(.:format)
  api_product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_product_repositories => /api/products/:product_id/repositories(.:format)
  api_product_repositories_path: function(_product_id, options) {
  return Utils.build_path(["product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_product_repository_sets => /api/products/:product_id/repository_sets(.:format)
  api_product_repository_sets_path: function(_product_id, options) {
  return Utils.build_path(["product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repository_sets",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_product_sync_index => /api/products/:product_id/sync(.:format)
  api_product_sync_index_path: function(_product_id, options) {
  return Utils.build_path(["product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"sync",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_provider => /api/providers/:id(.:format)
  api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_provider_sync_index => /api/providers/:provider_id/sync(.:format)
  api_provider_sync_index_path: function(_provider_id, options) {
  return Utils.build_path(["provider_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"sync",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_providers => /api/providers(.:format)
  api_providers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_certificate_serials_path => /api/consumers/:id/certificates/serials(.:format)
  api_proxy_certificate_serials_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"certificates",false]],[7,"/",false]],[6,"serials",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_certificates_delete_path => /api/consumers/:consumer_id/certificates/:id(.:format)
  api_proxy_consumer_certificates_delete_path_path: function(_consumer_id, _id, options) {
  return Utils.build_path(["consumer_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"consumer_id",false]],[7,"/",false]],[6,"certificates",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_certificates_path => /api/consumers/:id/certificates(.:format)
  api_proxy_consumer_certificates_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"certificates",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_compliance_path => /api/consumers/:id/compliance(.:format)
  api_proxy_consumer_compliance_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"compliance",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_deletionrecord_delete_path => /api/consumers/:id/deletionrecord(.:format)
  api_proxy_consumer_deletionrecord_delete_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"deletionrecord",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_dryrun_path => /api/consumers/:id/entitlements/dry-run(.:format)
  api_proxy_consumer_dryrun_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"entitlements",false]],[7,"/",false]],[6,"dry-run",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_entitlements_delete_path => /api/consumers/:id/entitlements(.:format)
  api_proxy_consumer_entitlements_delete_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"entitlements",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_entitlements_path => /api/consumers/:id/entitlements(.:format)
  api_proxy_consumer_entitlements_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"entitlements",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_entitlements_post_path => /api/consumers/:id/entitlements(.:format)
  api_proxy_consumer_entitlements_post_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"entitlements",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_export_path => /api/consumers/:id/export(.:format)
  api_proxy_consumer_export_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"export",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_owners_path => /api/consumers/:id/owner(.:format)
  api_proxy_consumer_owners_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"owner",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_consumer_releases_path => /api/consumers/:id/release(.:format)
  api_proxy_consumer_releases_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"release",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_entitlements_path => /api/entitlements/:id(.:format)
  api_proxy_entitlements_path_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"entitlements",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_owner_pools_path => /api/owners/:organization_id/pools(.:format)
  api_proxy_owner_pools_path_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"owners",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_owner_servicelevels_path => /api/owners/:organization_id/servicelevels(.:format)
  api_proxy_owner_servicelevels_path_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"owners",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"servicelevels",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_pools_path => /api/pools(.:format)
  api_proxy_pools_path_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_proxy_subscriptions_post_path => /api/subscriptions(.:format)
  api_proxy_subscriptions_post_path_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repositories => /api/repositories(.:format)
  api_repositories_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository => /api/repositories/:id(.:format)
  api_repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_distribution => /api/repositories/:repository_id/distributions/:id(.:format)
  api_repository_distribution_path: function(_repository_id, _id, options) {
  return Utils.build_path(["repository_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"distributions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_distributions => /api/repositories/:repository_id/distributions(.:format)
  api_repository_distributions_path: function(_repository_id, options) {
  return Utils.build_path(["repository_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"distributions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_errata => /api/repositories/:repository_id/errata(.:format)
  api_repository_errata_path: function(_repository_id, options) {
  return Utils.build_path(["repository_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_erratum => /api/repositories/:repository_id/errata/:id(.:format)
  api_repository_erratum_path: function(_repository_id, _id, options) {
  return Utils.build_path(["repository_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_package => /api/repositories/:repository_id/packages/:id(.:format)
  api_repository_package_path: function(_repository_id, _id, options) {
  return Utils.build_path(["repository_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_packages => /api/repositories/:repository_id/packages(.:format)
  api_repository_packages_path: function(_repository_id, options) {
  return Utils.build_path(["repository_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_repository_sync_index => /api/repositories/:repository_id/sync(.:format)
  api_repository_sync_index_path: function(_repository_id, options) {
  return Utils.build_path(["repository_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"sync",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_role => /api/roles/:id(.:format)
  api_role_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_role_ldap_group => /api/roles/:role_id/ldap_groups/:id(.:format)
  api_role_ldap_group_path: function(_role_id, _id, options) {
  return Utils.build_path(["role_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"ldap_groups",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_role_ldap_groups => /api/roles/:role_id/ldap_groups(.:format)
  api_role_ldap_groups_path: function(_role_id, options) {
  return Utils.build_path(["role_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"ldap_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_role_permission => /api/roles/:role_id/permissions/:id(.:format)
  api_role_permission_path: function(_role_id, _id, options) {
  return Utils.build_path(["role_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"permissions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_role_permissions => /api/roles/:role_id/permissions(.:format)
  api_role_permissions_path: function(_role_id, options) {
  return Utils.build_path(["role_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"permissions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_roles => /api/roles(.:format)
  api_roles_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_show_custom_info => /api/custom_info/:informable_type/:informable_id/:keyname(.:format)
  api_show_custom_info_path: function(_informable_type, _informable_id, _keyname, options) {
  return Utils.build_path(["informable_type","informable_id","keyname"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"custom_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[3,"informable_id",false]],[7,"/",false]],[3,"keyname",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_status => /api/status(.:format)
  api_status_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_status_memory => /api/status/memory(.:format)
  api_status_memory_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"status",false]],[7,"/",false]],[6,"memory",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_subscriptions => /api/subscriptions(.:format)
  api_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_sync_plan => /api/sync_plans/:id(.:format)
  api_sync_plan_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"sync_plans",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system => /api/systems/:id(.:format)
  api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_group => /api/system_groups/:id(.:format)
  api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_group_errata => /api/system_groups/:system_group_id/errata(.:format)
  api_system_group_errata_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_group_packages => /api/system_groups/:system_group_id/packages(.:format)
  api_system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_groups => /api/system_groups(.:format)
  api_system_groups_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_packages => /api/systems/:system_id/packages(.:format)
  api_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_subscription => /api/systems/:system_id/subscriptions/:id(.:format)
  api_system_subscription_path: function(_system_id, _id, options) {
  return Utils.build_path(["system_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"subscriptions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_system_subscriptions => /api/systems/:system_id/subscriptions(.:format)
  api_system_subscriptions_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_systems => /api/systems(.:format)
  api_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_task => /api/tasks/:id(.:format)
  api_task_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"tasks",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_update_custom_info => /api/custom_info/:informable_type/:informable_id/:keyname(.:format)
  api_update_custom_info_path: function(_informable_type, _informable_id, _keyname, options) {
  return Utils.build_path(["informable_type","informable_id","keyname"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"custom_info",false]],[7,"/",false]],[3,"informable_type",false]],[7,"/",false]],[3,"informable_id",false]],[7,"/",false]],[3,"keyname",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_user => /api/users/:id(.:format)
  api_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_user_role => /api/users/:user_id/roles/:id(.:format)
  api_user_role_path: function(_user_id, _id, options) {
  return Utils.build_path(["user_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[3,"user_id",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_user_roles => /api/users/:user_id/roles(.:format)
  api_user_roles_path: function(_user_id, options) {
  return Utils.build_path(["user_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[3,"user_id",false]],[7,"/",false]],[6,"roles",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_users => /api/users(.:format)
  api_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// api_version => /api/version(.:format)
  api_version_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"version",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// apipie_apipie => /apidoc(/:version)(/:resource)(/:method)(.:format)
  apipie_apipie_path: function(options) {
  return Utils.build_path([], ["version","resource","method","format"], [2,[2,[2,[2,[2,[7,"/",false],[6,"apidoc",false]],[1,[2,[7,"/",false],[3,"version",false]],false]],[1,[2,[7,"/",false],[3,"resource",false]],false]],[1,[2,[7,"/",false],[3,"method",false]],false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// applied_subscriptions_activation_key => /activation_keys/:id/applied_subscriptions(.:format)
  applied_subscriptions_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"applied_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// apply_api_changeset => /api/changesets/:id/apply(.:format)
  apply_api_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"apply",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// apply_changeset => /changesets/:id/apply(.:format)
  apply_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"apply",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// apply_default_info_status_organization => /organizations/:id/apply_default_info_status(.:format)
  apply_default_info_status_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"apply_default_info_status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// authenticate => /authenticate(.:format)
  authenticate_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"authenticate",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_content_views => /content_views/auto_complete(.:format)
  auto_complete_content_views_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_views",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_distributors => /distributors/auto_complete(.:format)
  auto_complete_distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_errata => /errata/auto_complete(.:format)
  auto_complete_errata_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"errata",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_library_packages => /packages/auto_complete_library(.:format)
  auto_complete_library_packages_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[6,"auto_complete_library",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_library_repositories => /repositories/auto_complete_library(.:format)
  auto_complete_library_repositories_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[6,"auto_complete_library",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_nvrea_library_packages => /packages/auto_complete_nvrea_library(.:format)
  auto_complete_nvrea_library_packages_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[6,"auto_complete_nvrea_library",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_packages => /packages/auto_complete(.:format)
  auto_complete_packages_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_products => /products/auto_complete(.:format)
  auto_complete_products_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"products",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_activation_keys => /activation_keys/auto_complete_search(.:format)
  auto_complete_search_activation_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_changesets => /changesets/auto_complete_search(.:format)
  auto_complete_search_changesets_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_gpg_keys => /gpg_keys/auto_complete_search(.:format)
  auto_complete_search_gpg_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"gpg_keys",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_organizations => /organizations/auto_complete_search(.:format)
  auto_complete_search_organizations_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_providers => /providers/auto_complete_search(.:format)
  auto_complete_search_providers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_roles => /roles/auto_complete_search(.:format)
  auto_complete_search_roles_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_sync_plans => /sync_plans/auto_complete_search(.:format)
  auto_complete_search_sync_plans_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_plans",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_search_users => /users/auto_complete_search(.:format)
  auto_complete_search_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_system_groups => /system_groups/auto_complete(.:format)
  auto_complete_system_groups_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// auto_complete_systems => /systems/auto_complete(.:format)
  auto_complete_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"auto_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// available_subscriptions_activation_key => /activation_keys/:id/available_subscriptions(.:format)
  available_subscriptions_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"available_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// available_verbs_api_roles => /api/roles/available_verbs(.:format)
  available_verbs_api_roles_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[6,"available_verbs",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_add_system_group_systems => /systems/bulk_add_system_group(.:format)
  bulk_add_system_group_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_add_system_group",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_content_install_systems => /systems/bulk_content_install(.:format)
  bulk_content_install_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_content_install",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_content_remove_systems => /systems/bulk_content_remove(.:format)
  bulk_content_remove_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_content_remove",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_content_update_systems => /systems/bulk_content_update(.:format)
  bulk_content_update_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_content_update",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_destroy_distributors => /distributors/bulk_destroy(.:format)
  bulk_destroy_distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[6,"bulk_destroy",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_destroy_systems => /systems/bulk_destroy(.:format)
  bulk_destroy_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_destroy",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_errata_install_systems => /systems/bulk_errata_install(.:format)
  bulk_errata_install_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_errata_install",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// bulk_remove_system_group_systems => /systems/bulk_remove_system_group(.:format)
  bulk_remove_system_group_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"bulk_remove_system_group",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// cancel_discovery_provider => /providers/:id/cancel_discovery(.:format)
  cancel_discovery_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"cancel_discovery",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// changelog_package => /packages/:id/changelog(.:format)
  changelog_package_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"changelog",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// changeset => /changesets/:id(.:format)
  changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// changesets => /changesets(.:format)
  changesets_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"changesets",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// checkin_api_system => /api/systems/:id/checkin(.:format)
  checkin_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"checkin",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// clear_helptips_user => /users/:id/clear_helptips(.:format)
  clear_helptips_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"clear_helptips",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// clone_api_content_view_definition => /api/content_view_definitions/:id/clone(.:format)
  clone_api_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"clone",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// clone_api_organization_content_view_definition => /api/organizations/:organization_id/content_view_definitions/:id/clone(.:format)
  clone_api_organization_content_view_definition_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"clone",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// clone_content_view_definition => /content_view_definitions/:id/clone(.:format)
  clone_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"clone",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// consumers_subscription => /subscriptions/:id/consumers(.:format)
  consumers_subscription_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"consumers",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_api_gpg_key => /api/gpg_keys/:id/content(.:format)
  content_api_gpg_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"gpg_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_content_view_definition => /content_view_definitions/:id/content(.:format)
  content_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_organization_environment_content_view_version => /organizations/:organization_id/environments/:environment_id/content_view_versions/:id/content(.:format)
  content_organization_environment_content_view_version_path: function(_organization_id, _environment_id, _id, options) {
  return Utils.build_path(["organization_id","environment_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"content_view_versions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_search => /content_search/:id(.:format)
  content_search_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_search_index => /content_search(.:format)
  content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"content_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view => /content_views/:id(.:format)
  content_view_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definition => /content_view_definitions/:id(.:format)
  content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definition_content_view => /content_view_definitions/:content_view_definition_id/content_views/:id(.:format)
  content_view_definition_content_view_path: function(_content_view_definition_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definition_filter => /content_view_definitions/:content_view_definition_id/filters/:id(.:format)
  content_view_definition_filter_path: function(_content_view_definition_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id(.:format)
  content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definition_filter_rules => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules(.:format)
  content_view_definition_filter_rules_path: function(_content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definition_filters => /content_view_definitions/:content_view_definition_id/filters(.:format)
  content_view_definition_filters_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_view_definitions => /content_view_definitions(.:format)
  content_view_definitions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"content_view_definitions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_views => /content_views(.:format)
  content_views_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_views_api_content_view_definition => /api/content_view_definitions/:id/content_views(.:format)
  content_views_api_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_views_dashboard_index => /dashboard/content_views(.:format)
  content_views_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_views_environment => /environments/:id/content_views(.:format)
  content_views_environment_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_views_organization_environment => /organizations/:organization_id/environments/:id/content_views(.:format)
  content_views_organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// content_views_promotion => /promotions/:id/content_views(.:format)
  content_views_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"content_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// copy_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/copy(.:format)
  copy_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"copy",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// copy_api_system_group => /api/system_groups/:id/copy(.:format)
  copy_api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"copy",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// copy_system_group => /system_groups/:id/copy(.:format)
  copy_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"copy",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// create_role_ldap_groups => /roles/:role_id/ldap_groups(.:format)
  create_role_ldap_groups_path: function(_role_id, options) {
  return Utils.build_path(["role_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"ldap_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// custom_info_distributor => /distributors/:id/custom_info(.:format)
  custom_info_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"custom_info",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// custom_info_system => /systems/:id/custom_info(.:format)
  custom_info_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"custom_info",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// dashboard_index => /dashboard(.:format)
  dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"dashboard",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// default_label_content_view_definitions => /content_view_definitions/default_label(.:format)
  default_label_content_view_definitions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[6,"default_label",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// default_label_organization_environments => /organizations/:organization_id/environments/default_label(.:format)
  default_label_organization_environments_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[6,"default_label",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// default_label_organizations => /organizations/default_label(.:format)
  default_label_organizations_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[6,"default_label",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// default_label_provider_product_repositories => /providers/:provider_id/products/:product_id/repositories/default_label(.:format)
  default_label_provider_product_repositories_path: function(_provider_id, _product_id, options) {
  return Utils.build_path(["provider_id","product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[6,"default_label",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// default_label_provider_products => /providers/:provider_id/products/default_label(.:format)
  default_label_provider_products_path: function(_provider_id, options) {
  return Utils.build_path(["provider_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[6,"default_label",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// delete_manifest_api_provider => /api/providers/:id/delete_manifest(.:format)
  delete_manifest_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"delete_manifest",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// delete_manifest_subscriptions => /subscriptions/delete_manifest(.:format)
  delete_manifest_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"delete_manifest",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// dependencies_api_changeset => /api/changesets/:id/dependencies(.:format)
  dependencies_api_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"dependencies",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// dependencies_changeset => /changesets/:id/dependencies(.:format)
  dependencies_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"dependencies",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// dependencies_package => /packages/:id/dependencies(.:format)
  dependencies_package_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"dependencies",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_favorite_search_index => /search/favorite/:id(.:format)
  destroy_favorite_search_index_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"search",false]],[7,"/",false]],[6,"favorite",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_filters_content_view_definition_filters => /content_view_definitions/:content_view_definition_id/filters/destroy_filters(.:format)
  destroy_filters_content_view_definition_filters_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[6,"destroy_filters",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_parameters_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/destroy_parameters(.:format)
  destroy_parameters_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"destroy_parameters",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_role_ldap_group => /roles/:role_id/ldap_groups/:id(.:format)
  destroy_role_ldap_group_path: function(_role_id, _id, options) {
  return Utils.build_path(["role_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"ldap_groups",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_rules_content_view_definition_filter_rules => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/destroy_rules(.:format)
  destroy_rules_content_view_definition_filter_rules_path: function(_content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[6,"destroy_rules",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_systems_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/destroy_systems(.:format)
  destroy_systems_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"destroy_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_systems_api_system_group => /api/system_groups/:id/destroy_systems(.:format)
  destroy_systems_api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"destroy_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// destroy_systems_system_group => /system_groups/:id/destroy_systems(.:format)
  destroy_systems_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"destroy_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// details_package => /packages/:id/details(.:format)
  details_package_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"details",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// details_promotion => /promotions/:id/details(.:format)
  details_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"details",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// disable_api_organization_product_repository_set => /api/organizations/:organization_id/products/:product_id/repository_sets/:id/disable(.:format)
  disable_api_organization_product_repository_set_path: function(_organization_id, _product_id, _id, options) {
  return Utils.build_path(["organization_id","product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repository_sets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"disable",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// disable_api_product_repository_set => /api/products/:product_id/repository_sets/:id/disable(.:format)
  disable_api_product_repository_set_path: function(_product_id, _id, options) {
  return Utils.build_path(["product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repository_sets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"disable",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// disable_content_product => /products/:id/disable_content(.:format)
  disable_content_product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"disable_content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// disable_helptip_users => /users/disable_helptip(.:format)
  disable_helptip_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[6,"disable_helptip",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// discover_provider => /providers/:id/discover(.:format)
  discover_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"discover",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// discovered_repos_provider => /providers/:id/discovered_repos(.:format)
  discovered_repos_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"discovered_repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// discovery_api_provider => /api/providers/:id/discovery(.:format)
  discovery_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"discovery",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// distributions_promotion => /promotions/:id/distributions(.:format)
  distributions_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"distributions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// distributor => /distributors/:id(.:format)
  distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// distributor_event => /distributors/:distributor_id/events/:id(.:format)
  distributor_event_path: function(_distributor_id, _id, options) {
  return Utils.build_path(["distributor_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// distributor_events => /distributors/:distributor_id/events(.:format)
  distributor_events_path: function(_distributor_id, options) {
  return Utils.build_path(["distributor_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"events",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// distributors => /distributors(.:format)
  distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"distributors",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// download_debug_certificate_organization => /organizations/:id/download_debug_certificate(.:format)
  download_debug_certificate_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"download_debug_certificate",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// download_distributor => /distributors/:id/download(.:format)
  download_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"download",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_activation_key => /activation_keys/:id/edit(.:format)
  edit_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_activation_key => /api/activation_keys/:id/edit(.:format)
  edit_api_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_changeset_content_view => /api/changesets/:changeset_id/content_views/:id/edit(.:format)
  edit_api_changeset_content_view_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_changeset_distribution => /api/changesets/:changeset_id/distributions/:id/edit(.:format)
  edit_api_changeset_distribution_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"distributions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_changeset_erratum => /api/changesets/:changeset_id/errata/:id/edit(.:format)
  edit_api_changeset_erratum_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_changeset_package => /api/changesets/:changeset_id/packages/:id/edit(.:format)
  edit_api_changeset_package_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_changeset_product => /api/changesets/:changeset_id/products/:id/edit(.:format)
  edit_api_changeset_product_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_changeset_repository => /api/changesets/:changeset_id/repositories/:id/edit(.:format)
  edit_api_changeset_repository_path: function(_changeset_id, _id, options) {
  return Utils.build_path(["changeset_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_consumer => /api/consumers/:id/edit(.:format)
  edit_api_consumer_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_organization => /api/organizations/:id/edit(.:format)
  edit_api_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_organization_content_view_definition => /api/organizations/:organization_id/content_view_definitions/:id/edit(.:format)
  edit_api_organization_content_view_definition_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_organization_environment => /api/organizations/:organization_id/environments/:id/edit(.:format)
  edit_api_organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_organization_sync_plan => /api/organizations/:organization_id/sync_plans/:id/edit(.:format)
  edit_api_organization_sync_plan_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"sync_plans",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_organization_system_group_packages => /api/organizations/:organization_id/system_groups/:system_group_id/packages/edit(.:format)
  edit_api_organization_system_group_packages_path: function(_organization_id, _system_group_id, options) {
  return Utils.build_path(["organization_id","system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_provider => /api/providers/:id/edit(.:format)
  edit_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_repository_package => /api/repositories/:repository_id/packages/:id/edit(.:format)
  edit_api_repository_package_path: function(_repository_id, _id, options) {
  return Utils.build_path(["repository_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_role => /api/roles/:id/edit(.:format)
  edit_api_role_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_system_group_packages => /api/system_groups/:system_group_id/packages/edit(.:format)
  edit_api_system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_system_packages => /api/systems/:system_id/packages/edit(.:format)
  edit_api_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_api_user => /api/users/:id/edit(.:format)
  edit_api_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_changeset => /changesets/:id/edit(.:format)
  edit_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_content_search => /content_search/:id/edit(.:format)
  edit_content_search_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_content_view => /content_views/:id/edit(.:format)
  edit_content_view_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_content_view_definition => /content_view_definitions/:id/edit(.:format)
  edit_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_content_view_definition_filter => /content_view_definitions/:content_view_definition_id/filters/:id/edit(.:format)
  edit_content_view_definition_filter_path: function(_content_view_definition_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/edit(.:format)
  edit_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_date_type_parameters_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/edit_date_type_parameters(.:format)
  edit_date_type_parameters_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit_date_type_parameters",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_distributor => /distributors/:id/edit(.:format)
  edit_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_environment => /environments/:id/edit(.:format)
  edit_environment_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_environment_user => /users/:id/edit_environment(.:format)
  edit_environment_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit_environment",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_gpg_key => /gpg_keys/:id/edit(.:format)
  edit_gpg_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"gpg_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_inclusion_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/edit_inclusion(.:format)
  edit_inclusion_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit_inclusion",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_manifest_subscriptions => /subscriptions/edit_manifest(.:format)
  edit_manifest_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"edit_manifest",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_organization => /organizations/:id/edit(.:format)
  edit_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_organization_environment => /organizations/:organization_id/environments/:id/edit(.:format)
  edit_organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_parameter_list_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/edit_parameter_list(.:format)
  edit_parameter_list_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit_parameter_list",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_password_reset => /password_resets/:id/edit(.:format)
  edit_password_reset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"password_resets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_product => /products/:id/edit(.:format)
  edit_product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_provider => /providers/:id/edit(.:format)
  edit_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_provider_product => /providers/:provider_id/products/:id/edit(.:format)
  edit_provider_product_path: function(_provider_id, _id, options) {
  return Utils.build_path(["provider_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_provider_product_repository => /providers/:provider_id/products/:product_id/repositories/:id/edit(.:format)
  edit_provider_product_repository_path: function(_provider_id, _product_id, _id, options) {
  return Utils.build_path(["provider_id","product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_repository => /repositories/:id/edit(.:format)
  edit_repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_role => /roles/:id/edit(.:format)
  edit_role_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_subscription => /subscriptions/:id/edit(.:format)
  edit_subscription_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_sync_plan => /sync_plans/:id/edit(.:format)
  edit_sync_plan_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"sync_plans",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_system => /systems/:id/edit(.:format)
  edit_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_system_group => /system_groups/:id/edit(.:format)
  edit_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_systems_system_group => /system_groups/:id/edit_systems(.:format)
  edit_systems_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_user => /users/:id/edit(.:format)
  edit_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// edit_user_session => /user_session/edit(.:format)
  edit_user_session_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"user_session",false]],[7,"/",false]],[6,"edit",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// email_logins_password_resets => /password_resets/email_logins(.:format)
  email_logins_password_resets_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"password_resets",false]],[7,"/",false]],[6,"email_logins",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// enable_api_organization_product_repository_set => /api/organizations/:organization_id/products/:product_id/repository_sets/:id/enable(.:format)
  enable_api_organization_product_repository_set_path: function(_organization_id, _product_id, _id, options) {
  return Utils.build_path(["organization_id","product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repository_sets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"enable",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// enable_api_product_repository_set => /api/products/:product_id/repository_sets/:id/enable(.:format)
  enable_api_product_repository_set_path: function(_product_id, _id, options) {
  return Utils.build_path(["product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repository_sets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"enable",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// enable_api_repository => /api/repositories/:id/enable(.:format)
  enable_api_repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"enable",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// enable_helptip_users => /users/enable_helptip(.:format)
  enable_helptip_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[6,"enable_helptip",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// enable_repo => /repositories/:id/enable_repo(.:format)
  enable_repo_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"enable_repo",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// enabled_repos_api_system => /api/systems/:id/enabled_repos(.:format)
  enabled_repos_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"enabled_repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// env_items_distributors => /distributors/env_items(.:format)
  env_items_distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[6,"env_items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// env_items_systems => /systems/env_items(.:format)
  env_items_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"env_items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// environment => /environments/:id(.:format)
  environment_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// environments => /environments(.:format)
  environments_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"environments",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// environments_distributors => /distributors/environments(.:format)
  environments_distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[6,"environments",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// environments_partial_organization => /organizations/:id/environments_partial(.:format)
  environments_partial_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"environments_partial",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// environments_systems => /systems/environments(.:format)
  environments_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"environments",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// errata_api_system => /api/systems/:id/errata(.:format)
  errata_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// errata_content_search_index => /content_search/errata(.:format)
  errata_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// errata_dashboard_index => /dashboard/errata(.:format)
  errata_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// errata_items_content_search_index => /content_search/errata_items(.:format)
  errata_items_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"errata_items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// errata_promotion => /promotions/:id/errata(.:format)
  errata_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// erratum => /errata/:id(.:format)
  erratum_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// events_organization => /organizations/:id/events(.:format)
  events_organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"events",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// export_api_distributor => /api/distributors/:id/export(.:format)
  export_api_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"export",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// facts_system => /systems/:id/facts(.:format)
  facts_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"facts",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// favorite_search_index => /search/favorite(.:format)
  favorite_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"search",false]],[7,"/",false]],[6,"favorite",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// filelist_package => /packages/:id/filelist(.:format)
  filelist_package_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"filelist",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// filelist_repository_distribution => /repositories/:repository_id/distributions/:id/filelist(.:format)
  filelist_repository_distribution_path: function(_repository_id, _id, options) {
  return Utils.build_path(["repository_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"distributions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"filelist",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// gpg_key => /gpg_keys/:id(.:format)
  gpg_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"gpg_keys",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// gpg_key_content_api_repository => /api/repositories/:id/gpg_key_content(.:format)
  gpg_key_content_api_repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"gpg_key_content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// gpg_keys => /gpg_keys(.:format)
  gpg_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"gpg_keys",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// history_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/history(.:format)
  history_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"history",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// history_api_system_group => /api/system_groups/:id/history(.:format)
  history_api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"history",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// history_items_subscriptions => /subscriptions/history_items(.:format)
  history_items_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"history_items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// history_search_index => /search/history(.:format)
  history_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"search",false]],[7,"/",false]],[6,"history",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// history_subscriptions => /subscriptions/history(.:format)
  history_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"history",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// import_manifest_api_provider => /api/providers/:id/import_manifest(.:format)
  import_manifest_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"import_manifest",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// import_products_api_provider => /api/providers/:id/import_products(.:format)
  import_products_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"import_products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// install_system_errata => /systems/:system_id/errata/install(.:format)
  install_system_errata_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"install",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// install_system_group_errata => /system_groups/:system_group_id/errata/install(.:format)
  install_system_group_errata_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"install",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_activation_keys => /activation_keys/items(.:format)
  items_activation_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_changesets => /changesets/items(.:format)
  items_changesets_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_content_view_definitions => /content_view_definitions/items(.:format)
  items_content_view_definitions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_distributor_events => /distributors/:distributor_id/events/items(.:format)
  items_distributor_events_path: function(_distributor_id, options) {
  return Utils.build_path(["distributor_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_distributors => /distributors/items(.:format)
  items_distributors_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_gpg_keys => /gpg_keys/items(.:format)
  items_gpg_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"gpg_keys",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_organizations => /organizations/items(.:format)
  items_organizations_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_providers => /providers/items(.:format)
  items_providers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_roles => /roles/items(.:format)
  items_roles_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_subscriptions => /subscriptions/items(.:format)
  items_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_sync_plans => /sync_plans/items(.:format)
  items_sync_plans_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_plans",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_system_errata => /systems/:system_id/errata/items(.:format)
  items_system_errata_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_system_events => /systems/:system_id/events/items(.:format)
  items_system_events_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_system_group_errata => /system_groups/:system_group_id/errata/items(.:format)
  items_system_group_errata_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_system_group_events => /system_groups/:system_group_id/events/items(.:format)
  items_system_group_events_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_system_groups => /system_groups/items(.:format)
  items_system_groups_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_systems => /systems/items(.:format)
  items_systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// items_users => /users/items(.:format)
  items_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[6,"items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// list_changesets => /changesets/list(.:format)
  list_changesets_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[6,"list",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// login => /login(.:format)
  login_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"login",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// logout => /logout(.:format)
  logout_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"logout",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// manifest_progress_provider => /providers/:id/manifest_progress(.:format)
  manifest_progress_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"manifest_progress",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// more_events_distributor_events => /distributors/:distributor_id/events/more_events(.:format)
  more_events_distributor_events_path: function(_distributor_id, options) {
  return Utils.build_path(["distributor_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"more_events",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// more_events_system_events => /systems/:system_id/events/more_events(.:format)
  more_events_system_events_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"more_events",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// more_items_system_group_events => /system_groups/:system_group_id/events/more_items(.:format)
  more_items_system_group_events_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"more_items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// more_packages_system_system_packages => /systems/:system_id/system_packages/more_packages(.:format)
  more_packages_system_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"system_packages",false]],[7,"/",false]],[6,"more_packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// more_products_distributor => /distributors/:id/more_products(.:format)
  more_products_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"more_products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// more_products_system => /systems/:id/more_products(.:format)
  more_products_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"more_products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// name_changeset => /changesets/:id/name(.:format)
  name_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"name",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_activation_key => /activation_keys/new(.:format)
  new_activation_key_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_activation_key => /api/activation_keys/new(.:format)
  new_api_activation_key_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_changeset_content_view => /api/changesets/:changeset_id/content_views/new(.:format)
  new_api_changeset_content_view_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_changeset_distribution => /api/changesets/:changeset_id/distributions/new(.:format)
  new_api_changeset_distribution_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"distributions",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_changeset_erratum => /api/changesets/:changeset_id/errata/new(.:format)
  new_api_changeset_erratum_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_changeset_package => /api/changesets/:changeset_id/packages/new(.:format)
  new_api_changeset_package_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_changeset_product => /api/changesets/:changeset_id/products/new(.:format)
  new_api_changeset_product_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_changeset_repository => /api/changesets/:changeset_id/repositories/new(.:format)
  new_api_changeset_repository_path: function(_changeset_id, options) {
  return Utils.build_path(["changeset_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"changeset_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_consumer => /api/consumers/new(.:format)
  new_api_consumer_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"consumers",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_organization => /api/organizations/new(.:format)
  new_api_organization_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_organization_content_view_definition => /api/organizations/:organization_id/content_view_definitions/new(.:format)
  new_api_organization_content_view_definition_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_organization_environment => /api/organizations/:organization_id/environments/new(.:format)
  new_api_organization_environment_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_organization_sync_plan => /api/organizations/:organization_id/sync_plans/new(.:format)
  new_api_organization_sync_plan_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"sync_plans",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_organization_system_group_packages => /api/organizations/:organization_id/system_groups/:system_group_id/packages/new(.:format)
  new_api_organization_system_group_packages_path: function(_organization_id, _system_group_id, options) {
  return Utils.build_path(["organization_id","system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_provider => /api/providers/new(.:format)
  new_api_provider_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_repository_package => /api/repositories/:repository_id/packages/new(.:format)
  new_api_repository_package_path: function(_repository_id, options) {
  return Utils.build_path(["repository_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_role => /api/roles/new(.:format)
  new_api_role_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"roles",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_system_group_packages => /api/system_groups/:system_group_id/packages/new(.:format)
  new_api_system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_system_packages => /api/systems/:system_id/packages/new(.:format)
  new_api_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_api_user => /api/users/new(.:format)
  new_api_user_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_changeset => /changesets/new(.:format)
  new_changeset_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_content_search => /content_search/new(.:format)
  new_content_search_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_content_view => /content_views/new(.:format)
  new_content_view_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_views",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_content_view_definition => /content_view_definitions/new(.:format)
  new_content_view_definition_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_content_view_definition_filter => /content_view_definitions/:content_view_definition_id/filters/new(.:format)
  new_content_view_definition_filter_path: function(_content_view_definition_id, options) {
  return Utils.build_path(["content_view_definition_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/new(.:format)
  new_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_discovered_repos_provider => /providers/:id/new_discovered_repos(.:format)
  new_discovered_repos_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"new_discovered_repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_distributor => /distributors/new(.:format)
  new_distributor_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_environment => /environments/new(.:format)
  new_environment_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"environments",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_gpg_key => /gpg_keys/new(.:format)
  new_gpg_key_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"gpg_keys",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_organization => /organizations/new(.:format)
  new_organization_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_organization_environment => /organizations/:organization_id/environments/new(.:format)
  new_organization_environment_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_product => /products/new(.:format)
  new_product_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"products",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_provider => /providers/new(.:format)
  new_provider_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_provider_product => /providers/:provider_id/products/new(.:format)
  new_provider_product_path: function(_provider_id, options) {
  return Utils.build_path(["provider_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_provider_product_repository => /providers/:provider_id/products/:product_id/repositories/new(.:format)
  new_provider_product_repository_path: function(_provider_id, _product_id, options) {
  return Utils.build_path(["provider_id","product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_repository => /repositories/new(.:format)
  new_repository_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_role => /roles/new(.:format)
  new_role_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_subscription => /subscriptions/new(.:format)
  new_subscription_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_sync_plan => /sync_plans/new(.:format)
  new_sync_plan_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_plans",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_system => /systems/new(.:format)
  new_system_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_system_group => /system_groups/new(.:format)
  new_system_group_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_user => /users/new(.:format)
  new_user_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// new_user_session => /user_session/new(.:format)
  new_user_session_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"user_session",false]],[7,"/",false]],[6,"new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// notices => /notices(.:format)
  notices_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"notices",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// notices_auto_complete_search => /notices/auto_complete_search(.:format)
  notices_auto_complete_search_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"notices",false]],[7,"/",false]],[6,"auto_complete_search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// notices_dashboard_index => /dashboard/notices(.:format)
  notices_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"notices",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// notices_details => /notices/:id/details(.:format)
  notices_details_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"notices",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"details",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// notices_get_new => /notices/get_new(.:format)
  notices_get_new_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"notices",false]],[7,"/",false]],[6,"get_new",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// notices_note_count => /notices/note_count(.:format)
  notices_note_count_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"notices",false]],[7,"/",false]],[6,"note_count",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// object_changeset => /changesets/:id/object(.:format)
  object_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"object",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// operations => /operations(.:format)
  operations_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"operations",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// organization => /organizations/:id(.:format)
  organization_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// organization_default_info => /organizations/:id/default_info/:informable_type(.:format)
  organization_default_info_path: function(_id, _informable_type, options) {
  return Utils.build_path(["id","informable_type"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"default_info",false]],[7,"/",false]],[3,"informable_type",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// organization_environment => /organizations/:organization_id/environments/:id(.:format)
  organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// organization_environment_content_view_version => /organizations/:organization_id/environments/:environment_id/content_view_versions/:id(.:format)
  organization_environment_content_view_version_path: function(_organization_id, _environment_id, _id, options) {
  return Utils.build_path(["organization_id","environment_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"content_view_versions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// organization_environments => /organizations/:organization_id/environments(.:format)
  organization_environments_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// organizations => /organizations(.:format)
  organizations_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"organizations",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// package => /packages/:id(.:format)
  package_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// package_group_categories_api_repository => /api/repositories/:id/package_group_categories(.:format)
  package_group_categories_api_repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"package_group_categories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// package_groups_api_repository => /api/repositories/:id/package_groups(.:format)
  package_groups_api_repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"package_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// packages_api_system => /api/systems/:id/packages(.:format)
  packages_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// packages_content_search_index => /content_search/packages(.:format)
  packages_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// packages_erratum => /errata/:id/packages(.:format)
  packages_erratum_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// packages_items_content_search_index => /content_search/packages_items(.:format)
  packages_items_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"packages_items",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// packages_promotion => /promotions/:id/packages(.:format)
  packages_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// packages_system_system_packages => /systems/:system_id/system_packages/packages(.:format)
  packages_system_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"system_packages",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// password_reset => /password_resets/:id(.:format)
  password_reset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"password_resets",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// password_resets => /password_resets(.:format)
  password_resets_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"password_resets",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// pools_api_activation_key => /api/activation_keys/:id/pools(.:format)
  pools_api_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// pools_api_distributor => /api/distributors/:id/pools(.:format)
  pools_api_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// pools_api_organization_activation_key => /api/organizations/:organization_id/activation_keys/:id/pools(.:format)
  pools_api_organization_activation_key_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// pools_api_system => /api/systems/:id/pools(.:format)
  pools_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"pools",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// product => /products/:id(.:format)
  product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"products",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// product_create_api_provider => /api/providers/:id/product_create(.:format)
  product_create_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"product_create",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products => /products(.:format)
  products_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_api_provider => /api/providers/:id/products(.:format)
  products_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_content_search_index => /content_search/products(.:format)
  products_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_distributor => /distributors/:id/products(.:format)
  products_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_organization_environment => /organizations/:organization_id/environments/:id/products(.:format)
  products_organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_promotion => /promotions/:id/products(.:format)
  products_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_repos_gpg_key => /gpg_keys/:id/products_repos(.:format)
  products_repos_gpg_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"gpg_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products_repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_repos_provider => /providers/:id/products_repos(.:format)
  products_repos_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products_repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_subscription => /subscriptions/:id/products(.:format)
  products_subscription_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// products_system => /systems/:id/products(.:format)
  products_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// promote_api_changeset => /api/changesets/:id/promote(.:format)
  promote_api_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"promote",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// promote_api_content_view => /api/content_views/:id/promote(.:format)
  promote_api_content_view_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"promote",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// promotion => /promotions/:id(.:format)
  promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// promotions => /promotions(.:format)
  promotions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"promotions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// promotions_dashboard_index => /dashboard/promotions(.:format)
  promotions_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"promotions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// provider => /providers/:id(.:format)
  provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// provider_product => /providers/:provider_id/products/:id(.:format)
  provider_product_path: function(_provider_id, _id, options) {
  return Utils.build_path(["provider_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// provider_product_repositories => /providers/:provider_id/products/:product_id/repositories(.:format)
  provider_product_repositories_path: function(_provider_id, _product_id, options) {
  return Utils.build_path(["provider_id","product_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// provider_product_repository => /providers/:provider_id/products/:product_id/repositories/:id(.:format)
  provider_product_repository_path: function(_provider_id, _product_id, _id, options) {
  return Utils.build_path(["provider_id","product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// provider_products => /providers/:provider_id/products(.:format)
  provider_products_path: function(_provider_id, options) {
  return Utils.build_path(["provider_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// providers => /providers(.:format)
  providers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"providers",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// publish_api_content_view_definition => /api/content_view_definitions/:id/publish(.:format)
  publish_api_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"publish",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// publish_api_organization_content_view_definition => /api/organizations/:organization_id/content_view_definitions/:id/publish(.:format)
  publish_api_organization_content_view_definition_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"publish",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// publish_content_view_definition => /content_view_definitions/:id/publish(.:format)
  publish_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"publish",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// publish_setup_content_view_definition => /content_view_definitions/:id/publish_setup(.:format)
  publish_setup_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"publish_setup",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// rails_info_properties => /rails/info/properties(.:format)
  rails_info_properties_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"rails",false]],[7,"/",false]],[6,"info",false]],[7,"/",false]],[6,"properties",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// redhat_provider_providers => /providers/redhat_provider(.:format)
  redhat_provider_providers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[6,"redhat_provider",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_api_content_view => /api/content_views/:id/refresh(.:format)
  refresh_api_content_view_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"refresh",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_content_product => /products/:id/refresh_content(.:format)
  refresh_content_product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"refresh_content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_content_view_definition_content_view => /content_view_definitions/:content_view_definition_id/content_views/:id/refresh(.:format)
  refresh_content_view_definition_content_view_path: function(_content_view_definition_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"content_views",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"refresh",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_manifest_api_provider => /api/providers/:id/refresh_manifest(.:format)
  refresh_manifest_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"refresh_manifest",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_manifest_subscriptions => /subscriptions/refresh_manifest(.:format)
  refresh_manifest_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"refresh_manifest",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_products_api_provider => /api/providers/:id/refresh_products(.:format)
  refresh_products_api_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"refresh_products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_products_providers => /providers/refresh_products(.:format)
  refresh_products_providers_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[6,"refresh_products",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// refresh_subscriptions_api_system => /api/systems/:id/refresh_subscriptions(.:format)
  refresh_subscriptions_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"refresh_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// releases_api_environment => /api/environments/:id/releases(.:format)
  releases_api_environment_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"releases",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// releases_api_system => /api/systems/:id/releases(.:format)
  releases_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"releases",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_subscriptions_activation_key => /activation_keys/:id/remove_subscriptions(.:format)
  remove_subscriptions_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"remove_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_system_group_packages => /system_groups/:system_group_id/packages/remove(.:format)
  remove_system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"remove",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_system_groups_activation_key => /activation_keys/:id/remove_system_groups(.:format)
  remove_system_groups_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"remove_system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_system_groups_system => /systems/:id/remove_system_groups(.:format)
  remove_system_groups_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"remove_system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_system_system_packages => /systems/:system_id/system_packages/remove(.:format)
  remove_system_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"system_packages",false]],[7,"/",false]],[6,"remove",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_systems_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/remove_systems(.:format)
  remove_systems_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"remove_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_systems_api_system_group => /api/system_groups/:id/remove_systems(.:format)
  remove_systems_api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"remove_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// remove_systems_system_group => /system_groups/:id/remove_systems(.:format)
  remove_systems_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"remove_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repo_compare_errata_content_search_index => /content_search/repo_compare_errata(.:format)
  repo_compare_errata_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"repo_compare_errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repo_compare_packages_content_search_index => /content_search/repo_compare_packages(.:format)
  repo_compare_packages_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"repo_compare_packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repo_discovery_provider => /providers/:id/repo_discovery(.:format)
  repo_discovery_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repo_discovery",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repo_errata_content_search_index => /content_search/repo_errata(.:format)
  repo_errata_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"repo_errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repo_packages_content_search_index => /content_search/repo_packages(.:format)
  repo_packages_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"repo_packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// report_api_environment_systems => /api/environments/:environment_id/systems/report(.:format)
  report_api_environment_systems_path: function(_environment_id, options) {
  return Utils.build_path(["environment_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[6,"report",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// report_api_organization_systems => /api/organizations/:organization_id/systems/report(.:format)
  report_api_organization_systems_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[6,"report",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// report_api_users => /api/users/report(.:format)
  report_api_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[6,"report",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repos_content_search_index => /content_search/repos(.:format)
  repos_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repos_promotion => /promotions/:id/repos(.:format)
  repos_promotion_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"promotions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repos",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repositories => /repositories(.:format)
  repositories_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repositories_api_environment => /api/environments/:id/repositories(.:format)
  repositories_api_environment_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repositories_api_environment_product => /api/environments/:environment_id/products/:id/repositories(.:format)
  repositories_api_environment_product_path: function(_environment_id, _id, options) {
  return Utils.build_path(["environment_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"environment_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repositories_api_organization_environment => /api/organizations/:organization_id/environments/:id/repositories(.:format)
  repositories_api_organization_environment_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"environments",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repositories_api_organization_product => /api/organizations/:organization_id/products/:id/repositories(.:format)
  repositories_api_organization_product_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repositories_api_product => /api/products/:id/repositories(.:format)
  repositories_api_product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"repositories",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repository => /repositories/:id(.:format)
  repository_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// repository_distribution => /repositories/:repository_id/distributions/:id(.:format)
  repository_distribution_path: function(_repository_id, _id, options) {
  return Utils.build_path(["repository_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"distributions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// role => /roles/:id(.:format)
  role_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// role_create_permission => /roles/:role_id/create_permission(.:format)
  role_create_permission_path: function(_role_id, options) {
  return Utils.build_path(["role_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"create_permission",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// role_permission_destroy => /roles/:role_id/permission/:permission_id/destroy_permission(.:format)
  role_permission_destroy_path: function(_role_id, _permission_id, options) {
  return Utils.build_path(["role_id","permission_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"permission",false]],[7,"/",false]],[3,"permission_id",false]],[7,"/",false]],[6,"destroy_permission",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// role_permission_update => /roles/:role_id/permission/:permission_id/update_permission(.:format)
  role_permission_update_path: function(_role_id, _permission_id, options) {
  return Utils.build_path(["role_id","permission_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"role_id",false]],[7,"/",false]],[6,"permission",false]],[7,"/",false]],[3,"permission_id",false]],[7,"/",false]],[6,"update_permission",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// roles => /roles(.:format)
  roles_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"roles",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// roles_show_permission => /roles/show_permission(.:format)
  roles_show_permission_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[6,"show_permission",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// root => /
  root_path: function(options) {
  return Utils.build_path([], [], [7,"/",false], arguments);
  },
// schedule_provider => /providers/:id/schedule(.:format)
  schedule_provider_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"schedule",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// search_api_repository_packages => /api/repositories/:repository_id/packages/search(.:format)
  search_api_repository_packages_path: function(_repository_id, options) {
  return Utils.build_path(["repository_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"repository_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// search_index => /search(.:format)
  search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"search",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// set_org_user_session => /user_session/set_org(.:format)
  set_org_user_session_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"user_session",false]],[7,"/",false]],[6,"set_org",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// setup_default_org_user => /users/:id/setup_default_org(.:format)
  setup_default_org_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"setup_default_org",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// short_details_erratum => /errata/:id/short_details(.:format)
  short_details_erratum_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"short_details",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// show_user_session => /user_session(.:format)
  show_user_session_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"user_session",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_changeset => /changesets/:id/status(.:format)
  status_changeset_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"changesets",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_content_view_definition => /content_view_definitions/:id/status(.:format)
  status_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_distributor_events => /distributors/:distributor_id/events/status(.:format)
  status_distributor_events_path: function(_distributor_id, options) {
  return Utils.build_path(["distributor_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"distributor_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_system_errata => /systems/:system_id/errata/status(.:format)
  status_system_errata_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_system_events => /systems/:system_id/events/status(.:format)
  status_system_events_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_system_group_errata => /system_groups/:system_group_id/errata/status(.:format)
  status_system_group_errata_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_system_group_events => /system_groups/:system_group_id/events/status(.:format)
  status_system_group_events_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_system_group_packages => /system_groups/:system_group_id/packages/status(.:format)
  status_system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// status_system_system_packages => /systems/:system_id/system_packages/status(.:format)
  status_system_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"system_packages",false]],[7,"/",false]],[6,"status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscription => /subscriptions/:id(.:format)
  subscription_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscription_status_api_system => /api/systems/:id/subscription_status(.:format)
  subscription_status_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"subscription_status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscriptions => /subscriptions(.:format)
  subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscriptions_activation_keys => /activation_keys/subscriptions(.:format)
  subscriptions_activation_keys_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscriptions_dashboard_index => /dashboard/subscriptions(.:format)
  subscriptions_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscriptions_distributor => /distributors/:id/subscriptions(.:format)
  subscriptions_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// subscriptions_system => /systems/:id/subscriptions(.:format)
  subscriptions_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_complete_api_repositories => /api/repositories/sync_complete(.:format)
  sync_complete_api_repositories_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[6,"sync_complete",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_dashboard_index => /dashboard/sync(.:format)
  sync_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"sync",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_ldap_roles_api_users => /api/users/sync_ldap_roles(.:format)
  sync_ldap_roles_api_users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"users",false]],[7,"/",false]],[6,"sync_ldap_roles",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_management => /sync_management/:id(.:format)
  sync_management_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_management",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_management_index => /sync_management/index(.:format)
  sync_management_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_management",false]],[7,"/",false]],[6,"index",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_management_manage => /sync_management/manage(.:format)
  sync_management_manage_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_management",false]],[7,"/",false]],[6,"manage",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_management_product_status => /sync_management/product_status(.:format)
  sync_management_product_status_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_management",false]],[7,"/",false]],[6,"product_status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_management_sync => /sync_management/sync(.:format)
  sync_management_sync_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_management",false]],[7,"/",false]],[6,"sync",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_management_sync_status => /sync_management/sync_status(.:format)
  sync_management_sync_status_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_management",false]],[7,"/",false]],[6,"sync_status",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_plan => /sync_plans/:id(.:format)
  sync_plan_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_plans",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_plan_api_organization_product => /api/organizations/:organization_id/products/:id/sync_plan(.:format)
  sync_plan_api_organization_product_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"sync_plan",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_plan_api_product => /api/products/:id/sync_plan(.:format)
  sync_plan_api_product_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"sync_plan",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_plans => /sync_plans(.:format)
  sync_plans_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"sync_plans",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_schedules_apply => /sync_schedules/apply(.:format)
  sync_schedules_apply_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_schedules",false]],[7,"/",false]],[6,"apply",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// sync_schedules_index => /sync_schedules/index(.:format)
  sync_schedules_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"sync_schedules",false]],[7,"/",false]],[6,"index",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system => /systems/:id(.:format)
  system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_errata => /systems/:system_id/errata(.:format)
  system_errata_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_erratum => /systems/:system_id/errata/:id(.:format)
  system_erratum_path: function(_system_id, _id, options) {
  return Utils.build_path(["system_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"errata",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_event => /systems/:system_id/events/:id(.:format)
  system_event_path: function(_system_id, _id, options) {
  return Utils.build_path(["system_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_events => /systems/:system_id/events(.:format)
  system_events_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"events",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_group => /system_groups/:id(.:format)
  system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_group_errata => /system_groups/:system_group_id/errata(.:format)
  system_group_errata_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_group_event => /system_groups/:system_group_id/events/:id(.:format)
  system_group_event_path: function(_system_group_id, _id, options) {
  return Utils.build_path(["system_group_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"events",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_group_events => /system_groups/:system_group_id/events(.:format)
  system_group_events_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"events",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_group_packages => /system_groups/:system_group_id/packages(.:format)
  system_group_packages_path: function(_system_group_id, options) {
  return Utils.build_path(["system_group_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"system_group_id",false]],[7,"/",false]],[6,"packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_groups => /system_groups(.:format)
  system_groups_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_groups_activation_key => /activation_keys/:id/system_groups(.:format)
  system_groups_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_groups_api_organization_activation_key => /api/organizations/:organization_id/activation_keys/:id/system_groups(.:format)
  system_groups_api_organization_activation_key_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_groups_api_system => /api/systems/:id/system_groups(.:format)
  system_groups_api_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_groups_dashboard_index => /dashboard/system_groups(.:format)
  system_groups_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_groups_system => /systems/:id/system_groups(.:format)
  system_groups_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"system_groups",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// system_system_packages => /systems/:system_id/system_packages(.:format)
  system_system_packages_path: function(_system_id, options) {
  return Utils.build_path(["system_id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"system_id",false]],[7,"/",false]],[6,"system_packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// systems => /systems(.:format)
  systems_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// systems_activation_key => /activation_keys/:id/systems(.:format)
  systems_activation_key_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"activation_keys",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// systems_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/systems(.:format)
  systems_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// systems_api_system_group => /api/system_groups/:id/systems(.:format)
  systems_api_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// systems_dashboard_index => /dashboard/systems(.:format)
  systems_dashboard_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"dashboard",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// systems_system_group => /system_groups/:id/systems(.:format)
  systems_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// tasks_api_organization_systems => /api/organizations/:organization_id/systems/tasks(.:format)
  tasks_api_organization_systems_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"systems",false]],[7,"/",false]],[6,"tasks",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_component_views_content_view_definition => /content_view_definitions/:id/update_component_views(.:format)
  update_component_views_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_component_views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_content_content_view_definition => /content_view_definitions/:id/update_content(.:format)
  update_content_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_content",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_environment_user => /users/:id/update_environment(.:format)
  update_environment_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_environment",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_locale_user => /users/:id/update_locale(.:format)
  update_locale_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_locale",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_parameter_content_view_definition_filter_rule => /content_view_definitions/:content_view_definition_id/filters/:filter_id/rules/:id/update_parameter(.:format)
  update_parameter_content_view_definition_filter_rule_path: function(_content_view_definition_id, _filter_id, _id, options) {
  return Utils.build_path(["content_view_definition_id","filter_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"content_view_definition_id",false]],[7,"/",false]],[6,"filters",false]],[7,"/",false]],[3,"filter_id",false]],[7,"/",false]],[6,"rules",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_parameter",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_preference_user => /users/:id/update_preference(.:format)
  update_preference_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_preference",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_repo_gpg_key_provider_product_repository => /providers/:provider_id/products/:product_id/repositories/:id/update_gpg_key(.:format)
  update_repo_gpg_key_provider_product_repository_path: function(_provider_id, _product_id, _id, options) {
  return Utils.build_path(["provider_id","product_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"providers",false]],[7,"/",false]],[3,"provider_id",false]],[7,"/",false]],[6,"products",false]],[7,"/",false]],[3,"product_id",false]],[7,"/",false]],[6,"repositories",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_gpg_key",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_roles_user => /users/:id/update_roles(.:format)
  update_roles_user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_roles",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_subscriptions_distributor => /distributors/:id/update_subscriptions(.:format)
  update_subscriptions_distributor_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"distributors",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_subscriptions_system => /systems/:id/update_subscriptions(.:format)
  update_subscriptions_system_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"systems",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_subscriptions",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_systems_api_organization_system_group => /api/organizations/:organization_id/system_groups/:id/update_systems(.:format)
  update_systems_api_organization_system_group_path: function(_organization_id, _id, options) {
  return Utils.build_path(["organization_id","id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"api",false]],[7,"/",false]],[6,"organizations",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// update_systems_system_group => /system_groups/:id/update_systems(.:format)
  update_systems_system_group_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"update_systems",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// upload_subscriptions => /subscriptions/upload(.:format)
  upload_subscriptions_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"subscriptions",false]],[7,"/",false]],[6,"upload",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// user => /users/:id(.:format)
  user_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"users",false]],[7,"/",false]],[3,"id",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// user_session => /user_session(.:format)
  user_session_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"user_session",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// user_session_logout => /user_session/logout(.:format)
  user_session_logout_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"user_session",false]],[7,"/",false]],[6,"logout",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// users => /users(.:format)
  users_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[7,"/",false],[6,"users",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// validate_name_library_packages => /packages/validate_name_library(.:format)
  validate_name_library_packages_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"packages",false]],[7,"/",false]],[6,"validate_name_library",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// validate_name_system_groups => /system_groups/validate_name(.:format)
  validate_name_system_groups_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"system_groups",false]],[7,"/",false]],[6,"validate_name",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// verbs_and_scopes => /roles/:organization_id/resource_type/verbs_and_scopes(.:format)
  verbs_and_scopes_path: function(_organization_id, options) {
  return Utils.build_path(["organization_id"], ["format"], [2,[2,[2,[2,[2,[2,[2,[2,[7,"/",false],[6,"roles",false]],[7,"/",false]],[3,"organization_id",false]],[7,"/",false]],[6,"resource_type",false]],[7,"/",false]],[6,"verbs_and_scopes",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// view_compare_errata_content_search_index => /content_search/view_compare_errata(.:format)
  view_compare_errata_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"view_compare_errata",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// view_compare_packages_content_search_index => /content_search/view_compare_packages(.:format)
  view_compare_packages_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"view_compare_packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// view_packages_content_search_index => /content_search/view_packages(.:format)
  view_packages_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"view_packages",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// views_content_search_index => /content_search/views(.:format)
  views_content_search_index_path: function(options) {
  return Utils.build_path([], ["format"], [2,[2,[2,[2,[7,"/",false],[6,"content_search",false]],[7,"/",false]],[6,"views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  },
// views_content_view_definition => /content_view_definitions/:id/views(.:format)
  views_content_view_definition_path: function(_id, options) {
  return Utils.build_path(["id"], ["format"], [2,[2,[2,[2,[2,[2,[7,"/",false],[6,"content_view_definitions",false]],[7,"/",false]],[3,"id",false]],[7,"/",false]],[6,"views",false]],[1,[2,[8,".",false],[3,"format",false]],false]], arguments);
  }}
;

  window.KT.routes.options = defaults;

}).call(this);
// Chosen, a Select Box Enhancer for jQuery and Protoype
// by Patrick Filler for Harvest, http://getharvest.com
//
// Version 0.9.5
// Full source at https://github.com/harvesthq/chosen
// Copyright (c) 2011 Harvest http://getharvest.com

// MIT License, https://github.com/harvesthq/chosen/blob/master/LICENSE.md
// This file is generated by `cake build`, do not edit it by hand.


// Modified to support a new option custom_compare, at some point we should go back and add this
// to the coffeescript version
(function() {
  var SelectParser;
  SelectParser = (function() {
    function SelectParser() {
      this.options_index = 0;
      this.parsed = [];
    }
    SelectParser.prototype.add_node = function(child) {
      if (child.nodeName === "OPTGROUP") {
        return this.add_group(child);
      } else {
        return this.add_option(child);
      }
    };
    SelectParser.prototype.add_group = function(group) {
      var group_position, option, _i, _len, _ref, _results;
      group_position = this.parsed.length;
      this.parsed.push({
        array_index: group_position,
        group: true,
        label: group.label,
        children: 0,
        disabled: group.disabled
      });
      _ref = group.childNodes;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        option = _ref[_i];
        _results.push(this.add_option(option, group_position, group.disabled));
      }
      return _results;
    };
    SelectParser.prototype.add_option = function(option, group_position, group_disabled) {
      if (option.nodeName === "OPTION") {
        if (option.text !== "") {
          if (group_position != null) {
            this.parsed[group_position].children += 1;
          }
          this.parsed.push({
            array_index: this.parsed.length,
            options_index: this.options_index,
            value: option.value,
            text: option.text,
            html: option.innerHTML,
            selected: option.selected,
            disabled: group_disabled === true ? group_disabled : option.disabled,
            group_array_index: group_position,
            classes: option.className,
            style: option.style.cssText
          });
        } else {
          this.parsed.push({
            array_index: this.parsed.length,
            options_index: this.options_index,
            empty: true
          });
        }
        return this.options_index += 1;
      }
    };
    return SelectParser;
  })();
  SelectParser.select_to_array = function(select) {
    var child, parser, _i, _len, _ref;
    parser = new SelectParser();
    _ref = select.childNodes;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      child = _ref[_i];
      parser.add_node(child);
    }
    return parser.parsed;
  };
  this.SelectParser = SelectParser;
}).call(this);
(function() {
  /*
  Chosen source: generate output using 'cake build'
  Copyright (c) 2011 by Harvest
  */
  var AbstractChosen, root;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  root = this;
  AbstractChosen = (function() {
    function AbstractChosen(form_field, options) {
      this.form_field = form_field;
      this.options = options != null ? options : {};
      this.set_default_values();
      this.is_multiple = this.form_field.multiple;
      this.default_text_default = this.is_multiple ? "Select Some Options" : "Select an Option";
      this.setup();
      this.set_up_html();
      this.register_observers();
      this.finish_setup();
    }
    AbstractChosen.prototype.set_default_values = function() {
      this.click_test_action = __bind(function(evt) {
        return this.test_active_click(evt);
      }, this);
      this.activate_action = __bind(function(evt) {
        return this.activate_field(evt);
      }, this);
      this.active_field = false;
      this.mouse_on_container = false;
      this.results_showing = false;
      this.result_highlighted = null;
      this.result_single_selected = null;
      this.allow_single_deselect = (this.options.allow_single_deselect != null) && this.form_field.options[0].text === "" ? this.options.allow_single_deselect : false;
      this.disable_search_threshold = this.options.disable_search_threshold || 0;
      this.choices = 0;
      return this.results_none_found = this.options.no_results_text || "No results match";
    };
    AbstractChosen.prototype.mouse_enter = function() {
      return this.mouse_on_container = true;
    };
    AbstractChosen.prototype.mouse_leave = function() {
      return this.mouse_on_container = false;
    };
    AbstractChosen.prototype.input_focus = function(evt) {
      if (!this.active_field) {
        return setTimeout((__bind(function() {
          return this.container_mousedown();
        }, this)), 50);
      }
    };
    AbstractChosen.prototype.input_blur = function(evt) {
      if (!this.mouse_on_container) {
        this.active_field = false;
        return setTimeout((__bind(function() {
          return this.blur_test();
        }, this)), 100);
      }
    };
    AbstractChosen.prototype.result_add_option = function(option) {
      var classes, style;
      if (!option.disabled) {
        option.dom_id = this.container_id + "_o_" + option.array_index;
        classes = option.selected && this.is_multiple ? [] : ["active-result"];
        if (option.selected) {
          classes.push("result-selected");
        }
        if (option.group_array_index != null) {
          classes.push("group-option");
        }
        if (option.classes !== "") {
          classes.push(option.classes);
        }
        style = option.style.cssText !== "" ? " style=\"" + option.style + "\"" : "";
        return '<li id="' + option.dom_id + '" class="' + classes.join(' ') + '"' + style + '>' + option.html + '</li>';
      } else {
        return "";
      }
    };
    AbstractChosen.prototype.results_update_field = function() {
      this.result_clear_highlight();
      this.result_single_selected = null;
      return this.results_build();
    };
    AbstractChosen.prototype.results_toggle = function() {
      if (this.results_showing) {
        return this.results_hide();
      } else {
        return this.results_show();
      }
    };
    AbstractChosen.prototype.results_search = function(evt) {
      if (this.results_showing) {
        return this.winnow_results();
      } else {
        return this.results_show();
      }
    };
    AbstractChosen.prototype.keyup_checker = function(evt) {
      var stroke, _ref;
      stroke = (_ref = evt.which) != null ? _ref : evt.keyCode;
      this.search_field_scale();
      switch (stroke) {
        case 8:
          if (this.is_multiple && this.backstroke_length < 1 && this.choices > 0) {
            return this.keydown_backstroke();
          } else if (!this.pending_backstroke) {
            this.result_clear_highlight();
            return this.results_search();
          }
          break;
        case 13:
          evt.preventDefault();
          if (this.results_showing) {
            return this.result_select(evt);
          }
          break;
        case 27:
          if (this.results_showing) {
            return this.results_hide();
          }
          break;
        case 9:
        case 38:
        case 40:
        case 16:
        case 91:
        case 17:
          break;
        default:
          return this.results_search();
      }
    };
    AbstractChosen.prototype.generate_field_id = function() {
      var new_id;
      new_id = this.generate_random_id();
      this.form_field.id = new_id;
      return new_id;
    };
    AbstractChosen.prototype.generate_random_char = function() {
      var chars, newchar, rand;
      chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZ";
      rand = Math.floor(Math.random() * chars.length);
      return newchar = chars.substring(rand, rand + 1);
    };
    return AbstractChosen;
  })();
  root.AbstractChosen = AbstractChosen;
}).call(this);
(function() {
  /*
  Chosen source: generate output using 'cake build'
  Copyright (c) 2011 by Harvest
  */
  var $, Chosen, get_side_border_padding, root;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  root = this;
  $ = jQuery;
  $.fn.extend({
    chosen: function(options) {
      if ($.browser.msie && ($.browser.version === "6.0" || $.browser.version === "7.0")) {
        return this;
      }
      return $(this).each(function(input_field) {
        if (!($(this)).hasClass("chzn-done")) {
          return new Chosen(this, options);
        }
      });
    }
  });
  Chosen = (function() {
    __extends(Chosen, AbstractChosen);
    function Chosen() {
      Chosen.__super__.constructor.apply(this, arguments);
    }
    Chosen.prototype.setup = function() {
      this.form_field_jq = $(this.form_field);
      return this.is_rtl = this.form_field_jq.hasClass("chzn-rtl");
    };
    Chosen.prototype.finish_setup = function() {
      return this.form_field_jq.addClass("chzn-done");
    };
    Chosen.prototype.set_up_html = function() {
      var container_div, dd_top, dd_width, sf_width;
      this.container_id = this.form_field.id.length ? this.form_field.id.replace(/(:|\.)/g, '_') : this.generate_field_id();
      this.container_id += "_chzn";
      this.f_width = this.form_field_jq.outerWidth();
      this.default_text = this.form_field_jq.data('placeholder') ? this.form_field_jq.data('placeholder') : this.default_text_default;
      container_div = $("<div />", {
        id: this.container_id,
        "class": "chzn-container" + (this.is_rtl ? ' chzn-rtl' : ''),
        style: 'width: ' + this.f_width + 'px;'
      });
      if (this.is_multiple) {
        container_div.html('<ul class="chzn-choices"><li class="search-field"><input type="text" value="' + this.default_text + '" class="default" autocomplete="off" style="width:25px;" /></li></ul><div class="chzn-drop" style="left:-9000px;"><ul class="chzn-results"></ul></div>');
      } else {
        container_div.html('<a href="javascript:void(0)" class="chzn-single"><span>' + this.default_text + '</span><div><b></b></div></a><div class="chzn-drop" style="left:-9000px;"><div class="chzn-search"><input type="text" autocomplete="off" /></div><ul class="chzn-results"></ul></div>');
      }
      this.form_field_jq.hide().after(container_div);
      this.container = $('#' + this.container_id);
      this.container.addClass("chzn-container-" + (this.is_multiple ? "multi" : "single"));
      if (!this.is_multiple && this.form_field.options.length <= this.disable_search_threshold) {
        this.container.addClass("chzn-container-single-nosearch");
      }
      this.dropdown = this.container.find('div.chzn-drop').first();
      dd_top = this.container.height();
      dd_width = this.f_width - get_side_border_padding(this.dropdown);
      this.dropdown.css({
        "width": dd_width + "px",
        "top": dd_top + "px"
      });
      this.search_field = this.container.find('input').first();
      this.search_results = this.container.find('ul.chzn-results').first();
      this.search_field_scale();
      this.search_no_results = this.container.find('li.no-results').first();
      if (this.is_multiple) {
        this.search_choices = this.container.find('ul.chzn-choices').first();
        this.search_container = this.container.find('li.search-field').first();
      } else {
        this.search_container = this.container.find('div.chzn-search').first();
        this.selected_item = this.container.find('.chzn-single').first();
        sf_width = dd_width - get_side_border_padding(this.search_container) - get_side_border_padding(this.search_field);
        this.search_field.css({
          "width": sf_width + "px"
        });
      }
      this.results_build();
      return this.set_tab_index();
    };
    Chosen.prototype.register_observers = function() {
      this.container.mousedown(__bind(function(evt) {
        return this.container_mousedown(evt);
      }, this));
      this.container.mouseup(__bind(function(evt) {
        return this.container_mouseup(evt);
      }, this));
      this.container.mouseenter(__bind(function(evt) {
        return this.mouse_enter(evt);
      }, this));
      this.container.mouseleave(__bind(function(evt) {
        return this.mouse_leave(evt);
      }, this));
      this.search_results.mouseup(__bind(function(evt) {
        return this.search_results_mouseup(evt);
      }, this));
      this.search_results.mouseover(__bind(function(evt) {
        return this.search_results_mouseover(evt);
      }, this));
      this.search_results.mouseout(__bind(function(evt) {
        return this.search_results_mouseout(evt);
      }, this));
      this.form_field_jq.bind("liszt:updated", __bind(function(evt) {
        return this.results_update_field(evt);
      }, this));
      this.search_field.blur(__bind(function(evt) {
        return this.input_blur(evt);
      }, this));
      this.search_field.keyup(__bind(function(evt) {
        return this.keyup_checker(evt);
      }, this));
      this.search_field.keydown(__bind(function(evt) {
        return this.keydown_checker(evt);
      }, this));
      if (this.is_multiple) {
        this.search_choices.click(__bind(function(evt) {
          return this.choices_click(evt);
        }, this));
        return this.search_field.focus(__bind(function(evt) {
          return this.input_focus(evt);
        }, this));
      }
    };
    Chosen.prototype.search_field_disabled = function() {
      this.is_disabled = this.form_field_jq.attr('disabled');
      if (this.is_disabled) {
        this.container.addClass('chzn-disabled');
        this.search_field.attr('disabled', true);
        if (!this.is_multiple) {
          this.selected_item.unbind("focus", this.activate_action);
        }
        return this.close_field();
      } else {
        this.container.removeClass('chzn-disabled');
        this.search_field.attr('disabled', false);
        if (!this.is_multiple) {
          return this.selected_item.bind("focus", this.activate_action);
        }
      }
    };
    Chosen.prototype.container_mousedown = function(evt) {
      var target_closelink;
      if (!this.is_disabled) {
        target_closelink = evt != null ? ($(evt.target)).hasClass("search-choice-close") : false;
        if (evt && evt.type === "mousedown") {
          evt.stopPropagation();
        }
        if (!this.pending_destroy_click && !target_closelink) {
          if (!this.active_field) {
            if (this.is_multiple) {
              this.search_field.val("");
            }
            $(document).click(this.click_test_action);
            this.results_show();
          } else if (!this.is_multiple && evt && ($(evt.target) === this.selected_item || $(evt.target).parents("a.chzn-single").length)) {
            evt.preventDefault();
            this.results_toggle();
          }
          return this.activate_field();
        } else {
          return this.pending_destroy_click = false;
        }
      }
    };
    Chosen.prototype.container_mouseup = function(evt) {
      if (evt.target.nodeName === "ABBR") {
        return this.results_reset(evt);
      }
    };
    Chosen.prototype.blur_test = function(evt) {
      if (!this.active_field && this.container.hasClass("chzn-container-active")) {
        return this.close_field();
      }
    };
    Chosen.prototype.close_field = function() {
      $(document).unbind("click", this.click_test_action);
      if (!this.is_multiple) {
        this.selected_item.attr("tabindex", this.search_field.attr("tabindex"));
        this.search_field.attr("tabindex", -1);
      }
      this.active_field = false;
      this.results_hide();
      this.container.removeClass("chzn-container-active");
      this.winnow_results_clear();
      this.clear_backstroke();
      this.show_search_field_default();
      return this.search_field_scale();
    };
    Chosen.prototype.activate_field = function() {
      if (!this.is_multiple && !this.active_field) {
        this.search_field.attr("tabindex", this.selected_item.attr("tabindex"));
        this.selected_item.attr("tabindex", -1);
      }
      this.container.addClass("chzn-container-active");
      this.active_field = true;
      this.search_field.val(this.search_field.val());
      return this.search_field.focus();
    };
    Chosen.prototype.test_active_click = function(evt) {
      if ($(evt.target).parents('#' + this.container_id).length) {
        return this.active_field = true;
      } else {
        return this.close_field();
      }
    };
    Chosen.prototype.results_build = function() {
      var content, data, startTime, _i, _len, _ref;
      startTime = new Date();
      this.parsing = true;
      this.results_data = root.SelectParser.select_to_array(this.form_field);
      if (this.is_multiple && this.choices > 0) {
        this.search_choices.find("li.search-choice").remove();
        this.choices = 0;
      } else if (!this.is_multiple) {
        this.selected_item.find("span").text(this.default_text);
      }
      content = '';
      _ref = this.results_data;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        data = _ref[_i];
        if (data.group) {
          content += this.result_add_group(data);
        } else if (!data.empty) {
          content += this.result_add_option(data);
          if (data.selected && this.is_multiple) {
            this.choice_build(data);
          } else if (data.selected && !this.is_multiple) {
            this.selected_item.find("span").text(data.text);
            if (this.allow_single_deselect) {
              this.single_deselect_control_build();
            }
          }
        }
      }
      this.search_field_disabled();
      this.show_search_field_default();
      this.search_field_scale();
      this.search_results.html(content);
      return this.parsing = false;
    };
    Chosen.prototype.result_add_group = function(group) {
      if (!group.disabled) {
        group.dom_id = this.container_id + "_o_" + group.array_index;
        return '<li id="' + group.dom_id + '" class="group-result">' + $("<div />").text(group.label).html() + '</li>';
      } else {
        return "";
      }
    };
    Chosen.prototype.result_do_highlight = function(el) {
      var high_bottom, high_top, maxHeight, visible_bottom, visible_top;
      if (el.length) {
        this.result_clear_highlight();
        this.result_highlight = el;
        this.result_highlight.addClass("highlighted");
        maxHeight = parseInt(this.search_results.css("maxHeight"), 10);
        visible_top = this.search_results.scrollTop();
        visible_bottom = maxHeight + visible_top;
        high_top = this.result_highlight.position().top + this.search_results.scrollTop();
        high_bottom = high_top + this.result_highlight.outerHeight();
        if (high_bottom >= visible_bottom) {
          return this.search_results.scrollTop((high_bottom - maxHeight) > 0 ? high_bottom - maxHeight : 0);
        } else if (high_top < visible_top) {
          return this.search_results.scrollTop(high_top);
        }
      }
    };
    Chosen.prototype.result_clear_highlight = function() {
      if (this.result_highlight) {
        this.result_highlight.removeClass("highlighted");
      }
      return this.result_highlight = null;
    };
    Chosen.prototype.results_show = function() {
      var dd_top;
      if (!this.is_multiple) {
        this.selected_item.addClass("chzn-single-with-drop");
        if (this.result_single_selected) {
          this.result_do_highlight(this.result_single_selected);
        }
      }
      dd_top = this.is_multiple ? this.container.height() : this.container.height() - 1;
      this.dropdown.css({
        "top": dd_top + "px",
        "left": 0
      });
      this.results_showing = true;
      this.search_field.focus();
      this.search_field.val(this.search_field.val());
      return this.winnow_results();
    };
    Chosen.prototype.results_hide = function() {
      if (!this.is_multiple) {
        this.selected_item.removeClass("chzn-single-with-drop");
      }
      this.result_clear_highlight();
      this.dropdown.css({
        "left": "-9000px"
      });
      return this.results_showing = false;
    };
    Chosen.prototype.set_tab_index = function(el) {
      var ti;
      if (this.form_field_jq.attr("tabindex")) {
        ti = this.form_field_jq.attr("tabindex");
        this.form_field_jq.attr("tabindex", -1);
        if (this.is_multiple) {
          return this.search_field.attr("tabindex", ti);
        } else {
          this.selected_item.attr("tabindex", ti);
          return this.search_field.attr("tabindex", -1);
        }
      }
    };
    Chosen.prototype.show_search_field_default = function() {
      if (this.is_multiple && this.choices < 1 && !this.active_field) {
        this.search_field.val(this.default_text);
        return this.search_field.addClass("default");
      } else {
        this.search_field.val("");
        return this.search_field.removeClass("default");
      }
    };
    Chosen.prototype.search_results_mouseup = function(evt) {
      var target;
      target = $(evt.target).hasClass("active-result") ? $(evt.target) : $(evt.target).parents(".active-result").first();
      if (target.length) {
        this.result_highlight = target;
        return this.result_select(evt);
      }
    };
    Chosen.prototype.search_results_mouseover = function(evt) {
      var target;
      target = $(evt.target).hasClass("active-result") ? $(evt.target) : $(evt.target).parents(".active-result").first();
      if (target) {
        return this.result_do_highlight(target);
      }
    };
    Chosen.prototype.search_results_mouseout = function(evt) {
      if ($(evt.target).hasClass("active-result" || $(evt.target).parents('.active-result').first())) {
        return this.result_clear_highlight();
      }
    };
    Chosen.prototype.choices_click = function(evt) {
      evt.preventDefault();
      if (this.active_field && !($(evt.target).hasClass("search-choice" || $(evt.target).parents('.search-choice').first)) && !this.results_showing) {
        return this.results_show();
      }
    };
    Chosen.prototype.choice_build = function(item) {
      var choice_id, link;
      choice_id = this.container_id + "_c_" + item.array_index;
      this.choices += 1;
      this.search_container.before('<li class="search-choice" id="' + choice_id + '"><span>' + item.html + '</span><a href="javascript:void(0)" class="search-choice-close" rel="' + item.array_index + '"></a></li>');
      link = $('#' + choice_id).find("a").first();
      return link.click(__bind(function(evt) {
        return this.choice_destroy_link_click(evt);
      }, this));
    };
    Chosen.prototype.choice_destroy_link_click = function(evt) {
      evt.preventDefault();
      if (!this.is_disabled) {
        this.pending_destroy_click = true;
        return this.choice_destroy($(evt.target));
      } else {
        return evt.stopPropagation;
      }
    };
    Chosen.prototype.choice_destroy = function(link) {
      this.choices -= 1;
      this.show_search_field_default();
      if (this.is_multiple && this.choices > 0 && this.search_field.val().length < 1) {
        this.results_hide();
      }
      this.result_deselect(link.attr("rel"));
      return link.parents('li').first().remove();
    };
    Chosen.prototype.results_reset = function(evt) {
      this.form_field.options[0].selected = true;
      this.selected_item.find("span").text(this.default_text);
      this.show_search_field_default();
      $(evt.target).remove();
      this.form_field_jq.trigger("change");
      if (this.active_field) {
        return this.results_hide();
      }
    };
    Chosen.prototype.result_select = function(evt) {
      var high, high_id, item, position;
      if (this.result_highlight) {
        high = this.result_highlight;
        high_id = high.attr("id");
        this.result_clear_highlight();
        if (this.is_multiple) {
          this.result_deactivate(high);
        } else {
          this.search_results.find(".result-selected").removeClass("result-selected");
          this.result_single_selected = high;
        }
        high.addClass("result-selected");
        position = high_id.substr(high_id.lastIndexOf("_") + 1);
        item = this.results_data[position];
        item.selected = true;
        this.form_field.options[item.options_index].selected = true;
        if (this.is_multiple) {
          this.choice_build(item);
        } else {
          this.selected_item.find("span").first().text(item.text);
          if (this.allow_single_deselect) {
            this.single_deselect_control_build();
          }
        }
        if (!(evt.metaKey && this.is_multiple)) {
          this.results_hide();
        }
        this.search_field.val("");
        this.form_field_jq.trigger("change");
        return this.search_field_scale();
      }
    };
    Chosen.prototype.result_activate = function(el) {
      return el.addClass("active-result");
    };
    Chosen.prototype.result_deactivate = function(el) {
      return el.removeClass("active-result");
    };
    Chosen.prototype.result_deselect = function(pos) {
      var result, result_data;
      result_data = this.results_data[pos];
      result_data.selected = false;
      this.form_field.options[result_data.options_index].selected = false;
      result = $("#" + this.container_id + "_o_" + pos);
      result.removeClass("result-selected").addClass("active-result").show();
      this.result_clear_highlight();
      this.winnow_results();
      this.form_field_jq.trigger("change");
      return this.search_field_scale();
    };
    Chosen.prototype.single_deselect_control_build = function() {
      if (this.allow_single_deselect && this.selected_item.find("abbr").length < 1) {
        return this.selected_item.find("span").first().after("<abbr class=\"search-choice-close\"></abbr>");
      }
    };
    Chosen.prototype.winnow_results = function() {
      var found, option, part, parts, regex, result_id, results, searchText, startTime, startpos, text, zregex, _i, _j, _len, _len2, _ref;
      startTime = new Date();
      this.no_results_clear();
      results = 0;
      searchText = this.search_field.val() === this.default_text ? "" : $('<div/>').text($.trim(this.search_field.val())).html();
      regex = new RegExp('^' + searchText.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"), 'i');
      zregex = new RegExp(searchText.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"), 'i');
      _ref = this.results_data;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        option = _ref[_i];
        if (!option.disabled && !option.empty) {
          if (option.group) {
            $('#' + option.dom_id).hide();
          } else if (!(this.is_multiple && option.selected)) {
            found = false;
            result_id = option.dom_id;


            if(this.options.custom_compare) {
                var item = this.results_data[option.dom_id.substr(option.dom_id.lastIndexOf("_") + 1)].value;
                if (this.options.custom_compare(searchText, option.html, item )){
                    found = true;
                    results += 1;
                }
            }
            if (regex.test(option.html)) {
              found = true;
              results += 1;
            }
            else if (option.html.indexOf(" ") >= 0 || option.html.indexOf("[") === 0) {
              parts = option.html.replace(/\[|\]/g, "").split(" ");
              if (parts.length) {
                for (_j = 0, _len2 = parts.length; _j < _len2; _j++) {
                  part = parts[_j];
                  if (regex.test(part)) {
                    found = true;
                    results += 1;
                  }
                }
              }
            }
            if (found) {
//              if (searchText.length) {
//                startpos = option.html.search(zregex);
//                text = option.html.substr(0, startpos + searchText.length) + '</em>' + option.html.substr(startpos + searchText.length);
//                text = text.substr(0, startpos) + '<em>' + text.substr(startpos);
//              } else {
                text = option.html;
//              }
              if ($("#" + result_id).html !== text) {
                $("#" + result_id).html(text);
              }
              this.result_activate($("#" + result_id));
              if (option.group_array_index != null) {
                $("#" + this.results_data[option.group_array_index].dom_id).show();
              }
            } else {
              if (this.result_highlight && result_id === this.result_highlight.attr('id')) {
                this.result_clear_highlight();
              }
              this.result_deactivate($("#" + result_id));
            }
          }
        }
      }
      if (results < 1 && searchText.length) {
        return this.no_results(searchText);
      } else {
        return this.winnow_results_set_highlight();
      }
    };
    Chosen.prototype.winnow_results_clear = function() {
      var li, lis, _i, _len, _results;
      this.search_field.val("");
      lis = this.search_results.find("li");
      _results = [];
      for (_i = 0, _len = lis.length; _i < _len; _i++) {
        li = lis[_i];
        li = $(li);
        _results.push(li.hasClass("group-result") ? li.show() : !this.is_multiple || !li.hasClass("result-selected") ? this.result_activate(li) : void 0);
      }
      return _results;
    };
    Chosen.prototype.winnow_results_set_highlight = function() {
      var do_high, selected_results;
      if (!this.result_highlight) {
        selected_results = !this.is_multiple ? this.search_results.find(".result-selected.active-result") : [];
        do_high = selected_results.length ? selected_results.first() : this.search_results.find(".active-result").first();
        if (do_high != null) {
          return this.result_do_highlight(do_high);
        }
      }
    };
    Chosen.prototype.no_results = function(terms) {
      var no_results_html;
      no_results_html = $('<li class="no-results">' + this.results_none_found + ' "<span></span>"</li>');
      no_results_html.find("span").first().html(terms);
      return this.search_results.append(no_results_html);
    };
    Chosen.prototype.no_results_clear = function() {
      return this.search_results.find(".no-results").remove();
    };
    Chosen.prototype.keydown_arrow = function() {
      var first_active, next_sib;
      if (!this.result_highlight) {
        first_active = this.search_results.find("li.active-result").first();
        if (first_active) {
          this.result_do_highlight($(first_active));
        }
      } else if (this.results_showing) {
        next_sib = this.result_highlight.nextAll("li.active-result").first();
        if (next_sib) {
          this.result_do_highlight(next_sib);
        }
      }
      if (!this.results_showing) {
        return this.results_show();
      }
    };
    Chosen.prototype.keyup_arrow = function() {
      var prev_sibs;
      if (!this.results_showing && !this.is_multiple) {
        return this.results_show();
      } else if (this.result_highlight) {
        prev_sibs = this.result_highlight.prevAll("li.active-result");
        if (prev_sibs.length) {
          return this.result_do_highlight(prev_sibs.first());
        } else {
          if (this.choices > 0) {
            this.results_hide();
          }
          return this.result_clear_highlight();
        }
      }
    };
    Chosen.prototype.keydown_backstroke = function() {
      if (this.pending_backstroke) {
        this.choice_destroy(this.pending_backstroke.find("a").first());
        return this.clear_backstroke();
      } else {
        this.pending_backstroke = this.search_container.siblings("li.search-choice").last();
        return this.pending_backstroke.addClass("search-choice-focus");
      }
    };
    Chosen.prototype.clear_backstroke = function() {
      if (this.pending_backstroke) {
        this.pending_backstroke.removeClass("search-choice-focus");
      }
      return this.pending_backstroke = null;
    };
    Chosen.prototype.keydown_checker = function(evt) {
      var stroke, _ref;
      stroke = (_ref = evt.which) != null ? _ref : evt.keyCode;
      this.search_field_scale();
      if (stroke !== 8 && this.pending_backstroke) {
        this.clear_backstroke();
      }
      switch (stroke) {
        case 8:
          this.backstroke_length = this.search_field.val().length;
          break;
        case 9:
          this.mouse_on_container = false;
          break;
        case 13:
          evt.preventDefault();
          break;
        case 38:
          evt.preventDefault();
          this.keyup_arrow();
          break;
        case 40:
          this.keydown_arrow();
          break;
      }
    };
    Chosen.prototype.search_field_scale = function() {
      var dd_top, div, h, style, style_block, styles, w, _i, _len;
      if (this.is_multiple) {
        h = 0;
        w = 0;
        style_block = "position:absolute; left: -1000px; top: -1000px; display:none;";
        styles = ['font-size', 'font-style', 'font-weight', 'font-family', 'line-height', 'text-transform', 'letter-spacing'];
        for (_i = 0, _len = styles.length; _i < _len; _i++) {
          style = styles[_i];
          style_block += style + ":" + this.search_field.css(style) + ";";
        }
        div = $('<div />', {
          'style': style_block
        });
        div.text(this.search_field.val());
        $('body').append(div);
        w = div.width() + 25;
        div.remove();
        if (w > this.f_width - 10) {
          w = this.f_width - 10;
        }
        this.search_field.css({
          'width': w + 'px'
        });
        dd_top = this.container.height();
        return this.dropdown.css({
          "top": dd_top + "px"
        });
      }
    };
    Chosen.prototype.generate_random_id = function() {
      var string;
      string = "sel" + this.generate_random_char() + this.generate_random_char() + this.generate_random_char();
      while ($("#" + string).length > 0) {
        string += this.generate_random_char();
      }
      return string;
    };
    return Chosen;
  })();
  get_side_border_padding = function(elmt) {
    var side_border_padding;
    return side_border_padding = elmt.outerWidth() - elmt.width();
  };
  root.get_side_border_padding = get_side_border_padding;
}).call(this);
//fgnass.github.com/spin.js#v1.2.7
!function(e,t,n){function o(e,n){var r=t.createElement(e||"div"),i;for(i in n)r[i]=n[i];return r}function u(e){for(var t=1,n=arguments.length;t<n;t++)e.appendChild(arguments[t]);return e}function f(e,t,n,r){var o=["opacity",t,~~(e*100),n,r].join("-"),u=.01+n/r*100,f=Math.max(1-(1-e)/t*(100-u),e),l=s.substring(0,s.indexOf("Animation")).toLowerCase(),c=l&&"-"+l+"-"||"";return i[o]||(a.insertRule("@"+c+"keyframes "+o+"{"+"0%{opacity:"+f+"}"+u+"%{opacity:"+e+"}"+(u+.01)+"%{opacity:1}"+(u+t)%100+"%{opacity:"+e+"}"+"100%{opacity:"+f+"}"+"}",a.cssRules.length),i[o]=1),o}function l(e,t){var i=e.style,s,o;if(i[t]!==n)return t;t=t.charAt(0).toUpperCase()+t.slice(1);for(o=0;o<r.length;o++){s=r[o]+t;if(i[s]!==n)return s}}function c(e,t){for(var n in t)e.style[l(e,n)||n]=t[n];return e}function h(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var i in r)e[i]===n&&(e[i]=r[i])}return e}function p(e){var t={x:e.offsetLeft,y:e.offsetTop};while(e=e.offsetParent)t.x+=e.offsetLeft,t.y+=e.offsetTop;return t}var r=["webkit","Moz","ms","O"],i={},s,a=function(){var e=o("style",{type:"text/css"});return u(t.getElementsByTagName("head")[0],e),e.sheet||e.styleSheet}(),d={lines:12,length:7,width:5,radius:10,rotate:0,corners:1,color:"#000",speed:1,trail:100,opacity:.25,fps:20,zIndex:2e9,className:"spinner",top:"auto",left:"auto",position:"relative"},v=function m(e){if(!this.spin)return new m(e);this.opts=h(e||{},m.defaults,d)};v.defaults={},h(v.prototype,{spin:function(e){this.stop();var t=this,n=t.opts,r=t.el=c(o(0,{className:n.className}),{position:n.position,width:0,zIndex:n.zIndex}),i=n.radius+n.length+n.width,u,a;e&&(e.insertBefore(r,e.firstChild||null),a=p(e),u=p(r),c(r,{left:(n.left=="auto"?a.x-u.x+(e.offsetWidth>>1):parseInt(n.left,10)+i)+"px",top:(n.top=="auto"?a.y-u.y+(e.offsetHeight>>1):parseInt(n.top,10)+i)+"px"})),r.setAttribute("aria-role","progressbar"),t.lines(r,t.opts);if(!s){var f=0,l=n.fps,h=l/n.speed,d=(1-n.opacity)/(h*n.trail/100),v=h/n.lines;(function m(){f++;for(var e=n.lines;e;e--){var i=Math.max(1-(f+e*v)%h*d,n.opacity);t.opacity(r,n.lines-e,i,n)}t.timeout=t.el&&setTimeout(m,~~(1e3/l))})()}return t},stop:function(){var e=this.el;return e&&(clearTimeout(this.timeout),e.parentNode&&e.parentNode.removeChild(e),this.el=n),this},lines:function(e,t){function i(e,r){return c(o(),{position:"absolute",width:t.length+t.width+"px",height:t.width+"px",background:e,boxShadow:r,transformOrigin:"left",transform:"rotate("+~~(360/t.lines*n+t.rotate)+"deg) translate("+t.radius+"px"+",0)",borderRadius:(t.corners*t.width>>1)+"px"})}var n=0,r;for(;n<t.lines;n++)r=c(o(),{position:"absolute",top:1+~(t.width/2)+"px",transform:t.hwaccel?"translate3d(0,0,0)":"",opacity:t.opacity,animation:s&&f(t.opacity,t.trail,n,t.lines)+" "+1/t.speed+"s linear infinite"}),t.shadow&&u(r,c(i("#000","0 0 4px #000"),{top:"2px"})),u(e,u(r,i(t.color,"0 0 1px rgba(0,0,0,.1)")));return e},opacity:function(e,t,n){t<e.childNodes.length&&(e.childNodes[t].style.opacity=n)}}),function(){function e(e,t){return o("<"+e+' xmlns="urn:schemas-microsoft.com:vml" class="spin-vml">',t)}var t=c(o("group"),{behavior:"url(#default#VML)"});!l(t,"transform")&&t.adj?(a.addRule(".spin-vml","behavior:url(#default#VML)"),v.prototype.lines=function(t,n){function s(){return c(e("group",{coordsize:i+" "+i,coordorigin:-r+" "+ -r}),{width:i,height:i})}function l(t,i,o){u(a,u(c(s(),{rotation:360/n.lines*t+"deg",left:~~i}),u(c(e("roundrect",{arcsize:n.corners}),{width:r,height:n.width,left:n.radius,top:-n.width>>1,filter:o}),e("fill",{color:n.color,opacity:n.opacity}),e("stroke",{opacity:0}))))}var r=n.length+n.width,i=2*r,o=-(n.width+n.length)*2+"px",a=c(s(),{position:"absolute",top:o,left:o}),f;if(n.shadow)for(f=1;f<=n.lines;f++)l(f,-2,"progid:DXImageTransform.Microsoft.Blur(pixelradius=2,makeshadow=1,shadowopacity=.3)");for(f=1;f<=n.lines;f++)l(f);return u(t,a)},v.prototype.opacity=function(e,t,n,r){var i=e.firstChild;r=r.shadow&&r.lines||0,i&&t+r<i.childNodes.length&&(i=i.childNodes[t+r],i=i&&i.firstChild,i=i&&i.firstChild,i&&(i.opacity=n))}):s=l(t,"animation")}(),typeof define=="function"&&define.amd?define(function(){return v}):e.Spinner=v}(window,document);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


$(document).ready(function() {

    // override textfield
    $.editable.types.text.content = function(string, settings, original) {
        $(':input:first', this).val($("<div/>").html(string).text());
    };

    // override textarea
    $.editable.types.textarea.content = function(string, settings, original) {
        $(':input:first', this).val($("<div/>").html(string).text());
    };

    $.editable.addInputType('password', {
        element : function(settings, original) {
            var input=$('<input type="password">');
            if(settings.width!=='none') {
                input.width(settings.width);
            }
            if(settings.height!=='none') {
                input.height(settings.height);
            }
            input.attr('autocomplete','off');
            $(this).append(input);
            return(input);
        }
    });

    $.editable.addInputType('custom_info', {
        element: function(settings, original) {
            var input = $("<input/>");
            $(this).append(input);
            return input;
        },
        content: function(string, settings, original) {
            var input = $(':input:first', this);
            $(input).val($("<div/>").html(string).text());
        }
    });

    // Create a custom input type for checkboxes
    $.editable.addInputType("checkbox", {
        element : function(settings, original) {
            var input = $('<input type="checkbox">');
            $(this).append(input);

            // Update <input>'s value when clicked
            $(input).click(function() {
                //var value = $(input).attr("checked") ? i18n.checkbox_yes : i18n.checkbox_no;
                var value = $(input).attr("checked") ? true : false;
                $(input).val(value);
            });
            return(input);
        },
        content : function(string, settings, original) {
            var checked = string.indexOf(i18n.checkbox_yes)!== -1 ? 1 : 0;
            var input = $(':input:first', this);
            $(input).attr("checked", checked);
            var value = $(input).attr("checked") ? i18n.checkbox_yes : i18n.checkbox_no;
            //var value = $(input).attr("checked") ? true : false;

            $(input).val(value);
        }
    });

    $.editable.addInputType('number', {
        element  :   function(settings, original){
            var width = settings.width ? settings.width : '40',
                input = jQuery('<input type="number" ' +
                    'min="' + settings.min + '"' +
                    'max="' + settings.max + '"' +
                    'value="' + settings.value +
                    '" style="width:' + width + 'px;">');
            $(this).append(input);
            if (settings.unlimited !== undefined) {
                var label = jQuery('&nbsp; <label><input type="checkbox" value=""/>&nbsp; ' + i18n.unlimited + '</label>');
                $(this).append(label);
                var unlimited = label.find("input");
                $(unlimited).bind('click', function(){
                    if($(unlimited).is(":checked")){
                        $(input).val('');
                        $(input).attr("disabled", true);
                    } else {
                        $(input).val('');
                        $(input).removeAttr('disabled');
                    }
                });
            }
            $(original).css('background-image', 'none');
            return(input);
        },

        content : function(string, settings, original) {
            var text_input = $('input', this).first();
            text_input.val(string);
            if (settings.unlimited !== undefined) {
                var check_input = $('input', this).last();
                if (string === settings.unlimited || string === i18n.unlimited) {
                    text_input.val('');
                    check_input.attr('checked', 'checked');
                    text_input.attr("disabled", true);
                } else {
                    check_input.removeAttr('checked');
                    text_input.removeAttr('disabled');
                }
            }
        },

        submit  : function(settings, original) {
            if (settings.unlimited !== undefined) {
                var text_input = $('input', this).first();
                if (text_input.val() === '') {
                    text_input.val(settings.unlimited);
                }
            }
        }
    });

    $.editable.addInputType("multiselect", {
        element: function (settings, original) {
            var select = $('<select multiple="multiple" />');

            if (settings.width !== 'none') { select.width(settings.width); }
            if (settings.size) { select.attr('size', settings.size); }

            $(this).append(select);
            return (select);
        },
        content: function (data, settings, original) {
            var json;

            /* If it is string assume it is json. */
            if (data instanceof String) {
                json = JSON.parse(data);
            } else {
                /* Otherwise assume it is a hash already. */
                json = data;
            }
            for (var key in json) {
                if (json.hasOwnProperty(key) && 'selected' !== key) {
                    var option = $('<option />').val(key).append(json[key]);
                    $('select', this).append(option);
                }
            }

            if ($(this).val() === json['selected'] ||
                $(this).html() === $.trim(original.revert)) {
                $(this).attr('selected', 'selected');
            }

            /* Loop option again to set selected. IE needed this... */
            $('select', this).children().each(function () {
                if (json.selected) {
                    var option = $(this);
                    $.each(json.selected, function (index, value) {
                        if (option.val() === value) {
                            option.attr('selected', 'selected');
                        }
                    });
                } else {
                    if (original.revert.indexOf($(this).html()) !== -1) {
                        $(this).attr('selected', 'selected');
                    }
                }
            });
        }
    });

    $.editable.addInputType( 'datepicker', {
        /* create input element */
        element: function( settings, original ) {
            var form = $( this ), input = $( '<input data-change="false"/>' );
            if (settings.width !== 'none') { input.width(settings.width); }
            if (settings.height !== 'none') { input.height(settings.height); }
            input.attr( 'autocomplete','off' );
            form.append( input );
            return input;
        },

        content : function(string, settings, original) {
            $(':input:first', this).val($.trim(string));
        },

        /* attach jquery.ui.datepicker to the input element */
        plugin: function( settings, original ) {
            var form = this, input = form.find( "input" );
            settings.onblur = 'nothing';

            datepicker = {
                // keep track of date selection state
                onSelect: function() {
                    input.attr('data-change', 'true');
                },
                // reset form if we lose focus and date was not selected
                onClose: function() {
                    if ($(this).attr('data-change') === 'false') {
                        original.reset( form );
                    }
                }
            };
            input.datepicker(datepicker).keyup(function(e) {
                if (e.keyCode === 8 || e.keyCode === 46) {
                    $.datepicker._clearDate(this);
                }
            });
        }
    });

    $.editable.addInputType( 'timepicker', {
        /* create input element */
        element: function( settings, original ) {
            var form = $( this ), input = $( '<input data-change="false"/>' );
            if (settings.width !== 'none') { input.width(settings.width); }
            if (settings.height !== 'none') { input.height(settings.height); }
            input.attr( 'autocomplete','off' );
            form.append( input );
            return input;
        },

        plugin: function( settings, original ) {
            var form = this, input = form.find( "input" );
            settings.onblur = 'ignore';
            input.timepickr({convention: 12})
                .click();
        }
    });
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/



KT.favorite = function(form, input) {
    var success =  function(data) {
            form.find(".qdropdown").html(data);

            // process the elements of the list and truncate any that are too long with ellipsis (...) (e.g. jquery.text-overflow.js)
            $(".one-line-ellipsis").ellipsis();
        },
        error =  function(data) {
        },
        save = function(event) {
            // we want to submit the request using Ajax (prevent page refresh)
            event.preventDefault();

            var newFavorite = form.find('input').attr('value');
            var url = $(this).attr('data-url');

            // send a request to the server to save/create this favorite
            $.ajax({
                type: "POST",
                url: url,
                data: {"favorite": newFavorite},
                cache: false,
                success: success,
                error: error
            });

        },
        destroy = function (data) {
            var id  = $(this).attr('data-id');
            var url = $(this).attr('data-url');

            // send a request to the server to save/create this favorite
            client_common.destroy(url, success, error);
        },
        clear = function(data){
            var search_input = form.find('input');
            search_input.val('');
            search_input.change();
            form.find('.qdropdown').hide();
        };


    return {
        save: save,
        destroy: destroy,
        clear: clear

    };
};

/**
 *
 * @param input_id  id of the search input
 * @param list_id   id of the list
 * @param list_module  module that provides the following api:
 *      replace_list(html, args) //insert a new list
 *      append(html)       //append new data to the list
 *      update_counts(current, total, results, clear) //update counts
 *      full_spinner()    //clear list and show full list spinner (for full list reload/search)
 *      current_count()  //returns # of items in list (used for offset)
 *
 * @param params   list of options:
 *          disable_fancy  - disable fancy queries
 *
 * @param extra_params list of extra parameters to pass along with search
 */
KT.search = function(form_id, list_id, list_module, params, extra_params){

    var form = $('#' + form_id),
    input = form.find('input'),
    list_elem = $('#' + list_id),
    url = params.url,
    search_key = "_search",
    search_hash = list_id + search_key,
    event_name = list_id + ".search",
    extend_event_name = list_id + ".search.extend",
    url_param = "search",
    favorite = KT.favorite(form, input),
    current_search = {},
    retrievingNewContent = false,
    trigger_name = params.trigger || "hashchange", //custom event to trigger search


    init = function(){

        $(window).scroll(extend);
        $(window).bind(trigger_name, hash_change);

        if (!params.disable_fancy) {
            form.fancyQueries();
        }

        setupSearch();
        //TODO make these classes
        form.delegate('#search_favorite_save', "click", favorite.save);
        form.delegate('#search_favorite_destroy', "click", favorite.destroy);
        form.delegate('#search_clear', "click", favorite.clear);
        form.delegate('.search_query', 'click', function(){
            input.val($(this).html());
            form.find('button').click();
            form.find('.qdropdown').hide();
        });

        reset_current_search();

    },
    search_bbq = function(){
      return search_hash;
    },
    search_event = function(){
        return event_name;
    },
    extend_event = function(){
        return extend_event_name;
    },
    hash_change = function(event, args){
        if (url){
            start_search(args);
        }
    },
    set_url = function(url_in){
        if (url !== url_in){
            reset_current_search();
            url = url_in;
        }
    },
    reset_current_search = function(){
        current_search = {};
        current_search[search_hash] = null;
    },
    refresh_search = function() {
        reset_current_search();
        start_search({});
    },
    setupSearch = function() {
        var button = form.find('button');
        button.click(function(){
            var value = input.val();
            if( button.attr('disabled') !== "disabled" ){
                if( value === "" ){
                    $.bbq.removeState(search_hash);
                } else {
                    var obj = {};
                    obj[search_hash] = value;
                    $.bbq.pushState(obj);
                }
            }
        });
        input.live('change', function(){
            if( $(this).val() === "" ){
                $.bbq.removeState(search_hash);
            }
        }).live('keypress', function(event){
                if( event.keyCode === 13 ){
                    event.preventDefault();
                    button.click();
                    return false;
                }
        });

        form.live('submit', function(e){
            e.preventDefault();
            start_search();
        });
    },
    start_search = function(args){
        var button = form.find('button'),
            data = get_params(),
            search_val = $.bbq.getState(search_hash),
            event = $.Deferred(),
            pre_state,
            params_changed = function(){
                var changed = KT.utils.all(current_search, function(item, index){
                    return item === $.bbq.getState(index);
                });
                return !changed;
            };

        if (params.pre_search_state){
            pre_state = params.pre_search_state();
        }

        //search already in process, or already searched for
        if ( !params_changed() ){
            $(document).trigger(event_name);
            return;
        }

        current_search[search_hash] = search_val;
        input.val(search_val);

        $(document).trigger(event_name, [event.promise()]);

        list_module.full_spinner();
        button.attr("disabled", "disabled");

        if(search_val) {
            data[url_param] = search_val;
        }

        if (extra_params) {
            $.each(extra_params, function(index, item){
                var item_id = item['hash_id'];

                data[item_id] = $.bbq.getState(item_id);
                current_search[item_id] = $.bbq.getState(item_id);
            });
        }

        $.ajax({
            url: url,
            data: data,
            cache: false,
            success: function (data) {
                var to_append = data.html ? data.html : data;
                list_module.replace_list(to_append, pre_state);
                button.removeAttr('disabled');
                list_module.update_counts(data['current_items'], data['total_items'], data['results_count'], true, pre_state);

                $('.ui-autocomplete').hide();
                list_elem.addClass("ajaxScroll");
                event.resolve();
            },
            error: function (e) {
                button.removeAttr('disabled');
                event.fail();
            }
        });
    },
    get_params = function(){
         return KT.panel.queryParameters();
    },
    extend = function(){
        var offset = list_module.current_count(),
            page_size = list_elem.attr("data-page_size"),
            search,
            pre_state = params.pre_search_state ? params.pre_search_state() : undefined,
            ajax_params = {
                "offset": offset
            },
            expand_list = list_elem.hasClass("expand_list") ? list_elem : list_elem.find(".expand_list");

        if ($.bbq) {
            search = $.bbq.getState(search_hash);
        }

        if (!url) {
            return;
        }

        if (list_elem.hasClass("ajaxScroll") && !retrievingNewContent && KT.common.scrollTop() >= ($(document).height() - $(window).height()) - 700) {
            retrievingNewContent = true;
            if (parseInt(page_size, 10) > parseInt(offset, 10)) {
                return; //If we have fewer items than the pagesize, don't try to fetch anything else
            }

            $.extend(ajax_params, get_params());
            if (search) {
                $.extend(ajax_params, {search:search});
            }
            expand_list.append('<div class="list-spinner"> <img src="' + KT.common.spinner_path() + '" class="ajax_scroll">  </div>');

            $.ajax({
                type: "GET",
                url: url,
                data: ajax_params,
                cache: false,
                success: function (data) {
                    retrievingNewContent = false;
                    expand_list.find('.list-spinner').remove();
                    list_module.append(data['html'], pre_state);
                    if (data['current_items'] + offset >= data["total_items"]) {
                        list_elem.removeClass("ajaxScroll");
                    }
                    list_module.update_counts(data['current_items'], 0, 0);
                    $(window).trigger(extend_event_name);
                },
                error: function () {
                    expand_list.find('.list-spinner').remove();
                    retrievingNewContent = false;
                }
            });
        }
    },
    enableAutoComplete = function(params){
        var url = params['url'],
            data = params['data'],
            request_issued = false,
            getAutoCompleteData;

            if(url) {
                getAutoCompleteData = function(request, response){
                    if( !request_issued ){
                        request_issued = true;
                        $.getJSON(url, { search    : request.term },
                            function(json){
                                request_issued = false;
                                response(json);
                            })
                            .error(function(){
                                request_issued = false;
                            });
                    }
                };
            }
            else {
                getAutoCompleteData = data;
            }
        autocomplete_override();
        input.catcomplete({
            source    : getAutoCompleteData,
            minLength: 0,
            delay    : 200,
            search    : function(event, ui) { $(".auto_complete_clear").hide(); },
            open    : function(event, ui) { $(".auto_complete_clear").show(); }
        });

        input.focus(function( event ) {
            if( $( this )[0].value === "" ) {
                $( this ).catcomplete( "search" );
            }
        });
    },
    autocomplete_override = function(){
        $.widget( "custom.catcomplete", $.ui.autocomplete, {
            _renderMenu: function( ul, items ) {
                var self = this,
                    currentCategory = "";

                $.each( items, function( index, item ) {
                    if ( item.category !== undefined && item.category !== currentCategory ) {
                        ul.append( "<li class='ui-autocomplete-category'>" + item.category + "</li>" );
                        currentCategory = item.category;
                    }
                    if ( item.error !== undefined ) {
                        ul.append( "<li class='ui-autocomplete-error'>" + item.error + "</li>" );
                    }
                    if( item.completed !== undefined ) {
                        $( "<li></li>" ).data( "item.autocomplete", item )
                            .append( "<a>" + "<strong class='ui-autocomplete-completed'>" + item.completed + "</strong>" + item.part + "</a>" )
                            .appendTo( ul );
                        } else {
                            self._renderItem( ul, item );
                        }
                });
            }
        });
    };


    init();

    return {
        enableAutoComplete    : enableAutoComplete,
        search_event : search_event,
        extend_event : extend_event,
        search_bbq   : search_bbq,
        set_url      : set_url,
        refresh_search : refresh_search
    };

};
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/
//some variables that are used throughout the panel
var thisPanel = null,
  subpanel = null,
  subpanelSpacing = 55,
  panelLeft = null,
  count = 0,
  page_width = null,
  left = null,
  right = null;

// Saves the state of the last subpanel viewed so selecting a new item from list will keep that subpanel open.
var last_ajax_panelpage;

$(document).ready(function () {
    page_width = $('.center').width();
    left = $('.left_panel');
    right = $('.right_panel');
    left.resize(function () {
        var apanel = $('.panel');
        panelLeft = left.width();
        $('.block').not('#new').width(panelLeft - 17);
        apanel.width((page_width - 21) - panelLeft);
        right.width((page_width - 62) - panelLeft);
        if (apanel.hasClass('opened')) {
            apanel.css({
                "left": (panelLeft)
            });
        }
        $('.list-title').width(panelLeft);
        $('#list-title').width(panelLeft);
        if ($(this).hasClass('column_panel_3')) {
            var fontsize = Math.floor((panelLeft / 430) * 100);
            //if it's bigger than 100%, make it 100%.
            fontsize = (fontsize > 100) ? 100 : fontsize;
            $('#systems .block').css({
                "font-size": parseInt(fontsize, 10) + "%"
            });
        }
    }).resize();
    //$('#list .block').linkHover({"timeout":200});
    thisPanel = $("#panel");
    subpanel = $('#subpanel');
    var activeBlock = null;
    var activeBlockId = null;
    var ajax_url = null;
    var panel_selected;


    $('.block').live('click', function (event) {
        var subpanel_href,
            subpanel_name,
            ajax_panelpage;

        if (event.target.nodeName === "A" && event.target.className.match('content_add_remove')) {
            return false;
        } else {
            activeBlock = $(this);
            ajax_url = activeBlock.attr("data-ajax_url");
            activeBlockId = activeBlock.attr('id');
            ajax_panelpage = activeBlock.attr("data-ajax_panelpage");

            // If the panel is currently open, get the currently open tab
            if (thisPanel.hasClass('opened') && $.bbq.getState("panel") !== "new") {
                subpanel_href = $('.panel_link.selected > a').last().attr('href');
                if (subpanel_href) {
                    last_ajax_panelpage = KT.panel.extract_panelpage(subpanel_href);
                }
            }

            if (last_ajax_panelpage === "new") {
                last_ajax_panelpage = undefined;
            }

            if((event.ctrlKey || event.metaKey) && !thisPanel.hasClass('opened') && (event.target.id !== "new") && !activeBlock.hasClass('active')) {
                if (activeBlock.hasClass('active')) {
                    activeBlock.removeClass('active');
                } else {
                    activeBlock.addClass('active');
                    activeBlock.find('.arrow-right').hide();
                }
            } else if ((event.ctrlKey || event.metaKey) && !thisPanel.hasClass('opened') && (event.target.id !== "new") && activeBlock.hasClass('active') && $('.block.active').length > 1) {
              activeBlock.removeClass('active');
            } else {
                if(activeBlock.hasClass('active') && thisPanel.hasClass('opened')){
                    KT.panel.closePanel(thisPanel);
                } else {
                    if (ajax_panelpage && $.bbq.getState("panel") !== "new" && !last_ajax_panelpage) {
                        last_ajax_panelpage = ajax_panelpage;
                    }
                    if (last_ajax_panelpage) {
                        $.bbq.pushState({
                            panel: activeBlockId,
                            panelpage: last_ajax_panelpage
                        });
                    } else {
                        $.bbq.removeState('panelpage');
                        $.bbq.pushState({
                            panel: activeBlockId
                        });
                    }
                    activeBlock.find('.arrow-right').show();
                }
            }
            //update the selected count
            KT.panel.updateResult();
            return false;
        }
    });
    $('.close').live("click", function () {
        if ($(this).attr("data-close") === "panel" || ($(this).attr("data-close") !== "subpanel" && $(this).parent().parent().hasClass('opened'))) {
            KT.panel.closePanel(thisPanel);
            KT.panel.closeSubPanel(subpanel);
        }
        else { //closing the subpanel
            KT.panel.closeSubPanel(subpanel);
        }
        return false;
    });

    $(window).resize(function () {
        KT.panel.panelResize($('#panel_main'), false);
        KT.panel.panelResize($('#subpanel_main'), true);
    });
    $('.subpanel_element').live('click', function () {
        KT.panel.openSubPanel($(this).attr('data-url'));
    });

    // It is possible for the pane (e.g. right) of a panel to contain navigation
    // links.  When that occurs, it should be possible to click the navigation
    // link and only that pane reflect the transition to the new page. The element
    // below helps to facilitate that by binding to the click event for a navigation
    // element with the specified id, sending a request to the server using the link
    // selected and then replacing the content of the pane with the response.
    $('.panel_link > a').live('click', function () {
        // if a view is a pane within a panel
        KT.panel.copy.hide_form();
        $.ajax({
            cache: 'false',
            type: 'GET',
            url: $(this).attr('href'),
            dataType: 'html',
            success: function (data) {
                var callbacks = KT.panel.get_expand_cb(),
                    cb = function(){},
                    activeBlock,
                    ajax_url,
                    ajax_panelpage;

                thisPanel.find(".panel-content").html(data);
                KT.common.jscroll_init($('.scroll-pane'));
                KT.common.jscroll_resize($('.jspPane'));
                KT.panel.panelResize($('#panel_main'), false);

                // Update the bbq
                if (!activeBlockId) {
                    activeBlockId = thisPanel.attr("id");
                }
                activeBlock = $('#' + KT.common.escapeId(activeBlockId));
                ajax_url = activeBlock.attr("data-ajax_url");
                ajax_panelpage = activeBlock.attr("data-ajax_panelpage");

                if (ajax_panelpage) {
                    // Replace old ajax_panelpage with new
                    ajax_panelpage = this.url.substr(this.url.lastIndexOf('/') + 1);
                    var bbq_panel = $.bbq.getState("panel");
                    $.bbq.removeState("panel");
                    $.bbq.removeState("panelpage");
                    $.bbq.pushState({
                        panel: bbq_panel,
                        panelpage: ajax_panelpage
                    });
                    // Set the new default panelpage
                    last_ajax_panelpage = ajax_panelpage;
                }

                KT.panel.copy.initialize();

                for( cb in callbacks ){
                    if(callbacks.hasOwnProperty(cb)) {
                        callbacks[cb]();
                    }
                }
            }
        });
        return false;
    });

    //register a common select none action
    KT.panel.actions.registerAction("select_none", {});
    $('#select-none').mouseup(function(){
        $('.block.active').removeClass('active');
        KT.panel.updateResult();
    });
    //create the initial selected count
    KT.panel.updateResult();
    //register the default actions for the page's actions partial
    KT.panel.actions.registerDefaultActions();

    $('#list-title h2').tipsy();

});

var list = (function(){
   return {
       last_child : function() {
         return $("#list").children().last();
       },
       add : function(html) {
           $('#list').append($(html).hide().fadeIn(function(){
               $(this).addClass("add", 250, function(){
                   $(this).removeClass("add", 250);
               });
           }));
           return false;
       },
       remove : function(id){
           $('#' + id).fadeOut(function(){
               $(this).empty().remove();
               KT.panel.updateResult();
           });
           return false;
       },
       refresh : function(id, url, success_cb){
           var jQid = $('#' + id);
            $.ajax({
                cache: 'false',
                type: 'GET',
                url: url,
                dataType: 'html',
                success: function(data) {
                    notices.checkNotices();
                    jQid.html(data);
                    if (success_cb) {
                        success_cb();
                    }
                }
            });
           return false;
       }
   };
})();

$(window).ready(function(){
    if ($('#container').length > 0) {
        KT.panel.registerPanel($('#panel-frame'), 0);
    }
    KT.panel.actions.resetActions();
});
KT.panel = (function ($) {
    var retrievingNewContent= false,
        control_bbq = true,
        current_scroll = 0,
        panels_list = [],
        left_list_content = "",
        expand_cb = [],
        search,
        //callback after a pane is loaded
        contract_cb = function () {},
        switch_content_cb = function () {},
        select_item = function (activeBlockId) {
            var activeBlock = $('#' + KT.common.escapeId(activeBlockId)),
                ajax_url = activeBlock.attr("data-ajax_url"),
                ajax_panelpage = activeBlock.attr("data-ajax_panelpage"),
                full_ajax_url,
                previousBlockId = null;

            if (ajax_panelpage) {
                // Initialize the default panelpage
                if (!last_ajax_panelpage) {
                    last_ajax_panelpage = ajax_panelpage;
                }
                full_ajax_url = ajax_url + '/' + last_ajax_panelpage;
            } else {
                full_ajax_url = ajax_url;
            }

            thisPanel = $("#panel");
            subpanel = $('#subpanel');
            if (activeBlock.length) {
                if (!thisPanel.hasClass('opened') && thisPanel.attr("data-id") !== activeBlockId) {
                    $('.block.active').removeClass('active');
                    // Open the Panel                           /4

                    handleScroll($('#panel-frame'));

                    thisPanel.css({
                        "z-index": "0"
                    });
                    thisPanel.parent().css({
                        "z-index": "0"
                    });
                    thisPanel.animate({
                        left: (panelLeft) + "px",
                        opacity: 1
                    }, 200, function () {
                        $(this).css({
                            "z-index": "0"
                        });
                    }).removeClass('closed').addClass('opened').attr('data-id', activeBlockId);
                    activeBlock.addClass('active');
                    previousBlockId = activeBlockId;
                    panelAjax(activeBlockId, full_ajax_url, thisPanel, false);
                }
                else if (thisPanel.hasClass('opened') && thisPanel.attr("data-id") !== activeBlockId) {
                    switch_content_cb();
                    $('.block.active').removeClass('active');
                    closeSubPanel(subpanel); //close the subpanel if it is open
                    // Keep the thisPanel open if they click another block
                    // remove previous classes besides opened
                    thisPanel.css({
                        "z-index": "0"
                    });
                    thisPanel.parent().css({
                        "z-index": "0"
                    });
                    thisPanel.addClass('opened').attr('data-id', activeBlockId);
                    $("#" + previousBlockId).removeClass('active');
                    activeBlock.addClass('active');
                    previousBlockId = activeBlockId;
                    thisPanel.removeClass('closed');
                    panelAjax(activeBlockId, full_ajax_url, thisPanel, false);
                }
            }
        },
        panelAjax = function (name, ajax_url, thisPanel, isSubpanel) {
            var spinner = thisPanel.find('.spinner'),
                panelContent = thisPanel.find(".panel-content");

            $(document).trigger('open_panel.tupane');
            spinner.show();
            KT.panel.copy.hide_form();
            panelContent.hide();

            $.ajax({
                cache: true,
                url: ajax_url,
                dataType: 'html',
                success: function (data, status, xhr) {
                    var pc = panelContent.html(data);
                    var callback;
                    spinner.hide();
                    pc.fadeIn(function () {
                        $(".panel-content :input:visible:enabled:first").focus();
                    });

                    KT.common.jscroll_init($('.scroll-pane'));
                    KT.common.jscroll_resize($('.jspPane'));

                    if (isSubpanel) {
                        panelResize($('#subpanel_main'), isSubpanel);
                    } else {
                        panelResize($('#panel_main'), isSubpanel);
                    }

                    KT.panel.copy.initialize();

                    for( callback in expand_cb ){
                        if(expand_cb.hasOwnProperty(callback)) {
                            expand_cb[callback](name);
                        }
                    }
                    // Add a handler for ellipsis
                    $(".one-line-ellipsis").ellipsis(true);
                },
                error: function (xhr, status, error) {
                    spinner.hide();
                    panelContent.html("<h2>" + i18n.error + "</h2><p>" + i18n.row_error + error + "</p>").fadeIn();
                }
            });
        },
        /* must pass a jQuery object */
        panelResize = function (paneljQ, isSubpanel) {
            if (paneljQ.length > 0) {
                adjustHeight(paneljQ, isSubpanel);
            }
            return paneljQ;
        },
        adjustHeight = function (paneljQ, isSubpanel) {
            var leftPanel = $('.left_panel'),
                tupane_panel = $('#panel'),
                header_spacing = tupane_panel.find('.head').height(),
                subnav_spacing = tupane_panel.find('nav').height() + 10,
                tupane_header = $('.tupane_header').height() || 0,
                tupane_footer = $('.tupane_footer').height() || 0,
                window_height = $(window).height(),
                container = $('#container'),
                height,
                default_height = 565,
                default_spacing = header_spacing + subnav_spacing + tupane_header + tupane_footer + 30;

            if (container.length > 0) {
                container_offset = $('#container').offset().top;
            }

            if (window_height <= (height + 80) && leftPanel.height() > 550) {
                height = window_height - container_offset - default_spacing;
            } else if( leftPanel.height() > 575 ){
                if( leftPanel.height() < window_height ){
                    height = leftPanel.height() - default_spacing;
                } else {
                    height = window_height - container_offset - default_spacing;
                }
            } else {
                height = default_height - default_spacing + 20;
            }


            if (isSubpanel) {
                //if we have a footer, lets make sure we cover it up
                if($('.tupane_footer').length > 0) {
                    height += $('.tupane_footer').outerHeight();
                }
            }


            paneljQ.height(height);

            if (paneljQ.length > 0) {
                // A corner-case in automation perhaps, but this can be reached where the 'jsp' data is
                // not present. Fencing to avoid javascript error being thrown which results in
                // jQuery.active not being properly decremented to zero (thus blocking test suite).
                // https://bugzilla.redhat.com/show_bug.cgi?id=858193
                if (paneljQ.data('jsp') !== undefined) {
                    paneljQ.data('jsp').reinitialise();
                }
            }
        },
        closePanel = function (jPanel) {
            var content,
                position;

            jPanel = jPanel || $('#panel');
            content = jPanel.find('.panel-content');
            if (jPanel.hasClass("opened")) {
                KT.panel.copy.hide_form();
                $('.block.active').removeClass('active');
                jPanel.animate({
                    left: 0,
                    opacity: 0
                }, 400, function () {
                    $(this).css({
                        "z-index": "-1"
                    });
                }).removeClass('opened').addClass('closed').attr("data-id", "");
                content.html('');
                position = KT.common.scrollTop();
                $.bbq.removeState("panel");
                $.bbq.removeState("panelpage");
                $(window).scrollTop(position);
                updateResult();
                contract_cb(name);
                closeSubPanel(subpanel);
            }
            return false;
        },
        closeSubPanel = function (jPanel) {
            if (jPanel.hasClass("opened")) {
                jPanel.animate({
                    left: 0,
                    opacity: 0
                }, 400, function () {
                    $(this).css({
                        "z-index": "-1"
                    });
                    $(this).removeClass('opened').addClass('closed');
                });
                updateResult();
            }
            return false;
        },
        updateResult = function(){
            //calc the number of active tupane rows
            var len = $('.block.active').length;
            //update the select
            $('#select-result').html(len + i18n.items_selected).effect("highlight", {}, 200);
            $('.numitems').html(len).effect("highlight", {}, 200);
            actions.resetActions(len);
            return len;
        },
        getSelected = function() {
            var to_ret = [];
            $('.block.active').each(function(){
                var id = $(this).attr("id");
                to_ret.push(id.split("_")[1]);
            });
            return to_ret;
        },
        numSelected = function() {
            return $('.block.active').length;
        },
        openSubPanel = function (url) {
            var thisPanel = $('#subpanel');
            thisPanel.animate({
                left: panelLeft + "px",
                opacity: 1
            }, 200, function () {
                $(this).css({
                    "z-index": "0"
                });
                $(this).parent().css({
                    "z-index": "0"
                });
                $(this).removeClass('closed').addClass('opened');
            });
            panelAjax('', url, thisPanel, true);
        },
        handleScroll = function (jQPanel, offset) {
            var scrollY = KT.common.scrollTop(),
                scrollX = KT.common.scrollLeft(),
                isFixed = jQPanel.css('position') === 'fixed',
                container = $('#container'),
                bodyY = parseInt(container.position().top, 10),
                left_panel = container.find('.left_panel'),
                left_bottom_pos = left_panel.offset().top + left_panel.height(),
                top;

            top_position = left_panel.offset().top;
            offset = offset ? offset : 0;
            offset += $('.center').offset().left;
            offset -= scrollX;

            if (jQPanel.length > 0) {
                if (scrollY <= container.offset().top) {
                    top = (container.offset().top - scrollY <= 30 && container.offset().top - scrollY >= -30) ? 30 : top_position - scrollY;

                    jQPanel.css({
                        position: 'fixed',
                        top: top,
                        left: offset
                    });
                }
                else {
                    if ( left_bottom_pos - (jQPanel.offset().top + jQPanel.height()) <= 40 ) {
                        jQPanel.css({
                            position: 'fixed',
                            top: (left_bottom_pos - jQPanel.height()) - scrollY,
                            left: offset
                        });
                    } else {
                        jQPanel.stop().css({
                            position: 'fixed',
                            top: 30,
                            left: offset
                        });
                    }
                }
            }
        },
        handleScrollResize = function (jQPanel, offset) {
            var scrollX = KT.common.scrollLeft();

            offset = offset ? offset : 0;
            offset += $('.center').offset().left;
            offset -= scrollX;

            if (jQPanel.length > 0) {
                if (jQPanel.css('position') === 'fixed') {
                    jQPanel.css('left', offset);
                }
            }
        },
        search_started = function (event, promise) {
            var refresh = $.bbq.getState("panel");
            if (!last_ajax_panelpage) {
                last_ajax_panelpage = $.bbq.getState("panelpage");
            }

            if (refresh) {
                if (promise) {
                    closePanel();
                    promise.done(function(){
                        $('.left_panel').resize();
                        select_item(refresh);
                    });
                }
                else {
                    select_item(refresh);
                }
            }
            else {
                closePanel();
            }
            return false;
        },
        registerPanel = function (jQPanel, offset) {
            var new_panel = {
                panel: jQPanel,
                offset: offset
            };
            $(window).scroll(function (event) {
                if( event.target === document){
                    handleScroll(jQPanel, offset);
                }
            });
            $(window).resize(function () {
                handleScrollResize(jQPanel, offset);
            });
            $(document).bind('helptip-closed', function () {
                handleScroll(jQPanel, offset);
            });
            $(document).bind('helptip-opened', function () {
                handleScroll(jQPanel, offset);
            });

            panels_list.push(new_panel);
        },
        registerSubPanelSubmit = function(form_id, form_submit_id, url_after_submit) {
            form_id.bind('ajax:beforeSend', function(){
               form_submit_id.addClass('disabled');
            }).bind("ajax:complete", function(){
               form_submit_id.removeClass('disabled');
            }).bind("ajax:success", function(){
                KT.panel.closeSubPanel($('#subpanel'));
                KT.panel.refreshPanel(url_after_submit);
            }).bind("ajax:error", function(){
               //validation notice appears
            });
        },
        // http://devnull.djolley.net/2010/11/accessing-query-string-parameters-from.html
        queryParameters = function () {
            var queryString = {};
            var qstr = window.location.search.substring(1);
            var params = qstr.split('&');
            $.each(params, function(index, item){
                var pair=item.split('=');
                if(pair[1]) {
                    queryString[pair[0]]=decodeURI(pair[1]);
                }
            });
            return queryString;
        },
        refreshPanel = function(source_url) {
            // If the user provides a 'source_url' the panel will be refreshed using that url;
            // otherwise, the source will be derived from the panel object's properties
            var active = $('#list').find('.active');
            var full_ajax_url;
            if (source_url === undefined) {
                full_ajax_url = active.attr("data-ajax_url") + '/' + active.attr("data-ajax_panelpage");
            } else {
                full_ajax_url = source_url;
            }
            KT.panel.panelAjax(active, full_ajax_url, $('#panel'), false);
        },
        extract_panelpage = function(url) {
            var a = document.createElement("a");
            a.href = url;
            var arr = a.pathname.split('/');
            var panelpage = '';
            for (var i = 4; i < arr.length; i += 1) {
                panelpage += '/' + arr[i];
            }
            return panelpage.substr(1);
        },
        actions = (function(){
            var action_list = {},
                current_request_action;

            var registerDefaultActions = function() {
                var actions = $(".panel_action");
                actions.each(function(index){
                    var action = $(this);
                    var options = action.find(".options");
                    action.find(".request_action").click(function() {
                        var params = action_list[action.attr("data-id")],
                            valid = true;

                        current_request_action = $(this);
                        if(params.valid_input_cb) {
                            // Has the user provided valid input for the request?
                            valid = params.valid_input_cb(current_request_action);
                        }
                        if (valid && !action.hasClass("disabled")) {
                            options.slideDown('fast');
                        }
                    });
                    action.find(".cancel").click(function() {
                        if ($(this).hasClass("disabled")){return}
                        options.slideUp('fast');
                    });
                    action.find(".trigger").click(function() {
                        var params = action_list[action.attr("data-id")];
                        var success = function() {
                            action.find("input").removeClass("disabled");
                            if (params.success_cb){
                                params.success_cb(getSelected());
                            }
                        };
                        var error = function() {
                          action.find("input").removeClass("disabled");
                          if(params.error_cb) {
                              params.error_cb(getSelected());
                          }
                        };

                        if ($(this).hasClass("disabled")){return}

                        options.slideUp('fast');

                        if(params.ajax_cb) {
                            params.ajax_cb(getSelected(), current_request_action, options);
                        }
                        else {
                            $.ajax({
                                cache: 'false',
                                type: params.method,
                                url: params.url,
                                data: {ids:getSelected()},
                                success: success,
                                error: error
                            });
                        }
                        if (getSelected() === 0) {
                            action.find("input").addClass("disabled");
                        }
                    });
                });
                updateResult();
            },
            registerAction = function(name, params) {
                /**
                 * params:
                 *    success_cb(data, selected_ids)
                 *    error_cb(data, selected_ids)
                 *    url      //URL for ajax call
                 *    method   //METHOD for ajax call
                 *    unselected_action // true if the action is 'doable' even if
                 *    ajax_cb(id_list, request_action, options, success_cb, error_cb)  //To manually do the ajax call yourself
                 *    valid_input_cb() // to validate the input for the request... return true if valid; otherwise, false
                 *    enable_cb()  // callback to provide custom initialization logic when 1 or more elements are selected
                 *    disable_cb() // callback to provide custom logic when all elements are cleared (i.e. select none)
                 */
              action_list[name] = params;
            },
            resetActions = function(num) {
              $.each(action_list, function(name, params){
                  if(!params.unselected_action) {
                    var div = $("[data-id=" + name + "]");
                    if (num > 0) {
                        div.removeClass("disabled");
                        if (params.enable_cb) {
                            params.enable_cb();
                        }
                    }
                    else {
                        div.addClass("disabled");
                        if (params.disable_cb) {
                            params.disable_cb();
                        }
                    }
                  }
              });
              var actions = $(".panel_action");
              actions.each(function(index){
                var action = $(this);
                action.find('.cancel').click();
              });
            };

            return {
                registerAction: registerAction,
                registerDefaultActions: registerDefaultActions,
                resetActions: resetActions
            };
        })();
    return {
        set_expand_cb: function (callBack) {
            expand_cb.push(callBack);
        },
        get_expand_cb: function () {
            return expand_cb;
        },
        set_contract_cb: function (callBack) {
            contract_cb = callBack;
        },
        set_switch_content_cb: function (callBack) {
            switch_content_cb = callBack;
        },
        select_item: select_item,
        numSelected: numSelected,
        search_started: search_started,
        openSubPanel: openSubPanel,
        updateResult: updateResult,
        closeSubPanel: closeSubPanel,
        closePanel: closePanel,
        panelResize: panelResize,
        panelAjax: panelAjax,
        control_bbq: control_bbq,
        registerPanel: registerPanel,
        registerSubPanelSubmit: registerSubPanelSubmit,
        queryParameters: queryParameters,
        refreshPanel : refreshPanel,
        actions: actions,
        handleScroll : handleScroll,
        extract_panelpage : extract_panelpage
    };
})(jQuery);

KT.panel.copy = (function () {
    var initialize = function() {
        // This function will initialize the support for copy, if copy is defined for the pane.  In katello, one
        // example of this can be found in app/views/system_groups/_tupane_header.html.haml.

        var copy_link = $('.pane_action.copy-tipsy');
        if(copy_link) {
            var cancel_button = $('#cancel_copy_button'), copy_form = $('#copy_form');

            KT.tipsy.custom.copy_tooltip(copy_link);

            copy_link.die();
            copy_link.live('click', show_form);

            cancel_button.die();
            cancel_button.live('click', hide_form);

            copy_form.die();
            copy_form.live('submit', perform_copy);
        }
    },
    show_form = function() {
        $('.pane_action.copy-tipsy').tipsy('show');
        $('#copy_form').find('#name_input').focus();
    },
    hide_form = function() {
        $('.pane_action.copy-tipsy').tipsy('hide');
        $('#copy_form').find('#name_input').blur();
    },
    perform_copy = function(event) {
        event.preventDefault();

        var copy_form = $('#copy_form'), copy_button = $('#copy_button');
        copy_button.attr('disabled', 'disabled');

        $.ajax({
            type: "POST",
            url: copy_form.data("url") + "?&authenticity_token=" + AUTH_TOKEN,
            data: copy_form.serialize(),
            cache: false,
            success: function(data) {
                $('.pane_action.copy-tipsy').tipsy('hide');
                KT.panel.list.createSuccess(data);
            },
            error: function(data) {
                copy_button.removeAttr('disabled');
            }
        });
        return false;
    };
    return {
        initialize: initialize,
        hide_form: hide_form,
        perform_copy: perform_copy
    };

})(jQuery);

KT.panel.list = (function () {
    var total_items_count = 0,
        current_items_count = 0,
        results_items_count = 0,
        search,
        list_section = $('#list section'),

        update_counts = function (current, total, results, clear) {
            if (clear) {
                current_items_count = current;
                total_items_count = total;
                results_items_count = results;
            }
            else {
                current_items_count += current;
                total_items_count += total;
                results_items_count += results;
            }
            $('#total_items_count').html(total_items_count);
            $('#current_items_count').html(current_items_count);
            $('#total_results_count').html(results_items_count);
        },
        last_child = function () {
            return list_section.children().last();
        },
        first_child = function () {
            return list_section.children().first();
        },
        append = function (html) {
            list_section.append($(html).hide().fadeIn(function () {
                list_section.addClass("add", 250, function () {
                    list_section.removeClass("add", 250);
                });
            }));
            return false;
        },
        prepend = function (html) {
            list_section.prepend($(html).hide().fadeIn(function () {
                list_section.addClass("add", 250, function () {
                    list_section.removeClass("add", 250);
                });
            }));
            return false;
        },
        remove = function (id) {
            $('#' + id).fadeOut(function () {
                $(this).empty().remove();
                update_counts(-1, -1, -1);
            });
            return false;
        },
        current_count = function(){
            return current_items_count;
        },
        replace_list = function(html) {
            var list_elem = $("#list");

            list_elem.find('.spinner').hide();
            list_section.html(html).show();
        },
        refresh_list = function() {
            search.refresh_search();
        },
        full_spinner = function() {
            var list_elem = $("#list");
            list_section.empty();
            list_elem.find('.spinner').show();
        },
        refresh = function (id, url, success_cb) {
            var jQid = $('#' + id);

            if ( $('#list').length > 0 ){
                $.ajax({
                    cache: 'false',
                    type: 'GET',
                    url: url,
                    dataType: 'html',
                    success: function (data) {
                        notices.checkNotices();
                        jQid.html(data);
                        // Obtain the value from column_1 and place it in pane_heading. This is
                        // to accommodate changes to an item's name, for example, and have it
                        // dynamically update in both left list and the right title.
                        //
                        // Unless an explicit #heading_title element exists, use the first div
                        // (which was the previous default behavior).
                        var heading_title = jQid.find('#heading_title');
                        if (heading_title.length === 0) {
                            heading_title = jQid.children('div:first');
                        }
                        $('.pane_heading').html(heading_title.html());
                        if (success_cb) {
                            success_cb();
                        }
                    }
                });
            }
            return false;
        },
        registerPage = function (resource_type, options) {
            options = options || {};

            search = KT.search("search_form", "list", this,
                {url: $("#list").attr("data-scroll_url")}, options['extra_params']);


            $(document).bind(search.search_event(), KT.panel.search_started);


            if (KT.panel_search_autocomplete){
              search.enableAutoComplete({data:KT.panel_search_autocomplete});
            }

            $(window).trigger('hashchange');

            KT.panel.control_bbq = false;


            $(document).ready(function () {
                if (options['extra_params']) {
                    for (var i = 0; i < options['extra_params'].length; i += 1) {
                        options['extra_params'][i]['init_func']();
                    }
                }
                $(window).trigger('hashchange', [true]);
            });

            if (options['create']) {
                $('#' + options['create']).live('submit', function (e) {
                    var button = $(this).find('input[type|="submit"]'),
                        data = KT.common.getSearchParams() || {},
                        validation = options['validation'] || function(){ return true; };

                    e.preventDefault();

                    if( options['extra_create_data'] ){
                        $.extend(data, options['extra_create_data']() );
                    }

                    if( validation() ){
                        button.attr("disabled", "disabled");

                        $(this).ajaxSubmit({
                            url: KT.routes[resource_type + '_path'](),
                            data: data,
                            success: createSuccess,
                            error: function (e) {
                                button.removeAttr('disabled');
                                notices.checkNotices();
                            }
                        });
                    }
                });
            }
        },
        createSuccess = function(data){
            var id;

            if (data['no_match']) {
                KT.panel.closePanel($('#panel'));
                notices.checkNotices();
                update_counts(0, 0, 1);
            }
            else {
                prepend(data);
                KT.panel.closePanel($('#panel'));
                id = first_child().attr("id");

                if (last_ajax_panelpage) {
                    $.bbq.pushState({
                        panel: id,
                        panelpage: last_ajax_panelpage
                    });
                } else {
                    $.bbq.pushState({
                        panel: id
                    });
                }
                KT.panel.select_item(id);
                notices.checkNotices();
                update_counts(1, 1, 1);
            }
        };
    return {
        extended_event    : function () { if(search){search.extend_event()} },
        registerPage    : registerPage,
        createSuccess    : createSuccess,
        remove            : remove,
        refresh            : refresh,
        append            : append,
        replace_list    : replace_list,
        update_counts   : update_counts,
        full_spinner    : full_spinner,
        current_count   : current_count,
        refresh_list    : refresh_list
    };
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 */


KT.object = KT.object || {};

// The KT Object label will handle the interaction where a user enters a
// name and upon completion, we want to retrieve a 'default' label from
// the server generated off of that name.
KT.object.label = (function(){
    var initial_name_value,
        retrieve_label = true,

        initialize = function(){
            initial_name_value = "";
            retrieve_label = true;

            if ($(".label_input").length === 0){
                return;
            }

            disable_inputs(undefined);

            $('.create_button').mousedown(function() {retrieve_label = false;});

            $('.name_input').focusout(function(event){
                var name_input = $(this),
                    name = name_input.val(),
                    label = $(this).closest('fieldset').next('fieldset'),
                    label_input = label.find('input.label_input');

                if ((retrieve_label === true) && (name.length > 0) && (name !== initial_name_value)) {
                    // user changed the name so go fetch the label from the server
                    show_label(label, false);

                    $.ajax({
                        type: "GET",
                        url: label_input.data("url"),
                        data: {name:name},
                        cache: false,
                        success: function(data) {
                            if (data.length > 0) {
                                // locate the label input and set it to the value received
                                label_input.val(data);
                            }
                            enable_inputs(name_input);
                            show_label(label, true);
                        },
                        error: function() {
                            enable_inputs(name_input);
                            show_label(label, true);
                        }
                    });
                } else {
                    enable_inputs(name_input);
                }
            });
            $('.name_input').bind('click, focusin', function() {
                disable_inputs($(this));
                initial_name_value = $(this).val();
            });
        },
    show_label = function(label, show) {
        if (show === true) {
            label.find('input.label_input').removeClass('hidden');
            label.find('img.label_spinner').addClass('hidden');
        } else {
            label.find('input.label_input').addClass('hidden');
            label.find('img.label_spinner').removeClass('hidden');
        }
    },
    disable_inputs = function(current_name_input) {
        if (current_name_input === undefined) {
            // disable all label inputs
            $(".label_input").attr("disabled", "disabled");
        } else {
            // disable the label input associated with the current name
            current_name_input.closest('fieldset').next('fieldset').find('input.label_input').attr("disabled", "disabled");
        }
    },
    enable_inputs = function(current_name_input) {
        if (current_name_input === undefined) {
            // enable all label inputs
            $(".label_input").removeAttr("disabled");
        } else {
            // enable the label input associated with the current name
            current_name_input.closest('fieldset').next('fieldset').find('input.label_input').removeAttr("disabled");
        }
    };

    return {
        initialize: initialize
    };
})();
KT.tipsy = KT.tipsy || {};


KT.tipsy.custom = (function(){
    var tooltip = function(element) {
        $(element).tipsy({
           gravity: 'e', live : true, html : true, title : KT.tipsy.templates.dynamic,
           hoverable : true, closeOnScroll:true, delayOut : 250, opacity : 1, delayIn : 300,
           className : 'tooltip',
           stickyClick : function(element, state){
               if (state === 'on'){
                   $(element).addClass('tipsy-hover').removeClass('tipsy-nohover');
               } else {
                   $(element).addClass('tipsy-nohover').removeClass('tipsy-hover');
               }
           },
           afterShow : function(){
               $('.details_container').addClass('scroll-pane');
               KT.common.jscroll_init($('.scroll-pane'));
           }});
    },
    url_tooltip = function(element, direction){
        var get_url = function(elem){return elem.data().url};
        element.tipsy({
           gravity: direction, live : true, html : true, title : KT.tipsy.templates.spinner,
           url: get_url,
           hoverable : true, delayOut : 250, opacity : 1, delayIn : 300, className : 'tooltip',
           stickyClick : function(element, state){
               if (state === 'on'){
                   $(element).addClass('tipsy-hover').removeClass('tipsy-nohover');
               } else {
                   $(element).addClass('tipsy-nohover').removeClass('tipsy-hover');
               }
           },
           afterShow : function(obj, tipsy_div){
                $('.details_container').addClass('scroll-pane');
                KT.common.jscroll_init($('.scroll-pane'));
           }});
    },
    copy_tooltip = function(element) {
        $(element).tipsy({
            gravity: 'n', trigger : 'manual', html : true, title : KT.tipsy.templates.copy_form,
            delayOut : 250, opacity : 1, delayIn : 300, className : 'copy-tipsy'
        });
    },
    disable_details_tooltip = function(element) {
        element.replaceWith('<span class="details-icon-nohover"></span>');
    },
    system_packages_tooltip = function(){
        var element = $(this);

        $('.system_content_action').tipsy({
           gravity: 'e', live : true, html : true, title : KT.tipsy.templates.table_template,
           hoverable : true, delayOut : 250, opacity : 1, delayIn : 300, className : 'tooltip',
           afterShow : function(){
               $('.details_container').addClass('scroll-pane');
               KT.common.jscroll_init($('.scroll-pane'));
           }});

        $('.system_packages_action').tipsy({
           gravity: 'e', live : true, html : true, title : KT.tipsy.templates.table_template,
           hoverable : true, delayOut : 250, opacity : 1, delayIn : 300, className : 'table_tooltip',
           afterShow : function(){
               $('.details_container').addClass('scroll-pane');
               KT.common.jscroll_init($('.scroll-pane'));
           }});
    },
    enable_forms_tooltips = function(){
      // Form inputs help tips
      $('form i.details-icon[data-help]').tipsy({
        gravity: 'e', live : true, html : true, title : KT.tipsy.templates.table_template,
        hoverable : true, delayOut : 250, opacity : 1, delayIn : 300, className : 'tooltip',
        afterShow : function(){
          $('.details_container').addClass('scroll-pane');
          KT.common.jscroll_init($('.scroll-pane'));
      }});
    };
    return {
        tooltip                  : tooltip,
        copy_tooltip             : copy_tooltip,
        disable_details_tooltip  : disable_details_tooltip,
        system_packages_tooltips : system_packages_tooltip,
        url_tooltip              : url_tooltip,
        enable_forms_tooltips    : enable_forms_tooltips
    };
})();

KT.tipsy.templates = (function(){
    var dynamic = function() {
      // The dynamic function will determine the template to use for rendering the tipsy based on the type of element.
      // Currently, this is assumed to be either errata-info or a simple list.
      var html, element = $(this);
      if (element.hasClass('errata-info')) {
          html = errata(element);
      }
      else if (element.attr('data-url')){
        html = url(element);
      } else if (element.hasClass('task-info')) {
          html = task(element);
      } else {
          html = list(element);
      }
      return html;
    },
    list = function(element) {
        // The list template assumes that the data shown in the template is essentially a simple list of items.
        // The items are pulled from the data-list attribute associated with the tipsy element.
        var html = '<div class="details_container">',
            items_list = [],
            generate_list = function(){
                var items_list = element.data('list'),
                    i = 0,
                    length = items_list.length,
                    html = "";

                for(i; i < length; i += 1){
                    html += "<li>" + items_list[i] + '</li>';
                }
                return html;
            };

        items_list = generate_list();

        html += '<div class="item-container"><ul style="margin:0 0 0 4px;" class="la">' + items_list + '</ul></div>';
        html += '</div>';

        return html;
    },
    errata = function(element){
        var html = '<div class="details_container">',
            packages_list = [],
            generate_packages = function(){
                var packages = element.data('packages')[0]["packages"],
                    i = 0,
                    length = packages.length,
                    html = "";

                for(i; i < length; i += 1){
                    html += "<li>" + packages[i]["filename"] + '</li>';
                }

                return html;
            };

        packages_list = generate_packages();

        html += '<div class="item-container"><label class="fl ra">ID:</label>' + '<p>' + element.data('id') + '</p></div>';
        html += '<div class="item-container"><label class="fl ra">Title:</label>' + '<p>' + element.data('title') + '</p></div>';
        html += '<div class="item-container"><label class="fl ra">Issued:</label>' + '<p>' + element.data('issued') + '</p></div>';
        html += '<div class="item-container"><label class="fl ra">Reference:</label>' + '<p><a target="new" href="' +  element.data('reference_url') + '">' + element.data('reference_url') + '</a></p></div>';
        html += '<div class="item-container"><label class="fl ra">Description:</label>' + '<p><br/><pre>' + element.data('description') + '</pre></p></div>';
        html += '<div class="item-container"><label class="fl" style="text-align:left;">Packages:</label>' + '<ul style="margin:0 0 0 4px;" class="la"><br/>' + packages_list + '</ul></div>';
        html += '</div>';

        return html;
    },
    spinner = function(){
        return '<span class="spinner" ></span>';
    },
    task = function(element) {
        var html = '<div class="details_container">';

        html += '<div class="item-container"><label class="fl ra">Result:</label>' + '<p><br/>' + element.data('result') + '</p></div>';

        html += '</div>';
        return html;
    },
    copy_form = function() {
        var element = $(this),
            html = '<div>';

        html += '<form id="copy_form" data-url="' + element.data('url') + '">';
        html += '<fieldset><div><label>' + i18n.name + '</label></div><div><input id="name_input" type="text" size="25" name="name"></div></fieldset>';
        html += '<fieldset><div><label>' + i18n.description + '</label></div><div><textarea id="description_input" rows="1" cols="31" name="description"></textarea></div></fieldset>';
        html += '<input id="copy_button" type="submit" class="fr button" value="' + i18n.copy + '">';
        html += '<input id="cancel_copy_button" type="button" class="fr button" value="' + i18n.cancel + '">';
        html += '<form>';
        html += '</div>';
        return html;
    },
    table_template = function(){
        var html = '<div class="details_container">',
            element = $(this);

        html += '<div class="item-container">' + '<p>' + element.data('help') + '</p></div>';
        html += '</div>';
        return html;
    };

    return {
        dynamic           : dynamic,
        copy_form         : copy_form,
        table_template    : table_template,
        spinner          : spinner
    };

})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


/**
 * @param div_id   -  id of div to house env_selector
 * @param name    -  the name of this env selector ( in case there is more than one)
 * @param environments - array of environment paths
 *                         each path being an array of hashes
 *                          containing  'id', 'name', 'selectable' (optional)
 * @param options_in
 *      inline (false)     -    Add all the paths inline, instead of a hidable panel
 *      select_mode (none)     -  selection mode ('single', 'none', 'multi')
 *      link_first (true)      -  if select_mode is not none, all the first nodes
 *                                in the path are 'linked' so checking one checks all
 *      submit_button_text (none)     - if set, a button is rendered with the specified text,
 *                                 clicking the button generates a message
 *      submit_event (path_#{name})    - if button exists, clicking it will trigger this event
 *                                       if no button and not inline, hovering out will trigger this event
 *      expand (true)           -  enable/disable the expanding of path nodes (disable for debugging)
 *
 */

KT.path_select = function(div_id, name, environments, options_in){

    var div,
        scroll_obj,
        paths_id,
        path_selector,
        options = {},
        utils = KT.utils,
        init = function(){
            div = $('#' + KT.common.escapeId(div_id));
            paths_id = "path_select_" + name;
            options.inline = default_opt(options_in.inline, false);
            options.activate_on_click = default_opt(options_in.activate_on_click, false);
            options.select_mode = default_opt(options_in.select_mode, 'none');

            options.submit_button_text = default_opt(options_in.submit_button_text, undefined);
            options.cancel_button_text = default_opt(options_in.cancel_button_text, undefined);
            options.submit_event = default_opt(options_in.submit_event, ('submit_paths_' + name));
            options.cancel_event = default_opt(options_in.cancel_event, ('cancel_paths_' + name));
            options.select_event = default_opt(options_in.select_event, ('selected_cell_' + name));
            options.link_first = default_opt(options_in.link_first, true);
            options.expand = default_opt(options_in.expand, true);
            options.footer = default_opt(options_in.footer, false);

            $(div).append(KT.path_select_template.selector(environments, paths_id, options.submit_button_text, options.cancel_button_text, options.footer));
            path_selector = $("#" + paths_id);
            path_selector.find('.node_select').not(':checked').hide();

            if(options.select_mode !== 'none'){
                setup_input_actions();
            }

            if(options.submit_button_text){
                path_selector.find('.KT_path_select_submit_button').click(function(e){
                    if(!options.inline) {
                        path_selector.hide();
                    }
                    $(document).trigger(options.submit_event, [get_selected()]);
                });
            }

            if(options.cancel_button_text){
                path_selector.find('.KT_path_select_cancel_button').click(function(e){
                    clear_selected();
                    if(!options.inline) {
                        path_selector.hide();
                    }
                    $(document).trigger(options.cancel_event);
                    return false;
                });
            }

            if(!options.inline){
                path_selector.addClass("hover_selector");

                path_selector.hide();

                if(options.activate_on_click)  {
                    div.click(function(e) { path_selector.show(); });
                } else {
                    div.hoverIntent({
                        over:function(){ path_selector.show(); },
                        timeout:500,
                        interval: 200,
                        out:hover_out
                    });
                }
            }

            scroll_obj = KT.env_select_scroll({});
            recalc_scroll();
        },
        reposition_left = function(){
            var selector_width, pos;

            if(options.inline){
                return false;
            }
            //This is a hack for IE, sadly

            path_selector.css('visibility', 'hidden');
            path_selector.show();

            selector_width = path_selector.outerWidth();
            pos = div.outerWidth()  - selector_width - 1;

            path_selector.css('left', pos + 'px');

            path_selector.hide();
            path_selector.css('visibility', 'visible');
        },
        hover_out = function(){
            path_selector.hide();
            $(document).trigger(options.submit_event, [get_selected()]);
        };
        reposition_right = function(){
            var margin = 10,
                window_width, selector_width, button_start, pos, top;

            if(options.inline){
                return false;
            }
            window_width = $(window).width();
            selector_width = path_selector.outerWidth()  ;
            button_start = div.offset().left;



            if(button_start + selector_width + margin > window_width){
                pos = window_width - (selector_width + margin);
                pos = pos - button_start;
            }
            else{
                pos = 0;
            }
            path_selector.css('left', pos + 'px');
        },
        default_opt = function(attribute, default_value){
            return attribute === undefined ? default_value : attribute;
        },
        setup_input_actions = function(){
            var anchors = path_selector.find('li'),
                nodes = path_selector.find('.node_select'),
                first_nodes = path_selector.find('ul').find('li:first'),
                on_select, on_deselect;

            anchors.hover(
                function(){
                    var input = $(this).find('.node_select');
                    if (!input.is(':visible')){
                        input.show();
                    }
                },
                function(){
                    var input = $(this).find('.node_select');
                        if(!input.is(":checked")){
                           input.hide();
                        }
                }
            );


            on_select = function(select_elem){
                select_nodes(select_elem);
                if(options.select_mode === 'single'){
                    unselect_nodes(nodes);
                    select_nodes(select_elem);
                }
                if(options.link_first && select_elem.parents('li').is(':first-child')){
                    select_nodes(first_nodes.find('input:checkbox').not(':checked'));
                }
                $(document).trigger(options.select_event, [true, $(this).data('node_id'), $(this).data('next_node_id')]);
            };
            on_deselect = function(select_elem){
                unselect_nodes(select_elem);
                if(options.select_mode === 'single'){
                    nodes.removeAttr('disabled');
                }
                if(options.link_first && select_elem.parents('li').is(':first-child')){
                    unselect_nodes(first_nodes.find('input:checkbox:checked').hide());
                }
                $(document).trigger(options.select_event, [false, $(this).data('node_id'), $(this).data('next_node_id')]);
            };
            nodes.change(function(){
                    if ($(this).is(':checked')){
                        on_select($(this));
                    }
                    else {
                        on_deselect($(this));
                    }
            });

        },
        select_nodes = function(checkbox_list){
            checkbox_list.attr('checked', 'checked').show();
            checkbox_list.parents('label').addClass('active');
        },
        unselect_nodes = function(checkbox_list){
            checkbox_list.removeAttr('checked');
            checkbox_list.parents('label').removeClass('active');
        },
        get_selected = function(){
            var selected = path_selector.find('input:checked'),
                to_ret = {};
            KT.utils.each(selected, function(item){
                var data = $(item).data();
                if(!options.link_first || to_ret[data.node_id] === undefined){
                    to_ret[data.node_id] = { 'id' : data.node_id, name:data.node_name,
                            next_id:data.next_node_id};
                }
            });
            return to_ret;
        },
        get_paths = function(){
            var flattened = utils.flatten(environments),
                tmp_hash = {};
            if (options.link_first){
                KT.utils.each(flattened, function(item){
                    tmp_hash[item.id] = item;
                });
                return KT.utils.values(tmp_hash);
            }
            else{
                return flattened;
            }
        },
        recalc_scroll = function(){
           if(!options.expand){
               return false;
           }
           if(!options.inline){
               path_selector.show();
               scroll_obj.bind('#' + KT.common.escapeId(paths_id));
               path_selector.hide();
           }
           else {
               scroll_obj.bind('#' + KT.common.escapeId(paths_id));
           }
        },
        get_submit_event = function(){
            return options.submit_event;
        },
        get_cancel_event = function(){
            return options.cancel_event;
        },
        get_select_event = function(){
            return options.select_event;
        },
        clear_selected = function(){
            path_selector.find('input:disabled').removeAttr('disabled');
            unselect_nodes(path_selector.find('input:checked').hide());
        },
        select = function(id, next_id){
           var nodes = path_selector.find('input:checkbox[data-node_id=' + id + ']');
           if(nodes.length > 1 && !options.link_first){
               nodes.and('[data-next_node_id=' + next_id + ']').not(':checked').click();
           }
           else{
               nodes.first().not(':checked').click();
           }
        };

    init();

    return {
        get_paths   : get_paths,
        get_selected: get_selected,
        get_submit_event : get_submit_event,
        get_select_event : get_select_event,
        clear_selected: clear_selected,
        select:select,
        reposition_left: reposition_left,
        paths_id: paths_id,
        hide: function() {path_selector.hide();}
    };
};


KT.path_select_template = {
    selector : function(paths, div_id, submit_button_text, cancel_button_text, footer){
        var html = '<div id="' + div_id + '" class="path_selector"><form>';
        if(cancel_button_text){
            html += '<div class="action_buttons">';
            html += KT.path_select_template.button("KT_path_select_cancel_button", cancel_button_text);
        }
        if(submit_button_text){
            html += KT.path_select_template.button("KT_path_select_submit_button", submit_button_text);
            html += '</div>';
        }

        html += KT.path_select_template.paths(paths);

        html += '</form>';

        if( footer ){
            html += KT.path_select_template.footer(footer);
        }

        html += '</div>';
        return html;
    },
    button : function(clazz, text){
        return '<input type="button" class="button ' + clazz +'" value="' + text + '">';
    },
    paths : function(paths){
        var html ='';

        KT.utils.each(paths, function(item){
            html += KT.path_select_template.path(item);
        });
        return html ;
    },
    path : function(path){
        var html = '';
        html += '<div><ul>';
        for(var i = 0; i < path.length; i += 1){
            html += KT.path_select_template.path_node(path[i], path[i+1]);
        }
        html += '</ul></div>';
        return html;
    },
    path_node: function(node, next){
        var html = '',
            next_node =  next  ? ('data-next_node_id="' + next.id + '"') : '',
            input = node.select ? '<span class="checkbox_holder"><input class="node_select" type="checkbox" ' +
                next_node +' data-node_id="' + node.id + '" data-node_name="' + node.name + '"></span>' : '';

        html += '<li data-node_id="' + node.id + '">'+ '<label><div>' + input +  node.name +  '</div></label></li>';
        return html;
    },
    footer : function(content){
        var html = '<footer>' + content + '</footer>';

        return html;
    }
};

/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/


var notices = (function() {
    return {
        setup_notices: function(pollingTimeOut) {
          if (pollingTimeOut === undefined) {
            pollingTimeOut = 45000;
          }

          notices.checkTimeout = pollingTimeOut;
          //start continual checking for new notifications
          notices.start();
        },
        clearPreviousFailures: function(requestType) {
            $('.' + requestType).closest('.jnotify-notification').remove();
        },
        displayNotice: function(level, notice, requestType) {
            var noticesParsed = $.parseJSON(notice),
                options = {
                    type: level,
                    slideSpeed: 200,
                    alwaysClosable: true
                },
                generate_list = function(notices){
                    var notices_list = '<ul class='+requestType+'>',
                        i, length = notices.length;

                    for( i=0; i < length; i += 1) {
                        notices_list += '<li>' + notices[i] + '</li>';
                    }
                    notices_list += '</ul>';

                    return notices_list;
                };

            var noticeObj = {};
            noticeObj.time = (new Date()).valueOf();
            noticeObj.level = level;
            noticeObj.notices = noticesParsed['notices'];
            noticeObj.validationErrors = noticesParsed['validation_errors'];
            noticeObj.requestType = requestType;
            $(document).trigger("notice", noticeObj);

            if (level === 'success') {
                notices.clearPreviousFailures(requestType);
            }

            if ((level === "error") || (level === "warning")) {
                options["sticky"] = true;
                options["fadeSpeed"] = 600;
            } else if( level === "message" ) {
                options["sticky"] = true;
                options["fadeSpeed"] = 600;
            } else {
                options["sticky"] = false;
                options["fadeSpeed"] = 600;
            }

            if( noticesParsed['validation_errors'] !== undefined ){
                var validation_html = generate_list(noticesParsed['validation_errors']);
                validation_html = '<span>' + i18n.validation_errors + '</span>' + validation_html;
                // set the options as this is an error
                options["type"] = "error";
                options["sticky"] = true;
                options["fadeSpeed"] = 600;
                $.jnotify(validation_html, options);
                $('.jnotify-message ul').css({'list-style': 'disc',
                              'margin-left': '30px'});
            }
            if( noticesParsed['notices'] && noticesParsed['notices'].length !== 0 ){
                $.jnotify(generate_list(noticesParsed['notices']), options);
            }
        },
        storeNotice: function(event, noticeObj) {
            var maxAge = 600000,
                curTime = (new Date()).valueOf(),
                idx = notices.noticeArray.length - 1;
            // Discard notices older than maxAge
            for (; idx>=0; idx-=1) {
                if (curTime - notices.noticeArray[idx].time > maxAge) {
                    notices.noticeArray.splice(idx, 1);
                }
            }
            notices.noticeArray.push(noticeObj);
        },
        addNotices: function(data) {
            var unread_notices = $("#unread_notices");
            var unread_notices_count = $("#unread_notices_count");
            if (!data || data.unread_count.length === 0) {
                return true;
            }
            unread_notices.data('last', parseInt(unread_notices.text(), 10));
            //if coming from the server may have new count
            if (data.unread_count > 0 && data.unread_count > unread_notices.data('last')) {
                unread_notices_count.text(data.unread_count);
                unread_notices.effect("bounce", "fast");
                unread_notices.data('last', data.unread_count);
            }

            $.each(data.new_notices, function(index, notice) {
                notices.displayNotice(notice.level, window.JSON.stringify({ "notices": [notice.text] }), notice.request_type);
            });

            return true;
        },
        checkNotices : function() {
            var url = KT.routes.notices_get_new_path();

            //Make sure when we load the page we get notifs
            $.ajax({
              type: 'GET',
              url: url,
              dataType: 'json',
              global: false,
              success: notices.addNotices
            });
        },
        checkNoticesInResponse : function(xhr) {
            var message, messageType;

            if (xhr !== undefined) {
                message = xhr.getResponseHeader('X-Message');
                if (message) {
                    messageType = xhr.getResponseHeader('X-Message-Type');
                    messageRequestType = xhr.getResponseHeader('X-Message-Request-Type');
                    notices.displayNotice(messageType, KT.common.decode(message), messageRequestType);
                }
            }
        },
        start: function () {
            var url = KT.routes.notices_get_new_path();

            // do not wait for PeriodUpdater, check new notices immediately
            $.ajax({
                type:"GET",
                url:url,
                cache:false,
                success:notices.addNotices
            });

            var pu = $.PeriodicalUpdater(url, {
                method: 'get',
                type: 'json',
                global: false,
                minTimeout: notices.checkTimeout,
                maxTimeout: notices.checkTimeout
            }, notices.addNotices);

        }
    };
})();

$(document).ready(function() {

    // perform periodic polling of notices (e.g. async scenarios)
    //notices.checkNotices();

    notices.noticeArray = [];
    $(document).bind("notice", notices.storeNotice);

    $(document).ajaxComplete(function(event, xhr, options){
        // look for notices in the response (e.g. sync scenarios)
        notices.checkNoticesInResponse(xhr);
    });
});


KT.env_select_scroll = function(options) {


    var anchor_padding = 20, //amount of padding each anchor has
        min_size = 40,
        min_size_selected = 75,
        px_per_sec = 400,
        freq = 20;

    var bind = function(element) {
        if (!element) {
            element = ".jbreadcrumb";
        }

        $(element).find('ul').each(function() {
            var trail = $(this),
                cont_width = $(this).width(),
                combined_width = 0,
                anchors = trail.find("a, label"),
                my_min_size_selected = min_size_selected;
            anchors.unbind("mouseout").unbind("mouseover").width('auto');

            anchors.each(function() {
                combined_width += $(this).width() + anchor_padding;
            });


            //if nothing is selected in this path, we won't have a special node that is bigger
            //   so reset min-size_selected to normal min_size for calculations
            if (trail.find(".active").length === 0) {
                my_min_size_selected = min_size;
            }


            //if we don't actually need more room, then don't add the scrolling
            if (combined_width < cont_width) {
                return true;
            }


            anchors.each(function(index) {
                var anchor = $(this),
                    out_interval,
                    over_interval,
                    total_width = anchor.width(),
                    my_min_size = min_size;


                //taken width is the width of all of the nodes contracted minus this one (includes selected ndoe)
                var taken_width = ((anchor_padding + min_size) * (anchors.length-2)) + anchor_padding + my_min_size_selected;

                //if the container is smaller than all of the other nodes + this one expanded, we need to shorten it
                if (cont_width <  (taken_width + total_width + anchor_padding) ) {
                    total_width = cont_width - taken_width - anchor_padding - 5 ;
                }

                //if its active and the min size of a selected node is smaller than what we had calculated, just use that
                if (anchor.hasClass("active")) {
                    my_min_size = my_min_size_selected;
                    if (my_min_size > total_width) {
                        return;
                    }
                }

                //reset the width to contracted state
                $(this).width(my_min_size);

                var total_time = (total_width - min_size)/px_per_sec,  //total time of animation
                    num_iterations = total_time*1000/freq,  //number of 'frames'
                    chunk_size = (total_width - min_size)/num_iterations; //how many pixels to move each frame


                var clear_out = function() {
                    if (out_interval) {
                        clearInterval(out_interval);
                        out_interval = undefined;
                    }
                },
                clear_over = function() {
                    if (over_interval) {
                        clearInterval(over_interval);
                        over_interval = undefined;
                    }
                };


                anchor.mouseover(function() {
                    if (over_interval) {
                        return false;
                    }

                    clear_out();

                    over_interval = setInterval(function() {
                        var width = anchor.width();
                        if (width < total_width) {
                            clear_out();
                            anchor.width((width+chunk_size));
                        }
                        else {
                            clear_over();
                        }
                    }, freq);
                });

                anchor.mouseout(function() {
                    if (out_interval) {
                        return false;
                    }

                    clear_over();


                    out_interval = setInterval(function() {
                        var width = anchor.width();
                        if (width >= my_min_size) {
                            clear_over();
                            anchor.width(width-chunk_size);
                        }
                        else {
                            clear_out();
                        }
                    }, freq);
                });

            });
        });
    };

    return {bind:bind};
};
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/

var env_select =   {
    /* Click callback should be a function:
     *
     * function(env_id, env_next_id, is_library)
     * env_next_id may be undefined
     *
     */
    scroll_obj: undefined,
    click_callback: undefined,

    // To facilitate triggering UI elements to change state based upon the currently selected environment,
    // this callback is called whenever the env is changed. Note that this does not guarantee that the env id
    // is actually different, just that the user interacted w/ the UI in a way that it updated.
    //
    // eg. Consider user.js, which saves off the original env id and then enables/disables the save button.
    // 1. The original id is set when the page is loaded (see _edit_environment.html.haml
    // 2. Callback is defined in user.js to enable/disable save button
    // 3. *IMPORTANT* Note that the callback is manually triggered in user.js:set_expand_cb(). This is where
    //    the widget is populated with a new org.
    // 4. Finally in this example case, once the new env is successfully saved the button should be disabled
    //    and the original env updated. (see user_methods.js)
    env_changed_callback: function(env_id) {},

    active_div:  undefined,
    recalc_scroll: function() {
        $(".path_entries").show();
        env_select.scroll_obj.bind();
        $(".path_entries").hide();
    },
    expand: function() {
        $('#path-collapsed').hide();
        $('#path-expanded').show();

        $('#path-entries').show();
    },
    close: function() {
        $('#path-collapsed').show();
        $('#path-expanded').hide();

        $('#path-entries').hide();
    },
    path_selected: function() {
        env_select.close();
        var content = $(this).html();
        $('#path-selected').find('ul').html(content);
        env_select.active_div = $(this);
        env_select.highlight_selected();

        if(env_select.env_changed_callback){
            env_select.env_changed_callback(env_select.get_selected_env());
        }

        return false;
    },
    get_selected_env: function() {
        return $(".path_link.active").attr("data-env_id");
    },
    set_selected: function(env_id) {
        $('[data-env_id=' + env_id + '].path_link').click();
    },
    env_selected: function() {
        env_select.close();
        var id = $(this).attr('data-env_id') ;

        $('a[data-env_id]').removeClass('active');
        $('a[data-env_id="'+ id + '"]').addClass('active');

        $(this).parentsUntil(".path_entry").trigger("click");

        if (env_select.click_callback) {
          env_select.click_callback(id, $(this));
        }
        if(env_select.env_changed_callback){
            env_select.env_changed_callback(id);
        }

        env_select.recalc_scroll();

        return false;
    },
    disable_active: function() {
        //This is used when the user is highlighting entries, to clear
        //  what we manually highlighted.  CSS Hover will take care of it from here.
        $(".path_entry").removeClass("path_entry_selected");
    },
    is_open: function() {
        return $('#path-entries').is(":visible");
    },
    highlight_selected: function () {
        $(".path_entry").removeClass("path_entry_selected");
        env_select.active_div.addClass("path_entry_selected");
        return false;
    },
    init : function(){
        $('#path-expanded').hide();
        $('#path-collapsed').live('click', env_select.expand);
        $('#path-expanded').live('click', env_select.close);
        $('.path_link').live('click', env_select.env_selected);
        $('.path_entry').live('click', env_select.path_selected);

        //If we mouse over the entries box, deselect what is already selected
        $('#path-entries').mouseover(env_select.disable_active);
        $('#path-entries').mouseout(env_select.highlight_selected);

        env_select.active_div = $(".path_link.active").parents(".path_entry");
        env_select.highlight_selected();

        //Close the drop down if the user clicks somewhere else
        $('body').click(function(event){
            if (($(event.target).parents("#path-entries").size() <= 0) && env_select.is_open()) {
                env_select.close();
              }
        });

        env_select.reset_hover();
        env_select.scroll_obj = KT.env_select_scroll({});
        env_select.recalc_scroll();
    },
    reset_hover: function() {
        $('#path-container').hoverIntent({
            over:env_select.expand,
            timeout:500,
            interval: 200,
            out:env_select.close
        });
    }
};

$(document).ready(function() {
    env_select.init();
});

/**
 * Copyright 2013 Red Hat, Inc.
 *
 * This software is licensed to you under the GNU General Public
 * License as published by the Free Software Foundation; either version
 * 2 of the License (GPLv2) or (at your option) any later version.
 * There is NO WARRANTY for this software, express or implied,
 * including the implied warranties of MERCHANTABILITY,
 * NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 * have received a copy of GPLv2 along with this software; if not, see
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 */

/**
 * @ngdoc module
 * @name  Katello
 *
 * @description
 *   Base module that defines the Katello module namespace and includes any thirdparty
 *   modules used by the application.
 */

var Katello = angular.module('Katello', [
    'alchemy',
    'alch-templates',
    'ngSanitize',
    'Katello.i18n',
    'Katello.menu',
    'Katello.systems',
    'Katello.system-groups'
]);

/**
 * @ngdoc config
 * @name  Katello.config
 *
 * @requires $httpProvider
 *
 * @description
 *   Used for establishing application wide configuration such as adding the Rails CSRF token
 *   to every request.
 */
Katello.config(['$httpProvider', function($httpProvider){
    $httpProvider.defaults.headers.common['X-CSRF-TOKEN'] = $('meta[name=csrf-token]').attr('content');
}]);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.menu
 *
 * @description
 *   Module for menu related functionality.
 */

angular.module('Katello.menu', ['alchemy', 'alch-templates', 'Katello.widgets']);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/


(function() {
    'use strict';

    angular.module('Katello.menu').controller('MenuController', ['$scope', '$location', '$document', function($scope, $location, $document) {

        $scope.menu       = KT.main_menu;
        $scope.user_menu  = KT.user_menu;
        $scope.admin_menu = KT.admin_menu;
        $scope.notices    = KT.notices;

        /**
         * Set the active menu item.
         * @param menuItem the menuItem to make active.
         */
        function setActiveMenuItem (menuItem) {
            if (menuItem) {
                $scope.menu.active_item = menuItem;
                $scope.admin_menu.active_item = menuItem;
                $scope.menu.active_item.active = true;
                $scope.admin_menu.active_item.active = true;
            }
        }

        /**
         * Get the active menu item based on the $location service.
         * @param menuItems
         * @returns the active menu item.
         */
        function getActiveMenuItem (menuItems) {
            var activeMenuItem;
            for (var i = 0; i < menuItems.length; i += 1) {
                var menuItem = menuItems[i];
                if ($location.absUrl().indexOf(menuItem.url) >= 0) {
                    activeMenuItem = menuItem;
                } else if (menuItem.hasOwnProperty('items')) {
                    // If the active page is a child of a top level menu item
                    // then set the top level menu item as active.
                    if (getActiveMenuItem(menuItem.items)) {
                        activeMenuItem = menuItem;
                    }
                }
            }
            return activeMenuItem;
        }

        // Hide the org switcher menu if the user clicks outside of it
        var orgSwitcherMenu = angular.element('#organizationSwitcher');
        $document.bind('click', function (event) {
            var target = angular.element(event.target);
            if (!orgSwitcherMenu.find(target).length) {
                $scope.orgSwitcher.visible = false;
                $scope.$apply();
            }
        });

        // Combine all menu items and figure out which one ought to be active.
        var allMenus = $scope.menu.items.concat($scope.user_menu.items).
            concat($scope.admin_menu.items).concat($scope.notices);
        setActiveMenuItem(getActiveMenuItem(allMenus));
    }]);
})();
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.widgets
 *
 * @description
 *   Module for common Katello widgets.
 */

angular.module('Katello.widgets', ['alchemy', 'alch-templates', 'infinite-scroll']);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc directive
 * @name Katello.directive:orgSwitcher
 *
 * @requires $http
 *
 * @description
 *  Used to provide an organization switcher for the logged in user.  Currently simply stuffs
 *  _allowed_orgs.html.haml into the #allowed-orgs ul element.
 *
 *  TODO: angularize this directive.
 *
 * @example
 *  <ul org-switcher></ul>
 */

angular.module('Katello.widgets').directive('orgSwitcher', ['$http', function($http) {
    return {
        restrict: 'A',
        transclude: true,

        controller: ['$scope', function($scope) {
            var jScrollApi;
            var $allowedOrgList = $('#allowed-orgs');
            var $spinner = $('#organizationSwitcher .spinner');

            $allowedOrgList.jScrollPane();
            jScrollApi = $allowedOrgList.data('jsp');

            $scope.orgSwitcher = {
                visible: false
            };

            $scope.orgSwitcher.toggleVisibility = function() {
                $scope.orgSwitcher.visible = !$scope.orgSwitcher.visible;
            };

            $spinner.fadeIn();
            $scope.orgSwitcher.refresh = function() {
                $http.get(KT.routes.allowed_orgs_user_session_path()).then(function(response) {
                    $spinner.fadeOut();
                    jScrollApi.getContentPane().html(response.data);
                    jScrollApi.reinitialise();

                    // Shrink the menu if there aren't enough organizations to fill it up.
                    var shouldResize = false;
                    var $listItems = $allowedOrgList.find('li');
                    setTimeout(function() {
                        var listHeight = $listItems.length * $listItems.height();
                        shouldResize = listHeight < parseInt($allowedOrgList.css("height"), 10);
                        if (shouldResize) {
                            jScrollApi.destroy();
                            $('#allowed-orgs').css("height", "auto");
                        }
                    }, 0);
                });
            };

            $scope.$watch('orgSwitcher.visible', function(newValue, oldValue) {
                if (newValue && (newValue !== oldValue)) {
                    // Refresh the list of organizations
                    $scope.orgSwitcher.refresh();
                }
            });

        }]
    };
}]);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.i18n
 *
 * @description
 *   Module for internationalization.
 */

angular.module('Katello.i18n', []);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/*global i18n*/

/**
 * @ngdoc filter
 * @name  Katello.i18n:i18n
 *
 * @description
 *   Looks up a key in the i18n dictionary and returns the translated string.
 *
 * @example
 *   {{ "some_i18n_code" | i18n }}
 *   {{ "some_i18n_code_with_replacements" | i18n: ["why", "hello"] }}
 */

angular.module('Katello.i18n').filter('i18n', function() {
    return function(i18nKey, replacements) {
        var translation;
        var translated = i18nKey;

        if (i18n.hasOwnProperty(i18nKey)) {
            translation = i18n[i18nKey];
            translated = translation;
            if (typeof translation === 'function') {
                translated = translation.apply(this, replacements);
            }
        }
        return translated;
    };
});
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.systems
 *
 * @description
 *   Module for systems related functionality.
 */

angular.module('Katello.systems', ['alchemy', 'alch-templates', 'ui.compat', 'Katello.widgets']);

/**
 * @ngdoc config
 * @name  Katello.config
 *
 * @requires $stateProvider
 *
 * @description
 *   Used for systems level configuration such as setting up the ui state machine.
 */
angular.module('Katello.systems').config(['$stateProvider', function($stateProvider){
    $stateProvider.state('systems', {
        views: {
            '@': {
                controller: 'SystemsController'
            }
        }
    });

    $stateProvider.state('systems.alter-content', {
        views: {
            '@': {
                controller: 'SystemsBulkActionController',
                templateUrl: 'views/systems/alter-content-bulk.html'
            }
        }
    });

    $stateProvider.state('systems.alter-system-groups', {
        views: {
            '@': {
                controller: 'SystemsBulkActionController',
                templateUrl: 'assets/views/systems/alter-systems-group-bulk.html'
            }
        }
    });

    $stateProvider.state('systems.bulk-delete', {
        views: {
            '@': {
                controller: 'SystemsBulkActionController',
                templateUrl: 'views/systems/systems-delete-bulk.html'
            }
        }
    });
}]);
/**
 Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc module
 * @name  Katello.system-groups
 *
 * @description
 *   Module for system groups related functionality.
 */

angular.module('Katello.system-groups', ['ngResource']);
/**
 * Copyright 2013 Red Hat, Inc.

 * This software is licensed to you under the GNU General Public
 * License as published by the Free Software Foundation; either version
 * 2 of the License (GPLv2) or (at your option) any later version.
 * There is NO WARRANTY for this software, express or implied,
 * including the implied warranties of MERCHANTABILITY,
 * NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 * have received a copy of GPLv2 along with this software; if not, see
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 **/

/**
 * @ngdoc factory
 * @name  Katello.system-groups.factory:SystemGroups
 *
 * @requires $resource
 * @requires Routes
 * @requires CurrentOrganization
 *
 * @description
 *   Provides a $resource for system groups.
 */

angular.module('Katello.system-groups').factory('SystemGroups',
    ['$resource', 'Routes', 'CurrentOrganization', function($resource, Routes, CurrentOrganization) {
        var SystemGroups = $resource(Routes.api_organization_system_groups_path(CurrentOrganization) + '/:systemGroupId',
                            {systemGroupId: '@systemGroupId'});

        return SystemGroups;
}]);
/**
 * Copyright 2013 Red Hat, Inc.
 *
 * This software is licensed to you under the GNU General Public
 * License as published by the Free Software Foundation; either version
 * 2 of the License (GPLv2) or (at your option) any later version.
 * There is NO WARRANTY for this software, express or implied,
 * including the implied warranties of MERCHANTABILITY,
 * NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 * have received a copy of GPLv2 along with this software; if not, see
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
 */

/**
 * @ngdoc module
 * @name  Katello.factory:Routes
 *
 * @description
 *   Provides a wrapper factory around KT.routes.
 */

angular.module('Katello').factory('Routes', [function() {
    var Routes = KT.routes;

    return Routes;
}]);
/* Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/







;
/* Copyright 2013 Red Hat, Inc.

 This software is licensed to you under the GNU General Public
 License as published by the Free Software Foundation; either version
 2 of the License (GPLv2) or (at your option) any later version.
 There is NO WARRANTY for this software, express or implied,
 including the implied warranties of MERCHANTABILITY,
 NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
 have received a copy of GPLv2 along with this software; if not, see
 http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
*/





















;
